VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CToolTips"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'   ******************************************************************************
'   * Purpose:  Handles showing and hiding the tooltips
'   * File:         CToolTips.cls
'   ******************************************************************************
Option Explicit

'   ***** ShowWindow constants
Private Const SW_SHOWNOACTIVATE = 4
Private Const SW_HIDE = 0
'   *****
'   ***** API declarations
Private Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long
Private Declare Function ShowWindow Lib "user32" (ByVal hWnd As Long, ByVal nCmdShow As Long) As Long
Private Declare Function GetActiveWindow Lib "user32" () As Long
'   *****

Private m_bToolTipsActive As Boolean ' holds whether or not tooltips are on
Private m_oActiveObject As Object ' the last object the mouse was over (button, groupitem, etc)
Private m_lToolTipsDelay As Long ' the delay before the tooltips appear and how long the pause before hidden
Private m_bToolTips As Boolean ' whether to show tooltips or not
Private m_bMouseDown As Boolean ' flag that the mouse is down

Event TimerStatusChanged(bEnabled As Boolean) ' fires when to change the enabled status of the timer
Event TimerIntervalChanged(lInterval As Long) ' fires when the timer interval needs to be changed

Public Property Let ToolTips(bToolTips As Boolean) ' sets that the tooltips will be shown
    m_bToolTips = bToolTips
End Property
Public Property Get ToolTips() As Boolean ' Returns that the tooltips will be shown
    ToolTips = m_bToolTips
End Property
Public Property Let ToolTipsDelay(lDelay As Long) ' sets the tooltips delay
    If lDelay < 1 Or lDelay > 65535 Then ' can only be up to 65535, this is a VB timer limitation
        Err.Raise 380, , "ToolTipsDelay must be between 1 and 65535." ' raise an error
        Exit Property
    End If
    
    m_lToolTipsDelay = lDelay
    RaiseEvent TimerIntervalChanged(lDelay) ' fire the event when changed
    
End Property
Public Property Get ToolTipsDelay() As Long ' returns the tooltips delay
    ToolTipsDelay = m_lToolTipsDelay
End Property
Public Property Set ActiveObject(obj As Object) ' sets the active object
    Set m_oActiveObject = obj
End Property
Public Property Get ToolTipsActive() As Boolean ' returns True when the tooltips are active
    ToolTipsActive = m_bToolTipsActive
End Property
    
Private Sub Class_Initialize()
'   ***** initialize the variables
    m_lToolTipsDelay = 1000
    m_bToolTips = True
'   *****
End Sub

Private Sub Class_Terminate()
    Unload frmToolTip ' unload the tooltip form
End Sub

Public Sub ShowToolTip(Optional oTarget As Object = Nothing)
'   ***********************************************************************
'   * Params:   oTarget: Optional: the object the mouse is over
'   * Purpose:  Shows the tooltips.
'   ***********************************************************************
    Dim pt As POINTAPI
    Dim lRet As Long
    Dim lWidth As Long
    Dim lHeight As Long
    Static s_iActiveButtonIndex As Long ' holds the index for the activebutton (the button the mouse is over)
    Dim obj As Object
    Dim sTip As String
    
    If m_bMouseDown Then ' don't show tooltips if the mouse is down
        Exit Sub
    End If
    
    If Not m_bToolTips Then ' don't show tooltips when False
        Exit Sub
    End If
    
    If oTarget Is Nothing Then ' if the oTarget wasn't passed
        Set obj = m_oActiveObject ' use the active object of the object
    Else
        Set obj = oTarget ' use the target object for the object
    End If
        
    If obj Is Nothing Then ' if the object is nothing, don't show tooltips
        Exit Sub
    End If
    
    On Error Resume Next
    
    sTip = obj.ToolTipText ' get the tooltiptext
    If Err Then ' object doesn't have a tooltiptext property
        sTip = obj.Text ' use the text property
    End If
    
    If Not obj.Index = s_iActiveButtonIndex Then ' when object under the mouse is different than the index of the active button
        lRet = GetCursorPos(pt) ' get the position of the mouse
    
        If Not m_bToolTipsActive Then ' if tooltips aren't active
            Load frmToolTip ' load the form
        End If
        
        frmToolTip.ToolTip = sTip ' set the tooltip property of the form
        frmToolTip.Move (pt.x) * Screen.TwipsPerPixelX, (pt.y + 20) * Screen.TwipsPerPixelY ' move the tooltip near the mouse
    
    End If
    
    If Not m_bToolTipsActive Then ' if not active
        frmToolTip.ZOrder ' set the zorder so the tooltip is above the control
    End If

    If Trim$(sTip) = "" Then ' if the tooltip has not text, don't show it
        lRet = ShowWindow(frmToolTip.hWnd, SW_HIDE) ' hide the tooltip
    Else
        lRet = ShowWindow(frmToolTip.hWnd, SW_SHOWNOACTIVATE) ' show the tooltip, but don't activate the form
    End If
    
    s_iActiveButtonIndex = obj.Index ' store the index of the active button
    m_bToolTipsActive = True ' set tooltips to active
     
    ' reset the timer interval to 10000 (this is when the tooltip goes away when the mouse doesn't move
    RaiseEvent TimerIntervalChanged(10000)
    RaiseEvent TimerStatusChanged(True) ' set timer.enabled to true
    
End Sub
Public Sub HideToolTip()
'   ***********************************************************************
'   * Purpose:  Hides the tooltip
'   ***********************************************************************
    m_bToolTipsActive = False ' tooltips are no longer active
    ShowWindow frmToolTip.hWnd, SW_HIDE ' hide the window
    
    ' reset the the timer interval to the timer interval
    RaiseEvent TimerIntervalChanged(m_lToolTipsDelay)
    RaiseEvent TimerStatusChanged(False) ' disable the timer
    
    Set m_oActiveObject = Nothing ' set the active object to nothing
End Sub

Public Sub MouseDown()
'   ***********************************************************************
'   * Purpose: Called when the mouse is down and so the tooltips must be hidden
'   ***********************************************************************
    m_bMouseDown = True ' set the flag
    If m_bToolTipsActive Then ' if active
        HideToolTip ' hide
    End If
End Sub

Public Sub MouseMove(hWnd As Long, obj As Object)
'   ***********************************************************************
'   * Params:   hWnd: the hwnd of the form, etc.,  obj: the object the mouse is over
'   * Purpose:  Fired when the mouse moves. Determines whether to show a new tooltip or not
'   ***********************************************************************
    Dim bIsActiveWindow As Boolean
    
'    bIsActiveWindow = IsActiveWindow(hWnd)
'    If Not bIsActiveWindow Then
'        If m_bToolTipsActive Then
'            HideToolTip
'        End If
'        Exit Sub
'    End If
    
    If m_bMouseDown Then ' when mousedown flag is set (do nothing)
        
    ElseIf m_bToolTipsActive Then ' when tooltips are active
        ShowToolTip obj ' show the tooltip
    
    ElseIf m_oActiveObject Is obj Then ' if mouse is over the same object
        
    Else
        Set m_oActiveObject = obj ' set the active object
        RaiseEvent TimerStatusChanged(True) ' enable the timer
    End If
End Sub
Public Sub MouseUp()
'   ***********************************************************************
'   * Purpose: Called on MouseUp. Resets the mousedown flag.
'   ***********************************************************************
    m_bMouseDown = False
End Sub

Private Function IsActiveWindow(hWnd As Long) As Boolean
'   ***********************************************************************
'   * Params:   hwnd: hwnd of the form, etc. the control is hosted by.
'   * Purpose:  Checks to see if the active window is the same as the param.
'   *               If it is, shows the tooltips. This stops tooltips from appearing when
'   *               another window has focus.
'   ***********************************************************************
    Dim hWndActiveWindow As Long
    
    hWndActiveWindow = GetActiveWindow() ' returns the handle to the active window
    
     IsActiveWindow = (hWndActiveWindow = hWnd) ' returns if they are the same
     
End Function

Public Sub TimerExpired()
'   ***********************************************************************
'   * Purpose: Called from the Timer event, when the timer expires.
'   ***********************************************************************
    RaiseEvent TimerStatusChanged(False) ' disable the timer (don't want it kicking off again)

    If m_bToolTipsActive Then ' if active
        HideToolTip ' hide the tooltips
    Else
        ShowToolTip ' show the tooltips
    End If
End Sub
