VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CTransBlt"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'   ******************************************************************************
'   * Purpose:  Handles the drawing of images
'   * File:         CTransblt.cls
'   ******************************************************************************
Option Explicit

Private Const errInvalidPropertyValue = 380
'   ***** system color constants
Private Const COLOR_BTNFACE = 15
Private Const COLOR_BTNHIGHLIGHT = 20
Private Const COLOR_BTNSHADOW = 16
'   *****
'   ***** bitblt constants
Private Const NOTSRCERASE = &H1100A6
Private Const SRCERASE = &H440328
Private Const MERGEPAINT = &HBB0226
Private Const SRCAND = &H8800C6
Private Const SRCCOPY = &HCC0020
Private Const SRCINVERT = &H660046
Private Const SRCPAINT = &HEE0086
Private Const NOTSRCCOPY = &H330008
Private Const DST_BITMAP = &O4
'   *****
'   ***** drawstate constants
Private Const DSS_NORMAL = &H0
Private Const DSS_UNION = &H10       ' Gray string appearance
Private Const DSS_DISABLED = &H20
Private Const DSS_MONO = &H80
Private Const DSS_RIGHT = &H8000
'   *****
'   ***** the bitmap structure
Private Type BITMAP
        bmType As Long
        bmWidth As Long
        bmHeight As Long
        bmWidthBytes As Long
        bmPlanes As Integer
        bmBitsPixel As Integer
        bmBits As Long
End Type
'   *****
'   ***** API declarations
Private Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
Private Declare Function SetBkColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function CreateBitmap Lib "gdi32" (ByVal nWidth As Long, ByVal nHeight As Long, ByVal nPlanes As Long, ByVal nBitCount As Long, lpBits As Any) As Long
Private Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function GetBkColor Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function GetObj Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Private Declare Function GetPixel Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long) As Long
Private Declare Function GetSysColor Lib "user32" (ByVal nIndex As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function SetPixel Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long, ByVal crColor As Long) As Long
Private Declare Function SetTextColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function StretchBlt Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal nSrcWidth As Long, ByVal nSrcHeight As Long, ByVal dwRop As Long) As Long
Private Declare Function DrawState Lib "user32" Alias "DrawStateA" (ByVal hdc As Long, ByVal hBrush As Long, ByVal lpDrawStateProc As Long, ByVal lParam As Long, ByVal wParam As Long, ByVal n1 As Long, ByVal n2 As Long, ByVal n3 As Long, ByVal n4 As Long, ByVal un As Long) As Long
Private Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long
'   *****

Public Function CreateMask(ByVal hDCDest As Long, ByVal hBmpSource As Long, ByVal lMaskColor As OLE_COLOR) As Long
'   ***********************************************************************
'   * Params:   hDCDest: the dc of the destination, used for compatibility, hBmpSource: the handle to the source bitmap,
'   *               lMaskColor: the color to use for the mask
'   * Return:   the handle to the mask picture
'   * Purpose:  Creates a monochrome bitmap of the source and returns its handle. The mask bitmap is not deleted here,
'   *               it is deleted elsewhere
'   ***********************************************************************
    Dim lPixel As Long
    Dim x As Long, y As Long
    Dim lRet As Long
    Dim tBmp As BITMAP 'description of the source bitmap
    Dim hDCMask As Long
    Dim hDCMem As Long
    Dim hBmpOldMem As Long
    Dim hBmpMask As Long
    Dim hBmpOldMask As Long
    Dim lOldBackColor As Long
    
    On Error GoTo CreateMask_err
    
    If hBmpSource = 0 Or hDCDest = 0 Then 'make sure there is a source bitmap and dc for the destination
        Exit Function
    End If
    
    GetObj hBmpSource, Len(tBmp), tBmp ' get the properties of the source bitmap
    
    hDCMem = CreateCompatibleDC(hDCDest)    'Create memory DC
    hBmpOldMem = SelectObject(hDCMem, hBmpSource)     ' select the source bitmap into the memory dc
    
    ' create the b/w mask
    hDCMask = CreateCompatibleDC(hDCDest)     'Create DC to hold mask
    ' create a mono bitmap
    hBmpMask = CreateBitmap(tBmp.bmWidth, tBmp.bmHeight, 1, 1, Null)
    ' select the new mono bitmap
    hBmpOldMask = SelectObject(hDCMask, hBmpMask)

'   ***** loops thru the bitmap and replaces all pixels of the mask color with white, everything else is set to black
   For x = 0 To tBmp.bmWidth - 1
        For y = 0 To tBmp.bmHeight - 1
            lPixel = GetPixel(hDCMem, x, y)
            
            Select Case lPixel
                Case lMaskColor:
                    SetPixel hDCMask, x, y, vbWhite
                Case Else
                    SetPixel hDCMask, x, y, vbBlack
            End Select
            
        Next y
    Next x
    
    CreateMask = hBmpMask ' return the handle to the mask bitmap
    
    SelectObject hDCMem, hBmpOldMem ' reselect the bitmap into the memory dc
    SelectObject hDCMask, hBmpOldMask ' reselect the bitmap into the mask dc
    
'   ***** delete the dcs that were created
    DeleteDC hDCMem
    DeleteDC hDCMask
'   *****

    Exit Function
    
CreateMask_err:
    CreateMask = 0 ' return 0
End Function

Public Sub DrawTransparentPicture(hDCDest As Long, hBmpSource As Long, hBmpMask As Long, x As Long, y As Long, _
                                                    lWidth As Long, lHeight As Long, Optional vBackColor As Variant)
'   ***********************************************************************
'   * Params:   hDCDest: the dc to draw the image on, hBmpSource: the handle to the source bitmap, hBmpMask:
'   *               the handle to the mask bitmap, x: the left of the image, y: the top of the image, lWidth: the width the stetch
'   *               (or shrink) the image to, lHeight: the height to stretch (or shrink) the image to, vBackColor: the backcolor
'   *               of the dc.
'   * Purpose:  Draws the source bitmap by masking out the maskcolor. This is done by overlaying the source bitmap
'   *               on top of the mask bitmap on top of the source bitmap.
'   ***********************************************************************
    Dim lOldBackColor As OLE_COLOR ' the previous backcolor of the dc
    Dim lOldForeColor As OLE_COLOR ' the previous forecolor of the dc
    Dim hdcSource As Long ' the dc with the source bitmap selected into it
    Dim hDCMask As Long ' the dc with the mask bitmap selected into i
    Dim hBmpOldSource As Long ' the old source bitmap
    Dim hBmpOldMask As Long ' the old mask bitmap
    Dim hDCMem As Long ' a temporary dc
    Dim hBmpMem As Long ' the bitmap selected into the temporary dc
    Dim hBmpOldMem As Long ' the old bitmap from the temporary dc
    Dim tBmp As BITMAP ' the bitmap structure
    Dim r As RECT
    Dim lRet As Long
    Dim lTempBackColor As OLE_COLOR ' holds the backcolor
    
    If IsMissing(vBackColor) Then ' backcolor was not passed
        lTempBackColor = GetBkColor(hDCDest) ' use the backcolor of the destination dc
    ElseIf IsNull(vBackColor) Then ' the backcolor was passed as null
        lTempBackColor = GetBkColor(hDCDest) ' use the backcolor of the destination dc
    Else
        If Not IsNumeric(vBackColor) Then ' if not numeric, get out
            Exit Sub
        End If
        lTempBackColor = CLng(vBackColor) ' converts the backcolor to a long
    End If
    
    GetObj hBmpSource, Len(tBmp), tBmp ' gets the source bitmaps properties

    hDCMem = CreateCompatibleDC(hDCDest) ' create the temporary dc
    hBmpMem = CreateCompatibleBitmap(hDCDest, tBmp.bmWidth, tBmp.bmHeight) ' create a bitmap like in the destination
    hBmpOldMem = SelectObject(hDCMem, hBmpMem) ' select the bitmap into the temporary dc

    lOldBackColor = SetBkColor(hDCMem, lTempBackColor) ' set the backcolor of the temp dc

    If IsMissing(vBackColor) Then ' if not passed, blit from the destination dc to the temp dc
        BitBlt hDCMem, 0, 0, lWidth, lHeight, hDCDest, x, y, SRCCOPY
    ElseIf IsNull(vBackColor) Then ' if null, blit from the destination dc to the temp dc
        BitBlt hDCMem, 0, 0, lWidth, lHeight, hDCDest, x, y, SRCCOPY
    Else
        ' create a rect
        r.Left = 0
        r.Top = 0
        r.Bottom = r.Top + lHeight
        r.Right = r.Left + lWidth
    
        ' fill the rect with the backcolor for the temporary dc
        ExtTextOut hDCMem, 0, 0, ETO_CLIPPED Or ETO_OPAQUE, r, "", 0, 0
    End If

    lRet = SetBkColor(hDCMem, RGB(255, 255, 255)) ' set the backcolor of the temp dc to white
    lOldForeColor = SetTextColor(hDCMem, RGB(0, 0, 0)) ' set the forecolor of the temp dc to black
    
'   ***** create a dc compatible to the destination dc and select the source bitmap into it
    hdcSource = CreateCompatibleDC(hDCDest)
    hBmpOldSource = SelectObject(hdcSource, hBmpSource)
'   *****

'   ***** create a dc compatible to the destination dc and select the mask bitmap into it
    hDCMask = CreateCompatibleDC(hDCDest)
    hBmpOldMask = SelectObject(hDCMask, hBmpMask)
'   *****

    BitBlt hDCMem, 0, 0, lWidth, lHeight, hdcSource, 0, 0, SRCINVERT ' blit from source bitmap inverted to temp dc
    BitBlt hDCMem, 0, 0, lWidth, lHeight, hDCMask, 0, 0, SRCAND ' blit from mask bitmap anded to temp dc
    BitBlt hDCMem, 0, 0, lWidth, lHeight, hdcSource, 0, 0, SRCINVERT ' blit from source bitmap inverted to temp dc
    
    ' stretch the bitmap the correct size when blitting to the destination dc
    StretchBlt hDCDest, x, y, lWidth, lHeight, hDCMem, 0, 0, tBmp.bmWidth, tBmp.bmHeight, SRCCOPY

    SetBkColor hDCMem, lOldBackColor ' reset the backcolor of the temp dc
    SetTextColor hDCMem, lOldForeColor ' reset the forecolor of the temp dc

'   ***** reselect the old bitmaps back into their dc
    SelectObject hdcSource, hBmpOldSource
    SelectObject hDCMask, hBmpOldMask
    SelectObject hDCMem, hBmpOldMem
'   *****

    DeleteObject hBmpMem ' delete the memory bitmap
    
'   ***** delete the dcs
    DeleteDC hdcSource
    DeleteDC hDCMask
    DeleteDC hDCMem
'   *****

End Sub
Public Sub DrawNormalPicture(hDCDest As Long, hBmpSource As Long, hBmpMask As Long, x As Long, y As Long, _
                                                    lWidth As Long, lHeight As Long)
'   ***********************************************************************
'   * Params:   hDCDest: the dc to draw the image on, hBmpSource: the handle to the source bitmap, hBmpMask:
'   *               the handle to the mask bitmap, x: the left of the image, y: the top of the image, lWidth: the width the stetch
'   *               (or shrink) the image to, lHeight: the height to stretch (or shrink) the image to
'   * Purpose:  Draws the picture without any modifications
'   ***********************************************************************
    Dim hdcSource As Long
    Dim hBmpOldSource As Long
    Dim tBmp As BITMAP
    
    GetObj hBmpSource, Len(tBmp), tBmp ' get the properties of the source bitmap
    
    hdcSource = CreateCompatibleDC(hDCDest)
    hBmpOldSource = SelectObject(hdcSource, hBmpSource)
    
    StretchBlt hDCDest, x, y, lWidth, lHeight, hdcSource, 0, 0, tBmp.bmWidth, tBmp.bmHeight, SRCCOPY
'    BitBlt hDCDest, x, y, lWidth, lHeight, hdcSource, 0, 0, SRCCOPY

    SelectObject hdcSource, hBmpOldSource
    
    DeleteDC hdcSource
End Sub

Public Sub DrawDitheredBackground(hDCDest As Long, hBmpSource As Long, hBmpMask As Long, x As Long, y As Long, _
                                                    lWidth As Long, lHeight As Long, Optional vBackColor As Variant)
'   ***********************************************************************
'   * Params:   hDCDest: the dc to draw the image on, hBmpSource: the handle to the source bitmap, hBmpMask:
'   *               the handle to the mask bitmap, x: the left of the image, y: the top of the image, lWidth: the width the stetch
'   *               (or shrink) the image to, lHeight: the height to stretch (or shrink) the image to, vBackcolor: Optional: the backcolor
'   *               of the dc.
'   * Purpose:  Draws a dithered picture using either the backcolor or the bitmap from the dc.
'   ***********************************************************************
    Dim lOldBackColor As OLE_COLOR
    Dim lOldForeColor As OLE_COLOR
    Dim hDCMem As Long
    Dim hBmpMem As Long
    Dim hBmpOldMem As Long
    Dim tBmp As BITMAP
    Dim r As RECT
    Dim lRet As Long
    Dim lTempBackColor As OLE_COLOR
    Dim hBrush As Long
    Dim hOldBrush As Long
    Dim hBmpMem2 As Long
    Dim hBmpOldMem2 As Long
    Dim hDCMem2 As Long
    Dim lOldBackColor2 As OLE_COLOR
    Dim lOldForeColor2 As OLE_COLOR
    
    If IsMissing(vBackColor) Then
        lTempBackColor = GetBkColor(hDCDest)
    ElseIf IsNull(vBackColor) Then
        lTempBackColor = GetBkColor(hDCDest)
    Else
        If Not IsNumeric(vBackColor) Then
            Exit Sub
        End If
        lTempBackColor = CLng(vBackColor)
    End If
    
    GetObj hBmpSource, Len(tBmp), tBmp

    hDCMem = CreateCompatibleDC(hDCDest)
    hBmpMem = CreateCompatibleBitmap(hDCDest, lWidth, lHeight)
    hBmpOldMem = SelectObject(hDCMem, hBmpMem)
    
    hDCMem2 = CreateCompatibleDC(hDCDest)
    hBmpMem2 = CreateCompatibleBitmap(hDCDest, lWidth * 2, lHeight * 2)
    hBmpOldMem2 = SelectObject(hDCMem2, hBmpMem2)

    lOldBackColor = SetBkColor(hDCMem, lTempBackColor)
    lOldForeColor = SetTextColor(hDCMem, RGB(0, 0, 0))

    lOldBackColor2 = SetBkColor(hDCMem2, lTempBackColor)
    lOldForeColor2 = SetTextColor(hDCMem2, RGB(0, 0, 0))
    
    If IsMissing(vBackColor) Then
        BitBlt hDCMem, 0, 0, lWidth, lHeight, hDCDest, x, y, SRCCOPY
        BitBlt hDCMem2, 0, 0, lWidth, lHeight, hDCDest, x, y, SRCCOPY
    Else
        r.Left = 0
        r.Top = 0
        r.Bottom = r.Top + lHeight
        r.Right = r.Left + lWidth
    
        ExtTextOut hDCMem, 0, 0, ETO_CLIPPED Or ETO_OPAQUE, r, "", 0, 0
        ExtTextOut hDCMem2, 0, 0, ETO_CLIPPED Or ETO_OPAQUE, r, "", 0, 0
    End If
    
    SetBkColor hDCMem2, lOldBackColor2
    SetTextColor hDCMem2, lOldForeColor2
    
    SelectObject hDCMem2, hBmpOldMem2
    
    hBrush = CreateSolidBrush(GetSysColor(COLOR_BTNHIGHLIGHT))
    
    DrawState hDCMem, hBrush, 0, hBmpMem2, 0&, 0&, 0&, 0&, 0&, DST_BITMAP Or DSS_UNION
    
    BitBlt hDCDest, x, y, lWidth, lHeight, hDCMem, 0, 0, SRCCOPY

    SetBkColor hDCMem, lOldBackColor
    SetTextColor hDCMem, lOldForeColor

    SelectObject hDCMem, hBmpOldMem

    DeleteObject hBmpMem
    DeleteObject hBmpMem2
    DeleteObject hBrush
    
    DeleteDC hDCMem
    DeleteDC hDCMem2
End Sub

Public Sub DrawDisabledPicture(hDCDest As Long, hBmpSource As Long, hBmpMask As Long, x As Long, y As Long, _
                                                    lWidth As Long, lHeight As Long, Optional vBackColor As Variant)
'   ***********************************************************************
'   * Params:   hDCDest: the dc to draw the image on, hBmpSource: the handle to the source bitmap, hBmpMask:
'   *               the handle to the mask bitmap, x: the left of the image, y: the top of the image, lWidth: the width the stetch
'   *               (or shrink) the image to, lHeight: the height to stretch (or shrink) the image to
'   * Purpose:  Draws the source bitmap disabled and blits transparent.
'   ***********************************************************************
    Dim lOldBackColor As OLE_COLOR
    Dim lOldForeColor As OLE_COLOR
    Dim hDCMem As Long
    Dim hBmpMem As Long
    Dim hBmpOldMem As Long
    Dim tBmp As BITMAP
    Dim r As RECT
    Dim lRet As Long
    Dim lTempBackColor As OLE_COLOR
    Dim hBrush As Long
    Dim hOldBrush As Long
    Dim hBmpMem2 As Long
    Dim hBmpOldMem2 As Long
    Dim hDCMem2 As Long
    
    If IsMissing(vBackColor) Then
        lTempBackColor = GetBkColor(hDCDest)
    ElseIf IsNull(vBackColor) Then
        lTempBackColor = GetSysColor(COLOR_BTNSHADOW)
    Else
        If Not IsNumeric(vBackColor) Then
            Exit Sub
        End If
        lTempBackColor = CLng(vBackColor)
    End If
    
    GetObj hBmpSource, Len(tBmp), tBmp

    hDCMem = CreateCompatibleDC(hDCDest)
    hBmpMem = CreateCompatibleBitmap(hDCDest, lWidth, lHeight)
    hBmpOldMem = SelectObject(hDCMem, hBmpMem)
    
    hDCMem2 = CreateCompatibleDC(hDCDest)
    hBmpMem2 = CreateCompatibleBitmap(hDCDest, tBmp.bmWidth, tBmp.bmHeight)
    hBmpOldMem2 = SelectObject(hDCMem2, hBmpMem2)

    lOldBackColor = SetBkColor(hDCMem, lTempBackColor)
    lOldForeColor = SetTextColor(hDCMem, RGB(0, 0, 0))

    If IsMissing(vBackColor) Then
        BitBlt hDCMem, 0, 0, lWidth, lHeight, hDCDest, x, y, SRCCOPY
    Else
        r.Left = 0
        r.Top = 0
        r.Bottom = r.Top + lHeight
        r.Right = r.Left + lWidth
        
        ExtTextOut hDCMem, 0, 0, ETO_CLIPPED Or ETO_OPAQUE, r, "", 0, 0
    
    End If
    
    lRet = SetBkColor(hDCMem, RGB(255, 255, 255))
    
    DrawTransparentPicture hDCMem2, hBmpSource, hBmpMask, 0, 0, lWidth, lHeight, vbWhite
    
    SelectObject hDCMem2, hBmpOldMem2
    
    DrawState hDCMem, 0, 0, hBmpMem2, 0, 0, 0, 0, 0, DST_BITMAP Or DSS_DISABLED
    
    BitBlt hDCDest, x, y, lWidth, lHeight, hDCMem, 0, 0, SRCCOPY

    SetBkColor hDCMem, lOldBackColor
    SetTextColor hDCMem, lOldForeColor

    SelectObject hDCMem, hBmpOldMem

    DeleteObject hBmpMem
    DeleteObject hBmpMem2
    
    DeleteDC hDCMem
    DeleteDC hDCMem2
End Sub



