VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTGroup"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  This is the GTGroup object and is used in the Groups collection
'   * File:         Group.cls
'   ******************************************************************************
Option Explicit

'   ***** Alignment constants
Public Enum GTAlignmentConstants
    gtLeftAlign
    gtRightAlign
    gtCenter
End Enum
'   *****
'   ***** View constants
Public Enum GTViewConstants
    gtLargePicture
    gtSmallPicture
End Enum
'   *****

'   ***** property variables
Private m_eAlignment As GTAlignmentConstants
Private m_vGroupItemBackColor As Variant
Private m_vGroupItemForeColor As Variant
Private m_lIndex As Long
Private m_eView As GTViewConstants
Private m_bVisible As Boolean
Private m_vTag As Variant
Private m_sText As String
Private m_oGroups As GTGroups
Private m_vKey As Variant
Private WithEvents m_oGroupItems As GTGroupItems
Attribute m_oGroupItems.VB_VarHelpID = -1
Private m_lTop As Long
Private m_lLeft As Long
Private m_lWidth As Long
Private m_lHeight As Long
Private m_lScrollOffset As Long
'   *****
'   ***** Returns/sets the scrolloffset (the first visible groupitem)
Friend Property Let ScrollOffset(lOffset As Long)
    m_lScrollOffset = lOffset
End Property
Friend Property Get ScrollOffset() As Long
    ScrollOffset = m_lScrollOffset
End Property
'   *****

'   ***** Returns/sets the left property. For internal use only
Friend Property Let InternalLeft(lLeft As Long)
    m_lLeft = lLeft
End Property
Friend Property Get InternalLeft() As Long
Attribute InternalLeft.VB_MemberFlags = "40"
    InternalLeft = m_lLeft
End Property
'   *****

'   ***** returns the value of left. Read Only.
Public Property Get Left() As Long
Attribute Left.VB_Description = "Returns/sets the distance between the internal left edge of an object and the left edge of its container."
Attribute Left.VB_HelpID = 52
    Left = m_lLeft
End Property
'   *****

'   ***** Returns/sets the top property. For internal use only
Friend Property Let InternalTop(lTop As Long)
    m_lTop = lTop
End Property
Friend Property Get InternalTop() As Long
Attribute InternalTop.VB_MemberFlags = "40"
    InternalTop = m_lTop
End Property
'   *****

'   ***** returns the value of top. Read Only.
Public Property Get Top() As Long
Attribute Top.VB_Description = "Returns/sets the distance between the internal top edge of an object and the top edge of its container."
Attribute Top.VB_HelpID = 92
    Top = m_lTop
End Property
'   *****

'   ***** Returns/sets the width property. For internal use only
Friend Property Let InternalWidth(lWidth As Long)
    m_lWidth = lWidth
End Property
Friend Property Get InternalWidth() As Long
Attribute InternalWidth.VB_MemberFlags = "40"
    InternalWidth = m_lWidth
End Property
'   *****

'   ***** returns the value of width. Read Only.
Public Property Get Width() As Long
Attribute Width.VB_Description = "Returns/sets the width of an object."
Attribute Width.VB_HelpID = 98
    Width = m_lWidth
End Property
'   *****

'   ***** Returns/sets the height property. For internal use only
Friend Property Let InternalHeight(lHeight As Long)
    m_lHeight = lHeight
End Property
Friend Property Get InternalHeight() As Long
Attribute InternalHeight.VB_MemberFlags = "40"
    InternalHeight = m_lHeight
End Property
'   *****

'   ***** returns the value of height. Read Only.
Public Property Get Height() As Long
Attribute Height.VB_Description = "Returns/sets the height of an object."
Attribute Height.VB_HelpID = 40
    Height = m_lHeight
End Property
'   *****

'   ***** the GTGroupItems object
Public Property Get GroupItems() As GTGroupItems
Attribute GroupItems.VB_Description = "Returns a reference to a collection of GroupItem objects."
Attribute GroupItems.VB_HelpID = 37
    Set GroupItems = m_oGroupItems
End Property
'   *****

'   ***** Returns/sets the Alignment constant. The text in the column (header and listitem) will
'   ***** be align according to this property
Public Property Let Alignment(eAlignment As GTAlignmentConstants)
Attribute Alignment.VB_Description = " Returns/sets a value that determines where an object is displayed on a form."
Attribute Alignment.VB_HelpID = 9
    m_eAlignment = eAlignment
    m_oGroups.ChildChanged Me, "Alignment"
End Property
Public Property Get Alignment() As GTAlignmentConstants
    Alignment = m_eAlignment
End Property
'   *****

'   ***** Returns/sets the color behind the groupitems. This will be Null if not set.
Public Property Let GroupItemBackColor(vBackColor As Variant)
Attribute GroupItemBackColor.VB_Description = "Returns/sets the background color used to display text and graphics in an object."
Attribute GroupItemBackColor.VB_HelpID = 34
    m_vGroupItemBackColor = vBackColor
    m_oGroups.ChildChanged Me, "GroupItemBackColor"
End Property
Public Property Get GroupItemBackColor() As Variant
    GroupItemBackColor = m_vGroupItemBackColor
End Property
'   *****

'   ***** Returns/sets the forecolor for the groupitems. This will be Null if not set.
Public Property Let GroupItemForeColor(vForeColor As Variant)
Attribute GroupItemForeColor.VB_Description = "Returns/sets the background color used to display text and graphics in an object."
Attribute GroupItemForeColor.VB_HelpID = 25
    m_vGroupItemForeColor = vForeColor
    m_oGroups.ChildChanged Me, "GroupItemForeColor"
End Property
Public Property Get GroupItemForeColor() As Variant
    GroupItemForeColor = m_vGroupItemForeColor
End Property
'   *****

'   **** used to set the value of index (for internal use only)
Friend Property Let InternalIndex(lIndex As Long)
Attribute InternalIndex.VB_MemberFlags = "40"
    m_lIndex = lIndex
End Property
'   *****

'   ***** returns the the value of Index
Public Property Get Index() As Long
Attribute Index.VB_Description = "Returns/sets the index of an object in a collection. Read-only at run time."
Attribute Index.VB_HelpID = 46
    Index = m_lIndex
End Property
'   *****

'   ***** Returns/sets if images from the LargeImages or SmallImages
'   ***** collections will be shown. LargeImages are centered, SmallImages
'   ***** are left justified.
Public Property Let View(eView As GTViewConstants)
Attribute View.VB_Description = "Returns/sets the what type of images are drawn for each GroupItem object."
Attribute View.VB_HelpID = 95
    m_eView = eView
    m_lScrollOffset = 0 ' reset scrolloffset when changing views
    m_oGroups.ChildChanged Me, "View"
End Property
Public Property Get View() As GTViewConstants
    View = m_eView
End Property
'   *****

'   ***** Returns/sets whether the group will be visible or not
Public Property Let Visible(bVisible As Boolean)
Attribute Visible.VB_Description = "Returns/sets a value that determines whether an object is visible or hidden."
Attribute Visible.VB_HelpID = 96
    m_bVisible = bVisible
    m_oGroups.ChildChanged Me, "Visible"
End Property
Public Property Get Visible() As Boolean
    Visible = m_bVisible
End Property
'   *****

'   ***** Returns/Sets any extra info for the button
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    Set m_vTag = vTag
    m_oGroups.ChildChanged Me, "Tag"
End Property
'   *****
Public Property Let Tag(vTag As Variant)
'   ***** Ver 1.0a have to check if an object before checking the value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
    m_oGroups.ChildChanged Me, "Tag"
End Property
Public Property Get Tag() As Variant
'   ***** Ver 1.0a If an object, have to use Set
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
'   *****
End Property
'   *****
'   ***** Returns/sets the Text property. This is the text that appears in the group.
Public Property Let Text(sText As String)
    m_sText = sText
    m_oGroups.ChildChanged Me, "Text"
End Property
Public Property Get Text() As String
Attribute Text.VB_Description = "Returns/sets the text to be displayed in a control."
Attribute Text.VB_HelpID = 88
Attribute Text.VB_UserMemId = 0
    Text = m_sText
End Property
'   *****

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set Collection(oGroups As GTGroups)
    Set m_oGroups = oGroups
End Property
'   *****

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
    If m_oGroups.IsDuplicateKey(vKey) Then
        Err.Raise vbObjectError + 1051, , "Key already exists."
        Exit Property
    End If
    m_vKey = vKey
    m_oGroups.ChildChanged Me, "Key"
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

Private Sub Class_Initialize()
'   ****************************************************
'   * Purpose:  Initializes the properties associated with the object
'   ****************************************************
    Set m_oGroupItems = New GTGroupItems
    m_bVisible = True
    m_eView = gtLargePicture
    m_eAlignment = gtLeftAlign
    m_vGroupItemBackColor = Null
    m_vGroupItemForeColor = Null
End Sub

Private Sub Class_Terminate()
    Set m_oGroupItems = Nothing ' releases the objects
End Sub

Private Sub m_oGroupItems_Added(oGroupItem As GTGroupItem)
'   ************************************************************
'   * Param:    oGroupItem: the GTGroupItem object that was added
'   * Purpose:  Called when a GTGroupItem object is added. In turn calls the GroupItemAdded
'   *               proc of the GTGroups collection. Eventually an event is fired in the user control.
'   ************************************************************
    m_oGroups.GroupItemAdded Me, oGroupItem
End Sub

Private Sub m_oGroupItems_Changed(oGroupItem As GTGroupItem, sProperty As String)
'   ************************************************************
'   * Param:    oGroupItem: the GTGroupItem object that was changed, sProperty: the property
'   *               was changed
'   * Purpose:  Called when a GTGroupItem object is changed. In turn calls the GroupItemChanged
'   *               proc of the GTGroups collection. Eventually an event is fired in the user control.
'   ************************************************************
    m_oGroups.GroupItemChanged Me, oGroupItem, sProperty
End Sub

Private Sub m_oGroupItems_Removed(ByVal oGroupItem As GTGroupItem)
'   ************************************************************
'   * Param:    oGroupItem: the GTGroupItem object that was removed
'   * Purpose:  Called when a GTGroupItem object is removed. In turn calls the GroupItemRemoved
'   *               proc of the GTGroups collection. Eventually an event is fired in the user control.
'   ************************************************************
    m_oGroups.GroupItemRemoved Me, oGroupItem
End Sub

