VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTGroupItem"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  This is the GTGroupItem object and is used in the GroupItems collection
'   * File:         GroupItem.cls
'   ******************************************************************************
Option Explicit

'   ***** property variables
Private m_oGroupItems As GTGroupItems
Private m_lIndex As Long
Private m_vLargePic As Variant
Private m_vSmallPic As Variant
Private m_vTag As Variant
Private m_sText As String
Private m_bVisible As Boolean
Private m_lWidth As Long
Private m_lHeight As Long
Private m_vKey As Variant
Private m_bDown As Boolean

'   ***** Down is set when the mouse is down on a groupitem
Friend Property Let Down(bDown As Boolean)
    m_bDown = bDown
End Property
Friend Property Get Down() As Boolean
    Down = m_bDown
End Property
'   *****

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set Collection(oGroupItems As GTGroupItems)
    Set m_oGroupItems = oGroupItems
End Property
'   *****

'   **** used to set the value of index (for internal use only)
Friend Property Let InternalIndex(lIndex As Long)
    m_lIndex = lIndex
End Property
'   *****

'   ***** returns the the value of Index
Public Property Get Index() As Long
Attribute Index.VB_Description = "Returns/sets the index of an object in a collection. Read-only at run time."
Attribute Index.VB_HelpID = 46
    Index = m_lIndex
End Property
'   *****

'   ***** Returns/Sets the value of LargePicture. This is the index or key of an image in the LargeImages collection
Public Property Let LargePicture(vPic As Variant)
Attribute LargePicture.VB_Description = "Returns/sets the index or key of a GTListImage object to be used."
Attribute LargePicture.VB_HelpID = 51
    m_vLargePic = vPic
    m_oGroupItems.ChildChanged Me, "LargePicture"
End Property
Public Property Get LargePicture() As Variant
    LargePicture = m_vLargePic
End Property
'   *****

'   ***** Returns/Sets the value of SmallPicture. This is the index or key of an image in the SmallImages collection
Public Property Let SmallPicture(vPic As Variant)
Attribute SmallPicture.VB_Description = "Returns/sets the index or key of a GTListImage object to be used."
Attribute SmallPicture.VB_HelpID = 76
    m_vSmallPic = vPic
    m_oGroupItems.ChildChanged Me, "SmallPicture"
End Property
Public Property Get SmallPicture() As Variant
    SmallPicture = m_vSmallPic
End Property
'   *****

'   ***** Returns/Sets any extra info for the button
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    Set m_vTag = vTag
    m_oGroupItems.ChildChanged Me, "Tag"
End Property
'   *****
Public Property Let Tag(vTag As Variant)
'   ***** Ver 1.0a have to check if an object before checking the value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
    m_oGroupItems.ChildChanged Me, "Tag"
End Property
Public Property Get Tag() As Variant
'   ***** Ver 1.0a If an object, have to use Set
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
'   *****
End Property
'   *****

'   ***** Returns/sets the Text property. This is the text that appears with the groupitem.
Public Property Let Text(sText As String)
    m_sText = sText
    m_oGroupItems.ChildChanged Me, "Text"
End Property
Public Property Get Text() As String
Attribute Text.VB_Description = "Returns/sets the text to be displayed in a control."
Attribute Text.VB_HelpID = 88
Attribute Text.VB_UserMemId = 0
    Text = m_sText
End Property
'   *****

'   ***** Returns/sets whether the button will be visible or not
Public Property Let Visible(bVisible As Boolean)
Attribute Visible.VB_Description = "Returns/sets a value that determines whether an object is visible or hidden."
Attribute Visible.VB_HelpID = 96
    m_bVisible = bVisible
    m_oGroupItems.ChildChanged Me, "Visible"
End Property
Public Property Get Visible() As Boolean
    Visible = m_bVisible
End Property
'   *****

'   ***** Returns the left property. For internal use only
Friend Property Get InternalLeft() As Long
'    InternalLeft = m_oGroupItems.getGroupitemleft(Me)
End Property
'   *****

'   ***** Returns the top property. For internal use only
Friend Property Get InternalTop() As Long
'    InternalTop = m_oGroupItems.GetGroupItemTop(Me)
End Property
'   *****

'   ***** Returns/sets the Width property. For internal use only
Friend Property Let InternalWidth(lWidth As Long)
    m_lWidth = lWidth
End Property
Friend Property Get InternalWidth() As Long
    InternalWidth = m_lWidth
End Property
'   *****

'   ***** Returns/sets the height property. For internal use only
Friend Property Let InternalHeight(lHeight As Long)
    m_lHeight = lHeight
End Property
Friend Property Get InternalHeight() As Long
    InternalHeight = m_lHeight
End Property
'   *****

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
    If m_oGroupItems.IsDuplicateKey(vKey) Then
        Err.Raise vbObjectError + 1051, , "Key already exists."
        Exit Property
    End If
    m_vKey = vKey
    m_oGroupItems.ChildChanged Me, "Key"
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

Private Sub Class_Terminate()
    Set m_oGroupItems = Nothing ' releases the object
End Sub
