VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTGroupItems"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  The GTGroupItems collection object.
'   * File:         GroupItems.cls
'   ******************************************************************************
Option Explicit

Private m_colGroupItems As Collection ' holds the groupitems collection
Private m_bSkipChildChanges As Boolean ' flag to not fire the changed event
Private m_oGroups As GTGroups ' reference to the GTGroups object

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event Added(oGroupItem As GTGroupItem)
Attribute Added.VB_MemberFlags = "40"
Event Removed(ByVal oGroupItem As GTGroupItem)
Attribute Removed.VB_MemberFlags = "40"
Event Changed(oGroupItem As GTGroupItem, sProperty As String)
Attribute Changed.VB_MemberFlags = "40"
'   *****

'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
'   *****NewEnum must return the IUnknown interface of a collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colGroupItems.[_NewEnum]
End Function
'   *****

'   ***** Returns the number of objects in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = "Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colGroupItems.Count
End Property
'   *****

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index As Variant) As GTGroupItem
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    If lIndex < 1 Or lIndex > m_colGroupItems.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colGroupItems(lIndex)
    
End Property
'   *****

Private Sub Class_Initialize()
    Set m_colGroupItems = New Collection ' intialize the collection
End Sub

Private Sub Class_Terminate()
    Set m_colGroupItems = Nothing ' release the object
End Sub

Private Function GetIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colGroupItems.Count
        If m_colGroupItems.Key = vIndex Then
            GetIndexFromKey = i
            Exit Function
        End If
    Next
    
End Function

Private Sub ReIndexGroupItems()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colGroupItems.Count
        Me(i).InternalIndex = i
    Next
    
End Sub

Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the object to remove
'   * Purpose:  Removes an object from the collection.
'   ***********************************************************************
    Dim lIndex As Long
    Dim oTempGroupItem As GTGroupItem
    
    lIndex = ConvertIndex(Index)
    
    Set oTempGroupItem = Me(lIndex)
    
    m_colGroupItems.Remove lIndex
    
    ReIndexGroupItems
    
    RaiseEvent Removed(oTempGroupItem)
    
End Sub
Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all objects
'   ***********************************************************************
'   ***** Ver. 1.0a Need to reinitalize the collection
    Set m_colGroupItems = New Collection
'    Set m_colGroupItems = nothing
'   *****
    RaiseEvent Removed(Nothing)
End Sub
Friend Sub ChildChanged(oGroupItem As GTGroupItem, sProperty As String)
'   ***********************************************************************
'   * Params:   oGroupItem: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTGroupItem object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If m_bSkipChildChanges Then
        Exit Sub
    End If
    
    RaiseEvent Changed(oGroupItem, sProperty)
    
End Sub

Public Function Add(Optional Index, Optional Text, Optional Key, Optional LargePicture, Optional SmallPicture) As GTGroupItem
Attribute Add.VB_Description = "Adds a GroupItem object to a GroupItems collection and returns a reference to the created object."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Text: the text of the GroupItem,
'   *               Key: Optional: the key of the new object, LargePicture: Optional: the index of key to
'   *               GTListImage object in the LargeImages collection, SmallPicture: Optional: the index
'   *               of key to GTListImage object in the SmallImages collection
'   * Return:   GTGroupItem: the object if a groupitem was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTGroupItem object to the GroupItems collection.
'   ***********************************************************************
    Dim oGroupItem As GTGroupItem
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    m_bSkipChildChanges = True

    Set oGroupItem = New GTGroupItem
    
    Set oGroupItem.Collection = Me
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Text) Then
        oGroupItem.Text = Text
    End If
    
    If Not IsMissing(Key) Then
        oGroupItem.Key = Key
    End If
    
    If Not IsMissing(LargePicture) Then
        oGroupItem.LargePicture = LargePicture
    End If
    
    If Not IsMissing(SmallPicture) Then
        oGroupItem.SmallPicture = SmallPicture
    End If
    
'   ***** If Index is passed the object is inserted in the collection
    If IsMissing(Index) Then
        m_colGroupItems.Add oGroupItem
        oGroupItem.InternalIndex = m_colGroupItems.Count
    Else
        m_colGroupItems.Add oGroupItem, , lIndex
        ReIndexGroupItems
    End If
'   *****
    m_bSkipChildChanges = False
    
    RaiseEvent Added(oGroupItem)
    
    Set Add = oGroupItem
    
    Exit Function
    
Add_err:
    MsgBox Err.Description, vbInformation, App.Title
    m_bSkipChildChanges = False
End Function

Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oGroupItem As GTGroupItem
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oGroupItem In m_colGroupItems
        If StrComp(vTempKey, Trim(oGroupItem.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oGroupItem As GTGroupItem
    
    For Each oGroupItem In m_colGroupItems
        If oGroupItem.Key = vIndex Then
            GetObjectIndexFromKey = oGroupItem.Index
            Exit Function
        End If
    Next
    
End Function
Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function

