VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTGroups"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  The GTGroups collection object.
'   * File:         Groups.cls
'   ******************************************************************************
Option Explicit

Private m_colGroups As Collection ' holds the GTGroup objects
Private m_bSkipChildChanges As Boolean ' flag to not fire the changed event
Private m_iOffset As Integer ' the distance to start from the edge of the control
Private m_lScaleWidth As Long ' the scalewidth of the usercontrol - the offset
Private m_lScaleHeight As Long ' the scaleheight of the usercontrol - the offset
Private m_iScalePerPixel As Integer ' scale units per pixel
Private m_iTextHeight As Integer ' the height of the text
Private m_oSelectedGroup As GTGroup ' the selected group
Private m_iHiMetricPerPixel As Integer ' the number of himetric units per pixel
Private m_lGroupItemTextHeight As Long ' the height of the groupitem text
Private m_lLargePicHeight As Long ' the height of the large images
Private m_lLargePicWidth As Long ' the width of the large images
Private m_lSmallPicHeight As Long ' the height of the small images
Private m_lSmallPicWidth As Long ' the width of the small images

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event Added(oGroup As GTGroup)
Attribute Added.VB_MemberFlags = "40"
Event Removed(ByVal oGroup As GTGroup)
Attribute Removed.VB_MemberFlags = "40"
Event Changed(oGroup As GTGroup, sProperty As String)
Attribute Changed.VB_MemberFlags = "40"
Event GroupItemAdded(oGroup As GTGroup, oGroupItem As GTGroupItem)
Attribute GroupItemAdded.VB_MemberFlags = "40"
Event GroupItemRemoved(oGroup As GTGroup, ByVal oGroupItem As GTGroupItem)
Attribute GroupItemRemoved.VB_MemberFlags = "40"
Event GroupItemChanged(oGroup As GTGroup, oGroupItem As GTGroupItem, sProperty As String)
Attribute GroupItemChanged.VB_MemberFlags = "40"
'   *****

'   ***** Returns/sets the LargePicWidth. This is the width of the largeimages collection
Friend Property Let LargePicWidth(lWidth As Long)
    m_lLargePicWidth = lWidth
End Property
Friend Property Get LargePicWidth() As Long
    LargePicWidth = m_lLargePicWidth
End Property
'   *****

'   ***** Returns/sets the LargePicHeight. This is the height of the largeimages collection
Friend Property Let LargePicHeight(lHeight As Long)
    m_lLargePicHeight = lHeight
End Property
Friend Property Get LargePicHeight() As Long
    LargePicHeight = m_lLargePicHeight
End Property
'   *****

'   ***** Returns/sets the SmallPicWidth. This is the width of the smallimages collection
Friend Property Let SmallPicWidth(lWidth As Long)
    m_lSmallPicWidth = lWidth
End Property
Friend Property Get SmallPicWidth() As Long
    SmallPicWidth = m_lSmallPicWidth
End Property
'   *****

'   ***** Returns/sets the SmallPicHeight. This is the height of the smallimages collection
Friend Property Let SmallPicHeight(lHeight As Long)
    m_lSmallPicHeight = lHeight
End Property
Friend Property Get SmallPicHeight() As Long
    SmallPicHeight = m_lSmallPicHeight
End Property
'   *****

'   ***** Returns/sets the textheight of the groupitems
Friend Property Let GroupItemTextHeight(lHeight As Long)
    m_lGroupItemTextHeight = lHeight
End Property
Friend Property Get GroupItemTextHeight() As Long
    GroupItemTextHeight = m_lGroupItemTextHeight
End Property
'   *****

'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
'   *****NewEnum must return the IUnknown interface of a collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colGroups.[_NewEnum]
End Function
'   *****

'   ***** sets the number of himetric units in each pixel. Can't get this value in the class so it has to be passed
'   ***** Internal use only
Friend Property Let HiMetricPerPixel(iHiMetric As Integer)
    m_iHiMetricPerPixel = iHiMetric
End Property
'   *****

'   ***** Sets the selected group
Friend Property Set SelectedGroup(oGroup As GTGroup)
    Set m_oSelectedGroup = oGroup
End Property
'   *****

'   ***** Returns/sets the textheight of the groups
Friend Property Let TextHeight(iTextHeight As Integer)
    m_iTextHeight = iTextHeight
    ResetTop
End Property
Friend Property Get TextHeight() As Integer
    TextHeight = m_iTextHeight
End Property
'   *****

'   ***** Returns/sets the number of scale units per pixel.
'   ***** Can't get this value in the class, so it needs to passed in
Friend Property Let ScalePerPixel(iScalePerPixel As Integer)
    m_iScalePerPixel = iScalePerPixel
End Property
Friend Property Get ScalePerPixel() As Integer
    ScalePerPixel = m_iScalePerPixel
End Property
'   *****

'   ***** Returns/sets the distance from the edge of the control. Based on the appearance and borderstyle
Friend Property Let Offset(iOffset As Integer)
    m_iOffset = iOffset
End Property
Friend Property Get Offset() As Integer
    Offset = m_iOffset
End Property
'   *****

'   ***** Returns/sets the scalewidth. This is the scalewidth of the control - offset
Friend Property Let ScaleWidth(lScaleWidth As Long)
    m_lScaleWidth = lScaleWidth
End Property
Friend Property Get ScaleWidth() As Long
    ScaleWidth = m_lScaleWidth
End Property
'   *****

'   ***** Returns/sets the scaleheight. This is the scaleheight of the control - offset
Friend Property Let ScaleHeight(lScaleHeight As Long)
    m_lScaleHeight = lScaleHeight
    ResetTop
End Property
Friend Property Get ScaleHeight() As Long
    ScaleHeight = m_lScaleHeight
End Property
'   *****

'   ***** Returns the number of objects in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = "Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colGroups.Count
End Property
'   *****

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index As Variant) As GTGroup
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    If lIndex < 1 Or lIndex > m_colGroups.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colGroups(lIndex)
    
End Property
'   *****

Private Sub Class_Initialize()
    Set m_colGroups = New Collection ' initialize the object
End Sub

Private Sub Class_Terminate()
    Set m_colGroups = Nothing ' release the object
End Sub

Public Function Add(Optional Index, Optional Text, Optional Key, Optional Alignment, Optional View) As GTGroup
Attribute Add.VB_Description = "Adds a Group object to a Groups collection at both design time and run time."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Text: the text of the Group,
'   *               Key: Optional: the key of the new object, Alignment: Optional: whether the text is centered,
'   *               left or right justified, View: Optional: Large or small images
'   * Return:   GTGroup: the object if a Group was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTGroup object to the Groups collection.
'   ***********************************************************************
    Dim oGroup As GTGroup
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    m_bSkipChildChanges = True
    
    Set oGroup = New GTGroup
    
    Set oGroup.Collection = Me
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Text) Then
        oGroup.Text = Text
    End If
    
    If Not IsMissing(Key) Then
        oGroup.Key = Key
    End If
    
    If Not IsMissing(Alignment) Then
        oGroup.Alignment = Alignment
    End If
    
    If Not IsMissing(View) Then
        oGroup.View = View
    End If
    
'   ***** If Index is passed, the object is inserted into the collection
    If IsMissing(Index) Then
        m_colGroups.Add oGroup
        oGroup.InternalIndex = m_colGroups.Count
    Else
        m_colGroups.Add oGroup, , , lIndex
        ReIndexGroups
    End If
'   *****

    m_bSkipChildChanges = False
    
    RaiseEvent Added(oGroup)
    
    Set Add = oGroup
    
    Exit Function
    
Add_err:
    MsgBox Err.Description, vbInformation, App.Title
    m_bSkipChildChanges = False
End Function

Private Sub ReIndexGroups()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colGroups.Count
        Me(i).InternalIndex = i
    Next
    
End Sub

Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the object to remove
'   * Purpose:  Removes an object from the collection.
'   ***********************************************************************
    Dim lIndex As Long
    Dim oTempGroup As GTGroup
    
    lIndex = ConvertIndex(Index)
    
    Set oTempGroup = Me(lIndex)
    
    m_colGroups.Remove lIndex
    
    ReIndexGroups
    
    RaiseEvent Removed(oTempGroup)
    
End Sub
Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all objects
'   ***********************************************************************
'   ***** Ver. 1.0a Need to reinitalize the collection
    Set m_colGroups = New Collection
'    Set m_colGroups = Nothing
'   *****
    RaiseEvent Removed(Nothing)
End Sub
Friend Sub ChildChanged(oGroup As GTGroup, sProperty As String)
'   ***********************************************************************
'   * Params:   oGroup: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTGroup object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If m_bSkipChildChanges Then
        Exit Sub
    End If
    
    RaiseEvent Changed(oGroup, sProperty)
    
End Sub

Friend Sub GroupItemAdded(oGroup As GTGroup, oGroupItem As GTGroupItem)
'   ***********************************************************************
'   * Params:   oGroup: the parent object that was added, oGroupItem: the object
'   *               that was added
'   * Purpose:  Raises the GroupItemAdded event
'   ***********************************************************************
    RaiseEvent GroupItemAdded(oGroup, oGroupItem)
End Sub
Friend Sub GroupItemRemoved(oGroup As GTGroup, ByVal oGroupItem As GTGroupItem)
'   ***********************************************************************
'   * Params:   oGroup: the parent object that was removed, oGroupItem: the object
'   *               that was removed
'   * Purpose:  Raises the GroupItemRemoved event
'   ***********************************************************************
    RaiseEvent GroupItemRemoved(oGroup, oGroupItem)
End Sub
Friend Sub GroupItemChanged(oGroup As GTGroup, oGroupItem As GTGroupItem, sProperty As String)
'   ***********************************************************************
'   * Params:   oGroup: the parent of the object that was changed,
'   *               oGroupItem: the object that was changed, sProperty:
'   *               the property that was changed
'   * Purpose:  Raises the GroupItemChanged event
'   ***********************************************************************
    RaiseEvent GroupItemChanged(oGroup, oGroupItem, sProperty)
End Sub

Private Sub ResetTop()
'   ***********************************************************************
'   * Purpose:  Resets the top of the groups.
'   ***********************************************************************
    Dim i As Long
    Dim lTop As Long
    
    If m_colGroups.Count = 0 Then
        Exit Sub
    End If
   
   lTop = m_iOffset
    
    For i = 1 To m_oSelectedGroup.Index
        Me(i).InternalTop = lTop
        
        lTop = lTop + m_iTextHeight
    Next
    
    lTop = m_lScaleHeight - m_iTextHeight
    
    For i = m_colGroups.Count To m_oSelectedGroup.Index + 1
        Me(i).InternalTop = lTop
        
        lTop = lTop - m_iTextHeight
        
    Next
    
End Sub

Friend Sub Move(lScaleWidth As Long, lScaleHeight As Long)
'   ***********************************************************************
'   * Params:   lScaleWidth: the scalewidth of the control -offset,
'   *               lScaleHeight: the scaleheight of the control -offset
'   * Purpose:  Sets the variables at 1 shot instead of having to set
'   *               each property.
'   ***********************************************************************
    m_lScaleHeight = lScaleHeight
    m_lScaleWidth = lScaleWidth
End Sub

Friend Function GetGroupItemTop(oTargetGroupItem As GTGroupItem) As Long
'   ***********************************************************************
'   * Params:   oTargetGroupItem: the groupitem to find.
'   * Return:   Long: the top of the target groupitem
'   * Purpose:  Loops thru the groups to find the top of the selected group,
'   *               then loops thru the groupitems calculation the top until
'   *               the target groupitem is found.
'   ***********************************************************************
    Dim oGroup As GTGroup
    Dim lTop As Long
    Dim i As Long
    Dim lGroupItemsTop As Long
    Dim lGroupItemsBottom As Long
    Dim oPicture As StdPicture
    Dim oGroupItem As GTGroupItem
    
    If m_oSelectedGroup Is Nothing Then
        Exit Function
    End If
    
    lTop = m_iOffset
    
'   ***** Loops thru the groups, when the selected group
'   ***** is found, it exits.
    For Each oGroup In m_colGroups
        If oGroup.Visible Then
            lTop = lTop + m_iTextHeight
        End If
        If oGroup Is m_oSelectedGroup Then
            Exit For
        End If
    Next
'   *****

'   ***** Loops thru the groupitems adding the height of the groupitems
    For Each oGroupItem In oGroup.GroupItems
        If oGroupItem Is oTargetGroupItem Then
            GetGroupItemTop = lTop
            Exit Function
        End If
        
        If m_oSelectedGroup.View = gtLargePicture Then
            lTop = lTop + m_lLargePicHeight + m_lGroupItemTextHeight + GROUPITEMSPACING
        Else
            lTop = lTop + IIf(m_lSmallPicHeight > m_lGroupItemTextHeight, m_lSmallPicHeight, m_lGroupItemTextHeight) + GROUPITEMSPACING
        End If
        
    Next
'   *****

End Function
Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oGroup As GTGroup
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oGroup In m_colGroups
        If StrComp(vTempKey, Trim(oGroup.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oGroup As GTGroup
    
    For Each oGroup In m_colGroups
        If oGroup.Key = vIndex Then
            GetObjectIndexFromKey = oGroup.Index
            Exit Function
        End If
    Next
    
End Function
Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function

