VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTListImage"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  This is the GTListImage object and is used in the ListImages collection
'   * File:         ListImage.cls
'   ******************************************************************************
Option Explicit
'   ***** API Declaration
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
'   *****
'   ***** property variables
Private m_vTag As Variant
Private m_oPic As StdPicture
Private m_lIndex As Long
Private m_vKey As Variant
Private m_oListImages As GTListImages
Private m_hMaskPic As Long
'   *****

'   ***** Returns/sets the MaskPicture. This is a handle to the mask picture. Used for transparent blits.
'   ***** For Internal use only.
'   ***** This is deleted in the Terminate event using the DeleteObject API.
Friend Property Let MaskPicture(hMask As Long)
    m_hMaskPic = hMask
End Property
Friend Property Get MaskPicture() As Long
Attribute MaskPicture.VB_MemberFlags = "40"
    MaskPicture = m_hMaskPic
End Property
'   *****

'   ***** Returns/Sets any extra info for the object
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    Set m_vTag = vTag
    m_oListImages.ChildChanged Me, "Tag"
End Property
'   *****
Public Property Let Tag(vTag As Variant)
'   ***** Ver 1.0a Exit if the same value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
    m_oListImages.ChildChanged Me, "Tag"
End Property
Public Property Get Tag() As Variant
Attribute Tag.VB_Description = "Stores any extra data needed for your program."
Attribute Tag.VB_HelpID = 87
Attribute Tag.VB_UserMemId = 0
'   ***** Ver 1.0a Added to check for an object
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
End Property
'   *****

' ***** Returns/sets the picture for the object. The mask picture is created when setting the picture
Public Property Set Picture(oPic As StdPicture)
Attribute Picture.VB_Description = "Returns/sets the image Picture"
Attribute Picture.VB_HelpID = 13
    
    If Not oPic.Type = vbPicTypeBitmap Then
        Err.Raise vbObjectError + 1050, , "Picture must be of type bitmap."
        Exit Property
    End If
    
    Set m_oPic = oPic
    
    ' returns the handle of the mask bitmap
    m_hMaskPic = m_oListImages.CreateMaskPicture(oPic.Handle)
    
    m_oListImages.ChildChanged Me, "Picture"
End Property
Public Property Get Picture() As StdPicture
    Set Picture = m_oPic
End Property
'   *****

'   **** used to set the value of index (for internal use only)
Friend Property Let InternalIndex(lIndex As Long)
Attribute InternalIndex.VB_MemberFlags = "40"
    m_lIndex = lIndex
End Property
'   *****

'   ***** returns the the value of Index
Public Property Get Index() As Long
Attribute Index.VB_Description = "Returns/sets the index of an object in a collection. Read-only at run time."
Attribute Index.VB_HelpID = 46
    Index = m_lIndex
End Property
'   *****

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
    If m_oListImages.IsDuplicateKey(vKey) Then
        Err.Raise vbObjectError + 1051, , "Key already exists."
        Exit Property
    End If
    m_vKey = vKey
    m_oListImages.ChildChanged Me, "Key"
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set ListImages(oListImages As GTListImages)
    Set m_oListImages = oListImages
End Property
'   *****

Private Sub Class_Terminate()
    DeleteObject m_hMaskPic ' deletes the mask picture
    Set m_oListImages = Nothing ' releases the object
End Sub

Public Sub DrawPicture(hDCDest As Long, x As Single, y As Single, Optional Width, Optional Height, _
                                    Optional Style, Optional BackColor As Variant)
Attribute DrawPicture.VB_Description = "Draws the image to a given device context (DC) at a specified location using a specified style."
Attribute DrawPicture.VB_HelpID = 159
'   **********************************************************************
'   * Params:   hDCDest: the dc to draw on, X: the left point, Y: the top point, Width: Optional: the width to draw
'   *               the picture, Height: Optional: the height to draw the picture, Style: Optional: a GTDrawPictureConstant,
'   *               BackColor: Optional: the backcolor for the dc
'   * Purpose:  Draws the image on the dc based on the Style param
'   **********************************************************************
    Dim lWidth As Long
    Dim lHeight As Long
    Dim eStyle As GTDrawPictureConstants
    
    If IsMissing(Width) Then
        lWidth = 0
    Else
        lWidth = CLng(Width)
    End If
    
    If IsMissing(Height) Then
        lHeight = 0
    Else
        lHeight = CLng(Height)
    End If
    
    If IsMissing(Style) Then
        eStyle = gtTransparent
    Else
        eStyle = Style
    End If
    
    m_oListImages.DrawPicture hDCDest, CLng(x), CLng(y), m_oPic.Handle, m_hMaskPic, lWidth, lHeight, eStyle, BackColor
End Sub


