VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CCommonDialog"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'   ******************************************************************************
'   * Purpose:  Calls the open dialog thru the APIs.
'   * File:         CCommonDialog.cls
'   ******************************************************************************
Option Explicit

' ***** the type used for the open dialog
Private Type OPENFILENAME
        lStructSize As Long
        hwndOwner As Long
        hInstance As Long
        lpstrFilter As String
        lpstrCustomFilter As String
        nMaxCustFilter As Long
        nFilterIndex As Long
        lpstrFile As String
        nMaxFile As Long
        lpstrFileTitle As String
        nMaxFileTitle As Long
        lpstrInitialDir As String
        lpstrTitle As String
        lFlags As Long
        nFileOffset As Integer
        nFileExtension As Integer
        lpstrDefExt As String
        lCustData As Long
        lpfnHook As Long
        lpTemplateName As String
End Type
' *****
' ***** variables for the open dialog
Private m_hwndOwner As Long
Private m_lInstance As Long
Private m_sFilter As String
Private m_sCustomFilter As String
Private m_lMaxCustFilter As Long
Private m_lFilterIndex As Long
Private m_sFileName As String
Private m_lMaxFile As Long
Private m_sFileTitle As String
Private m_lMaxFileTitle As Long
Private m_sInitialDir As String
Private m_sTitle As String
Private m_lFlags As Long
Private m_iFileOffset As Integer
Private m_iFileExtension As Integer
Private m_sDefExt As String
Private m_lCustData As Long
Private m_lpfnHook As Long
Private m_sTemplateName As String
' *****

Private m_lError As Long ' holds the error
'API function called by ShowOpen method
Private Declare Function GetOpenFileName Lib "comdlg32.dll" Alias "GetOpenFileNameA" (pOpenfilename As OPENFILENAME) As Long
'API function to retrieve extended error information
Private Declare Function CommDlgExtendedError Lib "comdlg32.dll" () As Long

Public Property Get Error() As Long ' returns the error number
    Error = m_lError
End Property
Public Property Get hwndOwner() As Long ' returns the hwnd that dialog will appear above
    hwndOwner = m_hwndOwner
End Property

Public Property Let hwndOwner(ByVal NewValue As Long) ' sets the hwnd that dialog will appear above
    m_hwndOwner = NewValue
End Property

Public Property Get Instance() As Long ' returns the instance
    Instance = m_lInstance
End Property

Public Property Let Instance(ByVal NewValue As Long) ' sets the instance
    m_lInstance = NewValue
End Property

Public Property Get CustomFilter() As String ' returns the filter string
    CustomFilter = m_sCustomFilter
End Property

Public Property Let CustomFilter(ByVal NewValue As String) ' sets the filter string
    m_sCustomFilter = NewValue
End Property

Public Property Get MaxCustFilter() As Long ' returns the max size for the filter string
    MaxCustFilter = m_lMaxCustFilter
End Property

Public Property Let MaxCustFilter(ByVal NewValue As Long) ' sets the max size for the filter string
    m_lMaxCustFilter = NewValue
End Property

Public Property Get FilterIndex() As Long ' returns the filter index (the default filter)
    FilterIndex = m_lFilterIndex
End Property

Public Property Let FilterIndex(ByVal NewValue As Long) ' sets the filter index (the default filter)
    m_lFilterIndex = NewValue
End Property

Public Property Get FileName() As String ' returns the filename that was selected, includes the path
    FileName = m_sFileName
End Property

Public Property Let FileName(ByVal NewValue As String) ' sets the file that the dialog will start from;
                                                                                    ' generally includes a path
    m_sFileName = NewValue
End Property

Public Property Get MaxFile() As Long ' returns the maximum file size
    MaxFile = m_lMaxFile
End Property

Public Property Let MaxFile(ByVal NewValue As Long) ' sets the maximum file size
    m_lMaxFile = NewValue
End Property

Public Property Get FileTitle() As String ' returns the filename without the path
    FileTitle = m_sFileTitle
End Property

Public Property Let FileTitle(ByVal NewValue As String) ' sets the filename without the path
    m_sFileTitle = NewValue
End Property

Public Property Get MaxFileTitle() As Long ' returns the maximum title length
    MaxFileTitle = m_lMaxFileTitle
End Property

Public Property Let MaxFileTitle(ByVal NewValue As Long) ' sets the maximum title length
    m_lMaxFileTitle = NewValue
End Property

Public Property Get InitialDir() As String ' returns the default directory
    InitialDir = m_sInitialDir
End Property

Public Property Let InitialDir(ByVal NewValue As String) ' sets the default directory
    m_sInitialDir = NewValue
End Property

Public Property Get Title() As String ' returns the text in the dialog caption
    Title = m_sTitle
End Property

Public Property Let Title(ByVal NewValue As String) ' sets the text in the dialog caption
    m_sTitle = NewValue
End Property

Public Property Get Flags() As Long ' returns the value of the options
    Flags = m_lFlags
End Property

Public Property Let Flags(ByVal NewValue As Long) ' sets the value of the options
    m_lFlags = NewValue
End Property

Public Property Get FileOffset() As Integer
    FileOffset = m_iFileOffset
End Property

Public Property Let FileOffset(ByVal NewValue As Integer)
    m_iFileOffset = NewValue
End Property

Public Property Get FileExtension() As Integer ' returns the file extension without the name of path
    FileExtension = m_iFileExtension
End Property

Public Property Let FileExtension(ByVal NewValue As Integer) ' sets the file extension without the name of path
    m_iFileExtension = NewValue
End Property

Public Property Get DefExt() As String ' returns the default extension
    DefExt = m_sDefExt
End Property

Public Property Let DefExt(ByVal NewValue As String) ' sets the default extension;
                                                                                ' used when no extension is supplied in the filename
    m_sDefExt = NewValue
End Property

Public Property Get CustData() As Long
    CustData = m_lCustData
End Property

Public Property Let CustData(ByVal NewValue As Long)
    m_lCustData = NewValue
End Property

Public Property Get Hook() As Long
    Hook = m_lpfnHook
End Property

Public Property Let Hook(ByVal NewValue As Long)
    m_lpfnHook = NewValue
End Property

Public Property Get TemplateName() As String
    TemplateName = m_sTemplateName
End Property

Public Property Let TemplateName(ByVal NewValue As String)
    m_sTemplateName = NewValue
End Property

Public Property Get Filter() As String ' returns the filter string
    Filter = m_sFilter
End Property

Public Property Let Filter(ByVal NewValue As String) ' sets the filter string
    m_sFilter = NewValue
End Property

Public Function ShowOpen() As Long
'   *********************************************************
'   * Return : Returns the value returned by the GetOpenFilename API
'   * Purpose: This routine calls prepares and calls the GetOpenFilename API
'   *****************************************************************
    Dim vOpenFile As OPENFILENAME
    Dim sFileNameBuff As String
    Dim sFileTitleBuff As String
    Dim lRet As Long
    Dim lExtendedError As Long
    
    On Error GoTo ShowFileDialogError
      
'   ***** this initializes MaxFilesize to 256. If this is missing, no dialog will show.
    If m_lMaxFile = 0 Then
        m_lMaxFile = 255
    End If
'   *****

    m_lError = 0
    lRet = 0 'APIReturn property
    lExtendedError = 0  'ExtendedError property
      
'   ***** sets types props from the variables
    vOpenFile.lStructSize = Len(vOpenFile)
    vOpenFile.lpstrFilter = m_sFilter
    vOpenFile.nFilterIndex = m_lFilterIndex
'   *****

'   ***** sets the filename
    sFileNameBuff = Space(m_lMaxFile + 1)
    LSet sFileNameBuff = m_sFileName & vbNullChar
    vOpenFile.lpstrFile = sFileNameBuff
    vOpenFile.nMaxFile = m_lMaxFile
'   *****

'   ***** initialize the title
    sFileTitleBuff = Space(m_lMaxFileTitle + 1)
    LSet sFileTitleBuff = vbNullChar
    vOpenFile.lpstrFileTitle = sFileTitleBuff
'   *****
'   ***** initialize other properties
    vOpenFile.lpstrInitialDir = m_sInitialDir
    vOpenFile.lpstrTitle = m_sTitle
    vOpenFile.lFlags = m_lFlags
    vOpenFile.lpstrDefExt = m_sDefExt
    vOpenFile.hwndOwner = m_hwndOwner
    vOpenFile.hInstance = m_lInstance
    vOpenFile.lpstrCustomFilter = m_sCustomFilter
    vOpenFile.nMaxCustFilter = m_lMaxCustFilter
    vOpenFile.nMaxFileTitle = m_lMaxFileTitle
    vOpenFile.nFileOffset = m_iFileOffset
    vOpenFile.nFileExtension = m_iFileExtension
    vOpenFile.lCustData = m_lCustData
    vOpenFile.lpfnHook = m_lpfnHook
    vOpenFile.lpTemplateName = m_sTemplateName
'   *****

    lRet = GetOpenFileName(vOpenFile) ' call the API
      
    If lRet = 0 Then ' not successful; an error has occurred
        m_lError = CommDlgExtendedError() ' gets the error number by calling the API
    Else
        'sFileName gets part of vOpenFile.lpstrFile to the left of first Chr$(0)
        m_sFileName = Left$(vOpenFile.lpstrFile, InStr(vOpenFile.lpstrFile, vbNullChar) - 1)
    End If
    
    ShowOpen = lRet ' 0 if an error occurred, non zero if successful
    Exit Function
    
ShowFileDialogError:
    MsgBox Err.Description, vbInformation, App.Title ' show an error that occurred in this proc, not in the open dialog
    Exit Function
    Resume 0
End Function

Private Sub Class_Initialize()
    ReDim m_tFilters(0) ' initializes the array
End Sub

