VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTIcons"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private m_colIcons As Collection
Private m_bSkipChildChanges As Boolean

Event IconAdded(ByVal Icon As GTIcon)
Attribute IconAdded.VB_MemberFlags = "40"
Event IconRemoved(ByVal Icon As GTIcon)
Attribute IconRemoved.VB_MemberFlags = "40"
Event IconChanged(ByVal Icon As GTIcon, Property As String)
Attribute IconChanged.VB_MemberFlags = "40"

' NewEnum must return the IUnknown interface of a
' collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colIcons.[_NewEnum]
End Function

Public Function Add(Optional Index, Optional Key, Optional Icon) As GTIcon
Attribute Add.VB_Description = "Adds a ListItem object to a ListItems collection only at run time."
    Dim oIcon As GTIcon
    Dim lIndex As Long
    
    On Error GoTo Add_err
    
    m_bSkipChildChanges = True
    
    Set oIcon = New GTIcon
    
    Set oIcon.Collection = Me
        
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Key) Then
        oIcon.Key = Key
    End If
    
    If IsMissing(Icon) Then
        Set oIcon.Icon = Nothing
    Else
        If Icon.Type <> vbPicTypeIcon Then
            MsgBox "Invalid icon."
            Exit Function
        End If
        Set oIcon.Icon = Icon
    End If
    
    If IsMissing(Index) Then
        m_colIcons.Add oIcon
        oIcon.InternalIndex = m_colIcons.Count
    Else
        m_colIcons.Add oIcon, , , lIndex
        ReIndexIcons
    End If
        
    RaiseEvent IconAdded(oIcon)
    
    m_bSkipChildChanges = False
    
    Set Add = oIcon
    Set oIcon = Nothing
    
    Exit Function
Add_err:
    MsgBox Err.Description, vbInformation, App.Title
    Set oIcon = Nothing
    m_bSkipChildChanges = False
End Function

Public Property Get Item(Index) As GTIcon
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    If lIndex < 1 Or lIndex > m_colIcons.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colIcons(lIndex)
End Property

Private Sub ReIndexIcons()
    Dim i As Long
    
    For i = 1 To m_colIcons.Count
        Me(i).InternalIndex = i
    Next i
End Sub

Private Sub Class_Initialize()
    Set m_colIcons = New Collection
End Sub

Private Sub Class_Terminate()
    Set m_colIcons = Nothing
End Sub

Public Property Get Count() As Long
Attribute Count.VB_Description = "Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colIcons.Count
End Property

Friend Sub ChildChanged(oIcon As GTIcon, Property As String)
    'Since the control cannot get events from the Icon class, it calls this class's function
    'which in turn raises and event.
    If m_bSkipChildChanges Then
        Exit Sub
    End If
    
    RaiseEvent IconChanged(oIcon, Property)
End Sub

Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
    Dim lIndex As Long
    Dim oIcon As GTIcon
    
    lIndex = ConvertIndex(Index)
    
    Set oIcon = m_colIcons(lIndex)
    
    m_colIcons.Remove lIndex
    
    ReIndexIcons
    
    RaiseEvent IconRemoved(oIcon)
End Sub

Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   *******************************************************************************************
'   * Params : None
'   * Purpose: This destroys the current Icons collection and creates a new one.
'   *******************************************************************************************

    Set m_colIcons = Nothing
    Set m_colIcons = New Collection
    RaiseEvent IconRemoved(Nothing)
End Sub

Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
    Dim oIcon As GTIcon
    
    For Each oIcon In m_colIcons
        If oIcon.Key = vIndex Then
            GetObjectIndexFromKey = oIcon.Index
            Exit Function
        End If
    Next
    
End Function

Private Function ConvertIndex(Index As Variant) As Long
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function
Friend Function IsDuplicateKey(vKey As Variant) As Boolean
    Dim oIcon As GTIcon
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oIcon In m_colIcons
        If StrComp(vTempKey, Trim(oIcon.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Friend Sub ReadObjectProps(oPropBag As PropertyBag)
    Dim i As Long
    Dim j As Long
    Dim lCount As Long
    Dim oIcon As GTIcon
    
    lCount = oPropBag.ReadProperty("IconCount", 0)
    
    For i = 1 To lCount
        
        Set oIcon = Me.Add()
        
        oIcon.Tag = oPropBag.ReadProperty("IconTag" & i, "")
        oIcon.Key = oPropBag.ReadProperty("IconKey" & i, "")
        oIcon.Name = oPropBag.ReadProperty("IconName" & i, "")
        Set oIcon.Icon = oPropBag.ReadProperty("Icon#" & i, 0)
        
    Next
    
End Sub

Friend Sub WriteObjectProps(oPropBag As PropertyBag)
    Dim i As Long
    Dim j As Long
    Dim oIcon As GTIcon
    
    Call oPropBag.WriteProperty("IconCount", m_colIcons.Count, 0)
    
    For i = 1 To m_colIcons.Count
        
        Set oIcon = m_colIcons(i)
        
        Call oPropBag.WriteProperty("IconTag" & i, oIcon.Tag, "")
        Call oPropBag.WriteProperty("IconName" & i, oIcon.Name, "")
        Call oPropBag.WriteProperty("IconKey" & i, oIcon.Key, "")
        Call oPropBag.WriteProperty("Icon#" & i, oIcon.Icon, 0)
        
    Next
    
End Sub


