Attribute VB_Name = "mGlobals"
Option Explicit
'   ***** Ver 1.0a Needed for subclassing
Public Const CTL_PREFIX = "INTRAY"
'   *****

'Error messages
Public Const errInvalidPropertyValue = 380
Public Const errIconIsNothing = 380

'IconData Action Messages
Private Const NIF_MESSAGE = &H1
Private Const NIF_ICON = &H2
Private Const NIF_TIP = &H4

'IconData Action Messages
Private Const NIM_ADD = &H0
Private Const NIM_MODIFY = &H1
Private Const NIM_DELETE = &H2

'IconData Event Messages
Private Const WM_USER = &H400
Public Const msgTrayEvent = WM_USER + 501

Private Type NOTIFYICONDATA
        lSize As Long
        hWnd As Long
        lID As Long
        lFlags As Long
        lCallbackMessage As Long
        hIcon As Long
        szTip As String * 64
End Type

Private m_tNID As NOTIFYICONDATA
Public m_colIcons As GTIcons

'IconData APIs
Private Declare Function Shell_NotifyIcon Lib "shell32.dll" _
        Alias "Shell_NotifyIconA" (ByVal dwMessage As Long, _
        lpData As NOTIFYICONDATA) As Long

'Timer APIs
Private Declare Function SetTimer Lib "user32" (ByVal hWnd _
        As Long, ByVal nIDEvent As Long, ByVal uElapse As _
        Long, ByVal lpTimerFunc As Long) As Long
Private Declare Function KillTimer Lib "user32" (ByVal hWnd _
        As Long, ByVal nIDEvent As Long) As Long

'Timer ID
Private Const nIDTimerEvent = 11101

'Mouse APIs
Public Type POINTAPI
        x As Long
        y As Long
End Type


'Keyboard APIs
Public Const VK_CONTROL = &H11
Public Const VK_SHIFT = &H10
Public Const VK_MENU = &H12

'Mouse button APIs
Public Const VK_LBUTTON = &H1
Public Const VK_RBUTTON = &H2
Public Const VK_MBUTTON = &H4

Public Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long

'Keyboard API
Public Declare Function GetKeyState Lib "user32" (ByVal nVirtKey As Long) As Integer

' Icon API
Declare Function DrawIconEx Lib "user32" (ByVal hdc As Long, ByVal xLeft As Long, ByVal yTop As Long, ByVal hIcon As Long, ByVal cxWidth As Long, ByVal cyWidth As Long, ByVal istepIfAniCur As Long, ByVal hbrFlickerFreeDraw As Long, ByVal diFlags As Long) As Long

Private m_eAnimationStyle As GTAnimationStyleConstants '*** properties
Private m_bContinuous As Boolean
Private m_iInterval As Integer
Private m_bReverseAnimation As Boolean

Private m_sToolTipText As String

Private m_iIconIndex As Integer
Private m_iTimerIncrement As Integer

Private m_hWnd As Long
Private m_bIconInTray As Boolean
Private m_bTimerOn As Boolean

Property Get Animate() As Boolean
    Animate = m_bTimerOn Or m_bIconInTray
End Property

Property Get AnimationStyle() As GTAnimationStyleConstants
    AnimationStyle = m_eAnimationStyle
End Property

Property Let AnimationStyle(ByVal NewValue As GTAnimationStyleConstants)
    m_eAnimationStyle = NewValue
End Property

Property Get Continuous() As Boolean
    Continuous = m_bContinuous
End Property

Property Let Continuous(ByVal NewValue As Boolean)
    m_bContinuous = NewValue
End Property

Property Get Interval() As Integer
    Interval = m_iInterval
End Property

Property Let Interval(ByVal NewValue As Integer)
    m_iInterval = NewValue
    If m_bTimerOn Then
        KillTimer m_hWnd, nIDTimerEvent
        m_bTimerOn = False
    End If
End Property

Property Get ReverseAnimation() As Boolean
    ReverseAnimation = m_bReverseAnimation
End Property

Property Let ReverseAnimation(ByVal NewValue As Boolean)
    m_bReverseAnimation = NewValue
    If m_bReverseAnimation Then
        m_iIconIndex = m_colIcons.Count
        m_iTimerIncrement = -1
    Else
        m_iIconIndex = 1
        m_iTimerIncrement = 1
    End If
End Property

Property Get IconIndex() As Integer
    IconIndex = m_iIconIndex
End Property

Property Let IconIndex(ByVal NewValue As Integer)
    m_iIconIndex = NewValue
End Property

Property Get ToolTipText() As String
    ToolTipText = m_sToolTipText
End Property

Property Let ToolTipText(ByVal NewValue As String)
    m_sToolTipText = NewValue
End Property

Public Sub Terminate()
    Set m_colIcons = Nothing
End Sub

Public Sub AddIconToTray(hwndClient As Long, hIcon As Long, sToolTipText As String)
'   *******************************************************
'   * Params : hwndClient       The handle of the client window
'   *          hIcon            The handle of the icon to add to the tray
'   *          sToopTipText     The icon's ToolTipText
'   *          bRunTime         A variable which indicates runtime mode
'   * Purpose: This routine adds an icon to the system tray.
'   ******************************************************************

    Dim lRet As Long
    
    On Error Resume Next
    
    If Not m_bIconInTray Then
        ' Fill in m_tNID with appropriate values
        m_tNID.szTip = m_sToolTipText & Chr$(0)
        ' Flags: the message, icon, and tip are valid and should be
        ' paid attention to.
        m_tNID.lFlags = NIF_MESSAGE Or NIF_ICON Or NIF_TIP
        m_tNID.lID = 100
        m_tNID.lSize = Len(m_tNID)
        ' The window handle of our callback control
        m_tNID.hWnd = hwndClient
        ' The message CBWnd will receive when there's an icon event
        m_tNID.lCallbackMessage = msgTrayEvent
        m_tNID.hIcon = hIcon
        ' Make the callback window wait for our defined message
    
        ' Add the icon to the taskbar tray
        lRet = Shell_NotifyIcon(NIM_ADD, m_tNID)
        m_bIconInTray = True
    End If
    
End Sub

Public Sub RemoveIconFromTray()
'   ************************************************************************
'   * Params : None
'   * Purpose: Removes our icon from the system tray.
'   ****************************************************************************

    ' On remove, we only have to give enough information for Windows
    ' to locate the icon, then tell the system to delete it.
    Dim lRet As Long
    
    On Error GoTo RemoveIconFromTray_err
    
    If m_bIconInTray Then
        lRet = Shell_NotifyIcon(NIM_DELETE, m_tNID)
        m_bIconInTray = False
    End If
    
    Exit Sub
    
RemoveIconFromTray_err:
'    MsgBox Err.Description, vbInformation, App.Title
    '***
End Sub

Public Sub ChangeIconInTray(hIcon As Long)
'   ****************************************************************
'   * Params : hIcon        The new icon to put in the tray
'   * Purpose: Replaces the icon in the system tray with a new one.
'   ********************************************************************

    Dim lRet As Long
    
    On Error Resume Next
    ' Fill the data structure with necessary information, including
    ' the icon
    m_tNID.lFlags = NIF_ICON
    m_tNID.hIcon = hIcon

     ' Call the modify part of the function to change the elements indicated
     ' by the uFlags Type member above - in this case, just the icon.
    lRet = Shell_NotifyIcon(NIM_MODIFY, m_tNID)
    
End Sub

Public Sub ChangeTrayIconToolTipText()
'   ******************************************************
'   * Params : sToolTipText     New tool tip for system tray icon
'   * Purpose: Changes the system tray icon's tool tip
'   **************************************************************

    Dim lRet As Long
    
    On Error Resume Next
    ' The tip is going to be important
    m_tNID.lFlags = NIF_TIP
    
    ' This has to have a null character appended to it or the tip
    ' will appear as a string with as many trailing spaces as there
    ' are in the fixed-length string.
    m_tNID.szTip = m_sToolTipText & Chr$(0)

    lRet = Shell_NotifyIcon(NIM_MODIFY, m_tNID)
    
End Sub

Public Sub TimerProc(ByVal hWnd As Long, uMsg As Integer, _
        idEvent As Integer, dwTime As Long)
'   ************************************************************
'   * Params : hWnd         The handle of the window for which the message was sent
'   *          uMsg         The ID of the message that was sent
'   *          idEvent
'   *          dwTime
'   * Purpose: This procedure is called by the windows timer to indicate that the
'   *          requested amount of time has expired at which point we change the icon.
'   ************************************************************

    Dim i As GTIcon
    Dim iOldIndex As Integer
    
    On Error Resume Next
    
    If m_bTimerOn Then
        
        ChangeIconInTray m_colIcons(m_iIconIndex).Icon
        If Err Then
            Err = 0
            Set i = m_colIcons(m_iIconIndex).Icon
            If Err Then
                Err.Raise 380, , "Cannot access Icons(" & m_iIconIndex & ").Icon"
            Else
                Err.Raise 380, , "Cannot change System Tray Icon"
            End If
            Exit Sub
        End If
        On Error GoTo 0
        
        iOldIndex = m_iIconIndex
        m_iIconIndex = GetNextIndex()
        Do While m_bTimerOn
            If Not m_colIcons(m_iIconIndex).Icon Is Nothing Then
                Exit Do
            End If
            
            If iOldIndex = m_iIconIndex Then
                Exit Do
            End If
            m_iIconIndex = GetNextIndex()
        Loop
    End If
End Sub

Public Sub InitIcons()
    Set m_colIcons = New GTIcons
End Sub

Public Sub StartTimer(ByVal hWnd As Long)
    Dim iLoop As Integer
    Dim iOldIndex As Integer
    
    On Error GoTo StartTimer_err
    
    If Not m_bTimerOn Then
        If m_bReverseAnimation Then
            m_iIconIndex = m_colIcons.Count
            m_iTimerIncrement = -1
        Else
            m_iIconIndex = 1
            m_iTimerIncrement = 1
        End If
        AddIconToTray hWnd, m_colIcons(m_iIconIndex).Icon, m_sToolTipText
        If m_iInterval > 0 Then
            m_hWnd = hWnd
            SetTimer m_hWnd, nIDTimerEvent, m_iInterval, _
                    AddressOf TimerProc
            m_bTimerOn = True
        Else
            m_bTimerOn = False
        End If
    End If
    
    iOldIndex = m_iIconIndex
    m_iIconIndex = GetNextIndex()
    Do While m_bTimerOn And m_colIcons(m_iIconIndex).Icon Is Nothing
        If iOldIndex = m_iIconIndex Then
            Exit Do
        End If
        m_iIconIndex = GetNextIndex()
    Loop
    
    Exit Sub

StartTimer_err:
    '***
End Sub

Public Sub EndTimer()

    On Error GoTo EndTimer_err
    
    If m_bTimerOn Then
        KillTimer m_hWnd, nIDTimerEvent
        m_bTimerOn = False
    End If
    
    Exit Sub

EndTimer_err:
    '***
End Sub

Public Function GetNextIndex() As Long
    Dim Index As Integer
    
    If m_bTimerOn Then
        Index = m_iIconIndex + m_iTimerIncrement
        Select Case Index
            Case Is > m_colIcons.Count
                Select Case m_eAnimationStyle
                    Case gtCycle
                        If m_bContinuous Then
                            Index = 1
                        Else
                            EndTimer
                        End If
                    Case gtPingPong
                        If Not m_bContinuous And m_bReverseAnimation Then
                            EndTimer
                        Else
                            Index = m_colIcons.Count - 1
                            m_iTimerIncrement = -1
                        End If
                End Select
            Case 0
                Select Case m_eAnimationStyle
                    Case gtCycle
                        If m_bContinuous Then
                            Index = m_colIcons.Count '- 1
                            m_iTimerIncrement = -1
                        Else
                            EndTimer
                        End If
                    Case gtPingPong
                        If m_bContinuous Or m_bReverseAnimation Then
                            Index = 2
                            m_iTimerIncrement = 1
                        Else
                            EndTimer
                        End If
                End Select
        End Select
    Else
        Index = m_iIconIndex
    End If
    GetNextIndex = Index
End Function

