VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CSplitter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'   ******************************************************************************
'   * Purpose:  Handles drawing of the splitter.
'   * File:         CSplitter.cls
'   ******************************************************************************
Option Explicit

'   ***** Splitter orientation constants
Private Enum GTSplitterOrientationConstants
    gtHorizontal
    gtVertical
End Enum
'   *****

'   ***** APIs needed for this class
Private Declare Function ClientToScreen Lib "user32" (ByVal hWnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function GetDesktopWindow Lib "user32" () As Long
Private Declare Function GetWindowDC Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function InvertRect Lib "user32" (ByVal hdc As Long, lpRect As RECT) As Long
Private Declare Function ReleaseDC Lib "user32" (ByVal hWnd As Long, ByVal hdc As Long) As Long
'   *****

'   ***** Structures need for this class
Private Type RECT
        Left As Long
        Top As Long
        Right As Long
        Bottom As Long
End Type

Private Type POINTAPI
        x As Long
        y As Long
End Type
'   *****

'   ***** Variables for this class
Private m_lValue As Long
Private m_bMouseDown As Boolean
Private m_rectSplitter As RECT
Private m_eOrientation As Integer
Private m_lParentWidth As Long
Private m_lParentHeight As Long
Private m_lSplitterWidth As Long
Private m_lSplitterHeight As Long
Private m_lParentHwnd As Long
Private m_lParentLeft As Long
Private m_lParentTop As Long
'   *****

'   ***** Sets the left of the bounding rect
Friend Property Let ParentLeft(lLeft As Long)
    m_lParentLeft = lLeft
End Property
'   *****
'   ***** Sets the top of the bounding rect
Friend Property Let ParentTop(lTop As Long)
    m_lParentTop = lTop
End Property
'   *****
'   ***** Sets the hwnd of the window to bound the rect
Public Property Let ParentHwnd(hWnd As Long)
    m_lParentHwnd = hWnd
End Property
'   *****
'   ***** Returns/sets the Orientation (Horizontal or Vertical) to draw the splitter
Public Property Let Orientation(eOrientation As Integer)
    m_eOrientation = eOrientation
End Property
Public Property Get Orientation() As Integer
    Orientation = m_eOrientation
End Property
'   ***** Sets the Width of the bounding rect
Public Property Let ParentWidth(lWidth As Long)
    m_lParentWidth = lWidth
End Property
'   *****
'   ***** Sets the Height of the bounding rect
Public Property Let ParentHeight(lHeight As Long)
    m_lParentHeight = lHeight
End Property
'   *****
'   ***** Returns/sets the width of the splitter. Used when orientation is Horizontal
Public Property Let Width(lWidth As Long)
    m_lSplitterWidth = lWidth
End Property
Public Property Get Width() As Long
    Width = m_lSplitterWidth
End Property
'   *****
'   ***** Returns/sets the height of the splitter. Used when orientation is Vertical
Public Property Let Height(lHeight As Long)
    m_lSplitterHeight = lHeight
End Property
Public Property Get Height() As Long
    Height = m_lSplitterHeight
End Property
'   *****
'   ***** Returns/Sets the Value (where the splitter starts and ends)
Public Property Let Value(lValue As Long)
    m_lValue = lValue
End Property
Public Property Get Value() As Long
    Value = m_lValue
End Property
'   *****

Private Sub DrawSplitter(lLeft As Long, lTop As Long)
'   ***********************************************************************
'   * Params:   lLeft: the left where to draw the splitter, lTop: the top where
'   *               to draw the splitter.
'   * Purpose:  Converts the coordinates to screen coordinates and draws
'   *               the splitter.
'   ***********************************************************************
    Dim pt As POINTAPI
    Dim lRet As Long

    pt.x = lLeft
    pt.y = lTop
    
    lRet = ClientToScreen(m_lParentHwnd, pt)
    
    m_rectSplitter.Left = pt.x
    m_rectSplitter.Top = pt.y

    pt.x = lLeft + m_lSplitterWidth
    pt.y = lTop + m_lSplitterHeight
    
    lRet = ClientToScreen(m_lParentHwnd, pt)
    
    m_rectSplitter.Right = pt.x
    m_rectSplitter.Bottom = pt.y
    
    DrawSplitterRect

End Sub

Public Sub MouseDown(lLeft As Long, lTop As Long, x As Single, y As Single)
'   ***********************************************************************
'   * Params:   lLeft: usually 0, lTop: usually 0, X: the x value of the mouse,
'   *               Y: the y value of the mouse.
'   * Purpose:  Sets the starting pos for the splitter rect and draws the splitter.
'   ***********************************************************************
    Dim lTempLeft As Long
    Dim lTempTop As Long
    
    m_bMouseDown = True

    m_lValue = -1 ' resets the value
    
    If m_eOrientation = gtHorizontal Then
        lTempLeft = lLeft + x
        lTempTop = lTop
    Else
        lTempTop = lTop + y
        lTempLeft = lLeft
    End If
    
    DrawSplitter lTempLeft, lTempTop

End Sub

Public Sub MouseMove(lLeft As Long, lTop As Long, x As Single, y As Single)
'   ***********************************************************************
'   * Params:   lLeft: usually 0, lTop: usually 0, X: the x value of the mouse,
'   *               Y: the y value of the mouse.
'   * Purpose:  Called on MouseMove. Calculates the place to draw the
'   *               splitter rect and draws it.
'   ***********************************************************************
    Dim lTempLeft As Long
    Dim lTempTop As Long
    
    If m_bMouseDown Then

        If m_eOrientation = gtHorizontal Then
            lTempLeft = lLeft + x
            If lTempLeft < m_lParentLeft Then ' set to the parent left which is the left boundary
                lTempLeft = m_lParentLeft
            ElseIf lTempLeft > m_lParentWidth + m_lParentLeft Then ' when greater than the width
                ' reset the width. This width is the width of the section between parent left and parentwidth
                lTempLeft = (m_lParentWidth - m_lSplitterWidth) + m_lParentLeft
            End If
            
            lTempTop = lTop
            
            m_lValue = lTempLeft
            
        Else
            lTempTop = lTop + y
            If lTempTop < m_lParentTop Then ' set to the parent top which is the upper boundary
                lTempTop = m_lParentTop
            ElseIf lTempTop > m_lParentHeight + m_lParentTop Then ' when greater than the width
                ' reset the width. This width is the width of the section between parent top and parentheight
                lTempTop = (m_lParentHeight - m_lSplitterHeight) + m_lParentTop
            End If
            
            lTempLeft = lLeft
            
            m_lValue = lTempTop
            
        End If
    
        DrawSplitterRect ' resets the splitterrect
        DrawSplitter lTempLeft, lTempTop

    End If
End Sub

Public Function MouseUp() As Boolean
'   ***********************************************************************
'   * Return:   Boolean: True if successful, False if not.
'   * Purpose:  Called on MouseUp. Removes the splitter rect and resets
'   *               the variables.
'   ***********************************************************************
    Dim rc As Long
    
    If m_bMouseDown Then
        DrawSplitterRect
        m_bMouseDown = False
        MouseUp = True
    End If
    
End Function

Private Sub DrawSplitterRect()
'   ***********************************************************************
'   * Purpose:  Draws an inverted rect on the desktop. This simulates
'   *               the splitter.
'   ***********************************************************************
    Dim lRet As Long
    Dim hdcScreen As Long
    Dim hwndScreen As Long
        
    hwndScreen = GetDesktopWindow()
    hdcScreen = GetWindowDC(hwndScreen)
    
    lRet = InvertRect(hdcScreen, m_rectSplitter)
    
    If hwndScreen <> 0 Then
        ReleaseDC hwndScreen, hdcScreen
    End If

End Sub

Private Sub Class_Initialize()
    m_eOrientation = gtHorizontal ' initialize the variable
    m_lValue = -1 ' initialize to -1
End Sub

