VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTColumnHeader"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  This is the GTColumnHeader object and is used in the ColumnHeaders collection
'   * File:         ColumnHeaders.cls
'   ******************************************************************************
Option Explicit
'   ***** alignment constants
Public Enum GTAlignmentConstants
    gtLeftJustify
    gtRightJustify
    gtCenter
End Enum
'   *****

'   ***** property variables
Private m_sText As String
Private m_lLeft As Long
Private m_lWidth As Long
Private m_bVisible As Boolean
Private m_bEnabled As Boolean
Private m_lIndex As Long
Private m_vKey As Variant
Private m_lTop As Long
Private m_lHeight As Long
Private m_eAlignment As GTAlignmentConstants
Private m_oColumnHeaders As GTColumnHeaders
Private m_bResize As Boolean
Private m_vTag As Variant
'   *****

'   ***** Returns/Sets any extra info for the object
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    Set m_vTag = vTag
End Property
'   *****
Public Property Let Tag(vTag As Variant)
'   ***** Ver 1.0a have to check if an object before checking the value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
End Property
Public Property Get Tag() As Variant
'   ***** Ver 1.0a If an object, have to use Set
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
'   *****
End Property
'   *****

'   ***** Returns/sets the left property. For internal use only. Always in pixels
Friend Property Let InternalLeft(lLeft As Long)
    m_lLeft = lLeft
    m_oColumnHeaders.ChildPropertyChanged Me, "Left"
End Property
Friend Property Get InternalLeft() As Long
Attribute InternalLeft.VB_MemberFlags = "40"
    InternalLeft = m_oColumnHeaders.GetColumnHeaderLeft(m_lIndex)
End Property
'   *****

'   ***** Returns/sets the top property. For internal use only. Always in pixels
Friend Property Let InternalTop(lTop As Long)
    m_lTop = lTop
    m_oColumnHeaders.ChildPropertyChanged Me, "Top"
End Property
Friend Property Get InternalTop() As Long
Attribute InternalTop.VB_MemberFlags = "40"
    InternalTop = m_lTop
End Property
'   *****

'   ***** Returns/sets the width property. For internal use only. Always in pixels
Friend Property Let InternalWidth(lWidth As Long)
    m_lWidth = lWidth
    m_oColumnHeaders.ChildPropertyChanged Me, "Width"
End Property
Friend Property Get InternalWidth() As Long
Attribute InternalWidth.VB_MemberFlags = "40"
    InternalWidth = m_lWidth
End Property
'   *****

'   ***** Returns/sets the height property. For internal use only. Always in pixels
Friend Property Let InternalHeight(lHeight As Long)
    m_lHeight = lHeight
    m_oColumnHeaders.ChildPropertyChanged Me, "Height"
End Property
Friend Property Get InternalHeight() As Long
Attribute InternalHeight.VB_MemberFlags = "40"
    InternalHeight = m_lHeight
End Property
'   *****

'   ***** Returns/sets that the column is being resized. Internal use only
Friend Property Let Resize(bResize As Boolean)
    m_bResize = bResize
End Property
Friend Property Get Resize() As Boolean
    Resize = m_bResize
End Property
'   *****

'   ***** Returns/sets the Alignment constant. The text in the column (header and listitem) will
'   ***** be align according to this property
Public Property Let Alignment(eAlign As GTAlignmentConstants)
Attribute Alignment.VB_Description = "Returns/sets a value that determines where an object is displayed on a form."
Attribute Alignment.VB_HelpID = 9
    m_eAlignment = eAlign
    m_oColumnHeaders.ChildPropertyChanged Me, "Alignment"
End Property
Public Property Get Alignment() As GTAlignmentConstants
    Alignment = m_eAlignment
End Property
'   *****

'   ***** Returns/sets the Text property. This is the text that appears in the header.
Public Property Let Text(sText As String)
    m_sText = sText
    m_oColumnHeaders.ChildPropertyChanged Me, "Text"
End Property
Public Property Get Text() As String
Attribute Text.VB_Description = "Returns/sets the text to be displayed in a control."
Attribute Text.VB_HelpID = 88
Attribute Text.VB_UserMemId = 0
    Text = m_sText
End Property
'   *****

'   ***** Returns/sets the width of the column. In scale units of the container.
Public Property Let Width(lWidth As Long)
Attribute Width.VB_Description = "Returns/sets the width of an object."
Attribute Width.VB_HelpID = 98
    If lWidth <= 0 Then
        Err.Raise vbObjectError + 1052, , "Width must be greater than 0."
        Exit Property
    End If
    m_lWidth = lWidth / m_oColumnHeaders.ScalePerPixel
    
    If m_lWidth < 4 Then
        m_lWidth = 4
    End If
    
    m_oColumnHeaders.ChildPropertyChanged Me, "Width"
End Property
Public Property Get Width() As Long
    Width = m_lWidth * m_oColumnHeaders.ScalePerPixel
End Property
'   *****

'   ***** Returns/sets whether or not the header and listitems will be visible. Applies to this
'   ***** header only
Public Property Let Visible(bVisible As Boolean)
Attribute Visible.VB_Description = "Returns/sets a value that determines whether an object is visible or hidden."
Attribute Visible.VB_HelpID = 96
    m_bVisible = bVisible
    m_oColumnHeaders.ChildPropertyChanged Me, "Visible"
End Property
Public Property Get Visible() As Boolean
    Visible = m_bVisible
End Property
'   *****

'   ***** returns the the value of Index
Public Property Let Index(lIndex As Long)
Attribute Index.VB_Description = "Returns the position in the collecton. Read-only at runtime."
Attribute Index.VB_HelpID = 46
    If g_bAmbientUserMode And Not g_bInternalUse Then
        Err.Raise vbObjectError + 1050, , "Index is Read-only at runtime."
        Exit Property
    End If
    m_lIndex = lIndex
End Property
Public Property Get Index() As Long
    Index = m_lIndex
End Property
'   *****

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
    If m_oColumnHeaders.IsDuplicateKey(vKey) Then
        Err.Raise vbObjectError + 1051, , "Key already exists."
        Exit Property
    End If
    m_vKey = vKey
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

'   ***** Returns the left position of the column.
Public Property Get Left() As Long
Attribute Left.VB_Description = "Returns/sets the distance between the internal left edge of an object and the left edge of its container."
Attribute Left.VB_HelpID = 52
    Left = m_oColumnHeaders.GetColumnHeaderLeft(m_lIndex) * m_oColumnHeaders.ScalePerPixel
End Property
'   *****

Private Sub Class_Initialize()
'   **************************************************************
'   * Purpose: Initalize the property variables
'   **************************************************************
    m_lWidth = 100
    m_eAlignment = vbCenter
    m_bVisible = True
End Sub

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set ColumnHeaders(oColumnHeaders As GTColumnHeaders)
    Set m_oColumnHeaders = oColumnHeaders
End Property
'   *****

'   ***** Returns the number of the associated subitem in the listitem.
Public Property Get SubItemIndex() As Long
Attribute SubItemIndex.VB_Description = "Returns the index of the subitem associated with a ColumnHeader object .l."
Attribute SubItemIndex.VB_HelpID = 85
    SubItemIndex = m_lIndex - 1
End Property
'   *****

'   ***** Returns the height of the column.
Public Property Get Height() As Long
Attribute Height.VB_Description = "Returns/sets the height of an object."
Attribute Height.VB_HelpID = 40
    Height = m_oColumnHeaders.ColumnHeaderHeight * m_oColumnHeaders.ScalePerPixel
End Property
'   *****

'   ***** Returns the top position of the column.
Public Property Get Top() As Long
Attribute Top.VB_Description = "Returns/sets the distance between the internal top edge of an object and the top edge of its container."
Attribute Top.VB_HelpID = 92
    Top = m_oColumnHeaders.Offset * m_oColumnHeaders.ScalePerPixel
End Property
'   *****
