VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTColumnHeaders"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  The GTColumnHeaders collection object.
'   * File:         ColumnHeaders.cls
'   ******************************************************************************
Option Explicit

Private m_colHeaders As Collection ' holds the GTColumnHeader objects
Private m_lScrollOffset As Long ' holds the scrolloffset (how much has been scrolled)
Private m_bSkipChildChanges As Boolean ' flag to not fire the changed event
Private m_lColumnHeaderHeight As Long ' holds the height of the the columnheaders
Private m_iLeftOffset As Integer ' holds the left starting position after the border
Private m_iAppearance As Integer ' holds the value of Appearance
Private m_iBorderStyle As Integer ' holds the value of BorderStyle
Private m_iOffset As Integer ' holds the size of the border
Private m_iScalePerPixel As Integer ' holds the number of scaleunits per pixel

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event ColumnHeaderRemoved(ColumnHeader As GTColumnHeader)
Attribute ColumnHeaderRemoved.VB_MemberFlags = "40"
Event ColumnHeaderAdded(ColumnHeader As GTColumnHeader)
Attribute ColumnHeaderAdded.VB_MemberFlags = "40"
Event ColumnHeaderChanged(ColumnHeader As GTColumnHeader, sProperty As String)
Attribute ColumnHeaderChanged.VB_MemberFlags = "40"
'   *****

'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
'   *****NewEnum must return the IUnknown interface of a collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colHeaders.[_NewEnum]
End Function
'   *****
'   ***** Returns/sets the number of scale units per pixel. Has to be passed into the class.
Friend Property Let ScalePerPixel(iScalePerPixel As Integer)
    m_iScalePerPixel = iScalePerPixel
End Property
Friend Property Get ScalePerPixel() As Integer
    ScalePerPixel = m_iScalePerPixel
End Property
'   *****

'   ***** Returns/sets the appearance of the control. Has to be passed in to the class
Friend Property Let Appearance(iAppearance As Integer)
    m_iAppearance = iAppearance
End Property
Friend Property Get Appearance() As Integer
    Appearance = m_iAppearance
End Property
'   *****

'   ***** Returns/sets the borderstyle of the control. Has to be passed in to the class
Friend Property Let BorderStyle(iBorderStyle As Integer)
    m_iBorderStyle = iBorderStyle
End Property
Friend Property Get BorderStyle() As Integer
    BorderStyle = m_iBorderStyle
End Property
'   *****

'   ***** Returns/sets the offset (how far the left and top start from the top). Based on the border
Friend Property Let Offset(iOffset As Integer)
    m_iOffset = iOffset
End Property
Friend Property Get Offset() As Integer
    Offset = m_iOffset
End Property
'   *****

'   ***** Returns/sets the columnheaderheight. Based on the height of the font. Has to be passed in.
Friend Property Let ColumnHeaderHeight(lHeight As Long)
    m_lColumnHeaderHeight = lHeight
End Property
Friend Property Get ColumnHeaderHeight() As Long
    ColumnHeaderHeight = m_lColumnHeaderHeight
End Property
'   *****

'   ***** Returns/sets the ScrollOffset. This is the how much the control has been scrolled.
Friend Property Let ScrollOffset(lOffset As Long)
    m_lScrollOffset = lOffset
End Property
'   *****

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index As Variant) As GTColumnHeader
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    If lIndex < 1 Or lIndex > m_colHeaders.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colHeaders(lIndex)
    
End Property
'   *****

Public Function Add(Optional Index, Optional Key, Optional Text, Optional Width, Optional Alignment) As GTColumnHeader
Attribute Add.VB_Description = "Adds a ColumnHeader object to a ColumnHeaders collection at both design time and run time."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Key: Optional: the key of the new object,
'   *               Text: Optional: the text for the columnheader, Width: Optional: the width of the column,
'   *               Alignment: Optional: the alignment of the column
'   * Return:   GTColumnHeader: the object if a columnheader was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTColumnHeader object to the ColumnHeaders collection.
'   ***********************************************************************
    Dim oColumnHeader As GTColumnHeader
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    g_bInternalUse = True
    
    m_bSkipChildChanges = True
    
    Set oColumnHeader = New GTColumnHeader
    
    Set oColumnHeader.ColumnHeaders = Me
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If IsMissing(Text) Then
        oColumnHeader.Text = "Column " & m_colHeaders.Count + 1
    Else
        oColumnHeader.Text = Text
    End If
    
    If Not IsMissing(Key) Then
        oColumnHeader.Key = Key
    End If
    
    If Not IsMissing(Width) Then
        oColumnHeader.InternalWidth = Width / m_iScalePerPixel
    End If
    
    If Not IsMissing(Alignment) Then
        oColumnHeader.Alignment = Alignment
    End If
    
'   ***** Insert the columnheader when Index is passed
    If IsMissing(Index) Then
        m_colHeaders.Add oColumnHeader
    Else
        m_colHeaders.Add oColumnHeader, , , lIndex
    End If
'   *****

    ReIndexHeaders
    
    m_bSkipChildChanges = False
    
    RaiseEvent ColumnHeaderAdded(oColumnHeader)
    
    g_bInternalUse = False
    
    Set Add = oColumnHeader
    
    Exit Function
    
Add_err:
    MsgBox Err.Description, vbInformation, App.Title
    Set oColumnHeader = Nothing
    m_bSkipChildChanges = False
End Function

Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the object to remove
'   * Purpose:  Removes an object from the collection.
'   ***********************************************************************
    Dim lIndex As Long
    Dim oTempHeader As GTColumnHeader
    
    lIndex = ConvertIndex(Index)
    
    Set oTempHeader = Me(lIndex)
    
    m_colHeaders.Remove lIndex
    
    g_bInternalUse = True
    
    ReIndexHeaders
    
    g_bInternalUse = False

    RaiseEvent ColumnHeaderRemoved(oTempHeader)
    
End Sub
Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all objects
'   ***********************************************************************
    Set m_colHeaders = New Collection
    RaiseEvent ColumnHeaderRemoved(Nothing)
End Sub

'   ***** Returns the number of objects in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = "Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colHeaders.Count
End Property
'   *****

Private Sub ReIndexHeaders()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colHeaders.Count
        Me(i).Index = i
    Next
    
End Sub

Private Sub Class_Initialize()
    Set m_colHeaders = New Collection ' initialize the collection
End Sub

Private Sub Class_Terminate()
    Set m_colHeaders = Nothing ' release the object
End Sub
Public Function HitTest(x As Single, y As Single) As GTColumnHeader
Attribute HitTest.VB_Description = "Returns a reference to the GTColumnHeader object located at the coordinates of x and y. Used with drag and drop operations."
Attribute HitTest.VB_MemberFlags = "40"
'   *********************************************************************************
'   * Params:   X: the left of the point to find, Y: the top of the point to find.
'   * Return:   GTColumnHeader: the pane that was found at the X,Y point; Nothing if
'   *               no pane was found.
'   * Purpose:  Finds the columnheader at the X,Y point and returns it.Loops thru the
'   *               collection checking and adding the header widths.
'   *********************************************************************************
    Dim oColumnHeader As GTColumnHeader
    Dim lLeft As Long
    Dim lWidth As Long
    
    lLeft = m_iOffset
    
    For Each oColumnHeader In m_colHeaders
        
        If oColumnHeader.Visible Then
'   ***** Checks to see if the mouse is in the columnheader
            If x > lLeft - m_lScrollOffset And x < (lLeft + oColumnHeader.InternalWidth - 2 - m_lScrollOffset) Then
                If y > m_iOffset And y < m_lColumnHeaderHeight Then
                    Set HitTest = oColumnHeader
                    Exit Function
                End If
            End If
'   *****
            lLeft = lLeft + oColumnHeader.InternalWidth
        End If
        
    Next
    
End Function
Friend Function ColumnResize(x As Single, y As Single) As GTColumnHeader
'   *********************************************************************************
'   * Params:   X: the left of the point to find, Y: the top of the point to find.
'   * Return:   GTColumnHeader: the columnheader that was found at the X,Y point; Nothing if
'   *               no colum nheaderwas found.
'   * Purpose:  Finds the columnheader at the X,Y point and returns it.Loops thru the
'   *               collection checking and adding the header widths.This function
'   *               is looking to see if the mouse is between 2 columnheaders so
'   *               that 1 can be resized.
'   *********************************************************************************
    Dim oColumnHeader As GTColumnHeader
    Dim lLeft As Long
    
    lLeft = m_iOffset
    
    For Each oColumnHeader In m_colHeaders
        If oColumnHeader.Visible Then
        
'   ***** Checks to see if the mouse is between 2 columnheaders.
            If y > m_iOffset And y < m_lColumnHeaderHeight Then
                If x <= (lLeft + oColumnHeader.InternalWidth - m_lScrollOffset + 2) And x >= (lLeft + oColumnHeader.InternalWidth - m_lScrollOffset - 2) Then
                    Set ColumnResize = oColumnHeader
                    Exit Function
                End If
            End If
'   *****
            lLeft = lLeft + oColumnHeader.InternalWidth
        End If
    Next
    
End Function
Friend Sub ChildPropertyChanged(oColumnHeader As GTColumnHeader, sProperty As String)
'   ***********************************************************************
'   * Params:   oColumnHeader: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTColumnHeader object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If m_bSkipChildChanges Then
        Exit Sub
    End If
    
    RaiseEvent ColumnHeaderChanged(oColumnHeader, sProperty)
End Sub

Friend Function GetColumnHeaderLeft(lIndex As Long) As Long
'   *********************************************************************************
'   * Params:   lIndex: The header index to get the left for.
'   * Return:   Long: the left of the header.
'   * Purpose:  Loops thru the headers collection calculating the left for the target header.
'   *********************************************************************************
    Dim lLeft As Long
    Dim i As Long
    
    lLeft = m_iOffset ' starts with depending on the borderstyle and appearance
    
    For i = 1 To lIndex - 1
        If m_colHeaders(i).Visible Then
            lLeft = lLeft + Me(i).InternalWidth
        End If
    Next
    
    GetColumnHeaderLeft = lLeft
    
End Function

Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oColumnHeader As GTColumnHeader
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oColumnHeader In m_colHeaders
'   ***** Checks to see if the param and any other key match
        If StrComp(vTempKey, Trim(oColumnHeader.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
'   *****
    Next
    
    IsDuplicateKey = False
End Function

Friend Function GetMaxWidth() As Long
'   ***********************************************************************
'   * Return:   Long: The width of the widest column.
'   * Purpose:  Loops thru the collection and returns the widest column.
'   ***********************************************************************
    Dim oColumnHeader As GTColumnHeader
    Dim lLeft As Long
    
    lLeft = m_iOffset
    
    For Each oColumnHeader In m_colHeaders
        If oColumnHeader.Visible Then
            lLeft = lLeft + oColumnHeader.InternalWidth
        End If
    Next
    
    GetMaxWidth = lLeft
    
End Function
Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oHeader As GTColumnHeader
    
    For Each oHeader In m_colHeaders
        If oHeader.Key = vIndex Then
            GetObjectIndexFromKey = oHeader.Index
            Exit Function
        End If
    Next
    
End Function
Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function

