VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTMessages"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  The GTMessages collection object.
'   * File:         GTMessages.cls
'   ******************************************************************************
Option Explicit

Private m_colMessages As Collection ' holds the GTMessages objects
Private m_bSkipChildChanges As Boolean ' flag to not fire the changed event
Private m_lWindowIndex As Long ' the index of the window that is being subclassed.

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event Added(Message As GTMessage)
Attribute Added.VB_MemberFlags = "40"
Event Removed(Message As GTMessage)
Attribute Removed.VB_MemberFlags = "40"
Event Changed(Message As GTMessage, sProperty As String)
Attribute Changed.VB_MemberFlags = "40"
'   *****

'   ***** the index of the window that is being subclassed for the messages in the collection.
'   ***** needed only for reading and writing the messages at design  time for those set from proppages
Friend Property Let WindowIndex(lIndex As Long)
    m_lWindowIndex = lIndex
End Property
'   *****

'   ***** Returns the number of objects in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = " Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colMessages.Count
End Property
'   *****

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index) As GTMessage
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)

    If lIndex < 1 Or lIndex > m_colMessages.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colMessages(lIndex)
End Property
'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
'   *****NewEnum must return the IUnknown interface of a collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colMessages.[_NewEnum]
End Function
'   *****

Public Function Add(Optional Index, Optional Key, Optional Value, Optional Text, Optional ProcessDefaultProc, _
                            Optional ProcessType) As GTMessage
Attribute Add.VB_Description = "Adds a GTMessage object to a GTMessages collection and returns a reference to the created object."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Key: Optional: the key of the new object,
'   *               Value: Optional: the value of the message to intercept, ProcessDefaultProc: Optional:
'   *               whether or not fire the default proc, ProcessType: Optional: whether to fire the default proc before
'   *               or after the Message event
'   * Return:   GTMessage: the object if a message was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTMessage object to the Messages collection.
'   ***********************************************************************
    Dim oMessage As GTMessage
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    Set oMessage = New GTMessage
    
    m_bSkipChildChanges = True
    
    Set oMessage.Messages = Me
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Key) Then
        oMessage.Key = Key
    End If
    
    If Not IsMissing(Value) Then
        oMessage.Value = Value
    End If
    
    If Not IsMissing(Text) Then
        oMessage.Text = Text
    End If
    
    If Not IsMissing(ProcessDefaultProc) Then
        oMessage.ProcessDefaultProc = ProcessDefaultProc
    End If
    
    If Not IsMissing(ProcessType) Then
        oMessage.ProcessType = ProcessType
    End If
    
    If IsMissing(Index) Then
        oMessage.InternalIndex = m_colMessages.Count + 1
        m_colMessages.Add oMessage
    Else
        m_colMessages.Add oMessage, , , lIndex
        ReIndexObjects
    End If
    
    m_bSkipChildChanges = False
                            
    RaiseEvent Added(oMessage)
    
    Set Add = oMessage
    
    Exit Function
    
Add_err:
    m_bSkipChildChanges = False
    MsgBox Err.Description, vbInformation, App.Title
End Function
Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the object to remove
'   * Purpose:  Removes an object from the collection.
'   ***********************************************************************
    Dim oDeletedMessage As GTMessage
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    Set oDeletedMessage = Me(lIndex)
    
    m_colMessages.Remove lIndex
    
    ReIndexObjects
    
    RaiseEvent Removed(oDeletedMessage)
    
End Sub
Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oMessage As GTMessage
    
    For Each oMessage In m_colMessages
        If oMessage.Key = vIndex Then
            GetObjectIndexFromKey = oMessage.Index
            Exit Function
        End If
    Next
    
End Function

Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oMessage As GTMessage
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oMessage In m_colMessages
        If StrComp(vTempKey, Trim(oMessage.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all objects
'   ***********************************************************************
    Set m_colMessages = New Collection
    
    RaiseEvent Removed(Nothing)
End Sub

Friend Sub ChildChanged(oMessage As GTMessage, sProperty As String)
'   ***********************************************************************
'   * Params:   oMessage: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTMessage object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If Not m_bSkipChildChanges Then
        RaiseEvent Changed(oMessage, sProperty)
    End If
End Sub

Friend Sub ReadObjectProps(oPropBag As PropertyBag)
'   ***********************************************************************
'   * Params:   oPropBag: the property bag to read property values from
'   * Purpose:  Reads the properties from the property bag. Reads only
'   *               the properties for objects that were changed at design time
'   *               thru the property pages.
'   ***********************************************************************
    Dim i As Long
    Dim j As Long
    Dim lCount As Long
    Dim oMessage As GTMessage
    
    lCount = oPropBag.ReadProperty("NumMessages", 0)
    
    For i = 1 To lCount
        
        Set oMessage = Me.Add()
        
        oMessage.Tag = oPropBag.ReadProperty("Window" & m_lWindowIndex & "MessageTag" & i, "")
        oMessage.Key = oPropBag.ReadProperty("Window" & m_lWindowIndex & "MessageKey" & i, "")
        oMessage.Text = oPropBag.ReadProperty("Window" & m_lWindowIndex & "MessageText" & i, "")
        oMessage.Value = oPropBag.ReadProperty("Window" & m_lWindowIndex & "MessageValue" & i, 0)
        
    Next
    
End Sub

Private Sub ReIndexObjects()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colMessages.Count
        Me(i).InternalIndex = i
    Next
    
End Sub

Friend Sub WriteObjectProps(oPropBag As PropertyBag)
'   ***********************************************************************
'   * Params:   oPropBag: the property bag to write property values to
'   * Purpose:  Writes the properties to the property bag. Writes only
'   *               the properties for objects that were changed at design time
'   *               thru the property pages.
'   ***********************************************************************
    Dim i As Long
    Dim j As Long
    Dim oMessage As GTMessage
    
    Call oPropBag.WriteProperty("NumMessages", m_colMessages.Count, 0)
    
    For i = 1 To m_colMessages.Count
        
        Set oMessage = m_colMessages(i)
        
        Call oPropBag.WriteProperty("Window" & m_lWindowIndex & "MessageText" & i, oMessage.Text, "")
        Call oPropBag.WriteProperty("Window" & m_lWindowIndex & "MessageTag" & i, oMessage.Tag, "")
        Call oPropBag.WriteProperty("Window" & m_lWindowIndex & "MessageKey" & i, oMessage.Key, "")
        Call oPropBag.WriteProperty("Window" & m_lWindowIndex & "MessageValue" & i, oMessage.Value, 0)
        
    Next
    
End Sub

Private Sub Class_Initialize()
    Set m_colMessages = New Collection ' initialize the collection
End Sub

Private Sub Class_Terminate()
    Set m_colMessages = Nothing ' release the object
End Sub

Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function

Friend Function IsDuplicateMsg(lMsg As Long) As Boolean
'   ***********************************************************************
'   * Params:   lMsg: the msg to look for
'   * Return:   Boolean: True if the msg was found, False if not
'   * Purpose:  Loops thru the collection to see if the msg
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oMessage As GTMessage
    
    If lMsg = 0 Then
        IsDuplicateMsg = False
        Exit Function
    End If
    
    For Each oMessage In m_colMessages
        If lMsg = oMessage.Value Then
            IsDuplicateMsg = True
            Exit Function
        End If
    Next
    
    IsDuplicateMsg = False
End Function


