VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTListItem"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  This is the GTListItem object and is used in the ListItems collection
'   * File:         ListItems.cls
'   ******************************************************************************
Option Explicit
'   ***** property variables
Private aSubItems() As String ' array of items used for all columns but column 1
Private m_sText As String ' text for column 1 only
Private m_vKey As Variant
Private m_lIndex As Long
Private m_lLeft As Long
Private m_lTop As Long
Private m_lWidth As Long
Private m_lHeight As Long
Private m_oListItems As GTListItems
Private m_bSelected As Boolean
Private m_vTag As Variant
'   *****

'   ***** Returns/Sets any extra info for the object
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    Set m_vTag = vTag
End Property
'   *****
Public Property Let Tag(vTag As Variant)
'   ***** Ver 1.0a have to check if an object before checking the value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
End Property
Public Property Get Tag() As Variant
'   ***** Ver 1.0a If an object, have to use Set
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
'   *****
End Property
'   *****

Friend Sub ColumnAdded()
'   ***********************************************************************
'   * Purpose:  Adds another element to the SubItems array. Internal use only.
'   ***********************************************************************
    ReDim Preserve aSubItems(UBound(aSubItems) + 1) As String
End Sub
Friend Sub CreateSubItems(lNumSubItems As Long)
'   ***********************************************************************
'   * Params:   lNumSubItems: the number of columns.
'   * Purpose:  Dimensions the subitems array to 1 less than the
'   *               number of columns. Internal use only.
'   ***********************************************************************
    ReDim aSubItems(lNumSubItems - 1) As String
End Sub

'   ***** Returns/sets the value for the subitem that was passed in.
Public Property Let SubItems(lSubItem As Long, sText As String)
Attribute SubItems.VB_Description = "Returns/sets an array of strings representing the ListIitem's data."
Attribute SubItems.VB_HelpID = 86
    aSubItems(lSubItem - 1) = sText
    m_oListItems.ChildPropertyChanged Me, "SubItems"
End Property
Public Property Get SubItems(lSubItem As Long) As String
    SubItems = aSubItems(lSubItem - 1)
End Property
'   *****

Friend Sub ColumnRemoved()
'   ***********************************************************************
'   * Purpose:  Removes an element from the SubItems array. Internal use only.
'   ***********************************************************************
    ReDim Preserve aSubItems(UBound(aSubItems) - 1) As String
End Sub

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
    If m_oListItems.IsDuplicateKey(vKey) Then
        Err.Raise 50001, , "Key already exists."
        Exit Property
    End If
    m_vKey = vKey
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

'   ***** returns/sets the the value of Index
Public Property Let Index(lIndex As Long)
Attribute Index.VB_Description = "Returns the position in the collecton. Read-only at runtime."
Attribute Index.VB_HelpID = 46
    If g_bAmbientUserMode And Not g_bInternalUse Then
        Err.Raise 50000, , "Index is Read-only at runtime."
        Exit Property
    End If
    m_lIndex = lIndex
    m_oListItems.ChildPropertyChanged Me, "Index"
End Property
Public Property Get Index() As Long
    Index = m_lIndex
End Property
'   *****

'   ***** Returns/sets the Text property. This is the text that appears in the first column.
Public Property Let Text(sText As String)
    m_sText = sText
    m_oListItems.ChildPropertyChanged Me, "Text"
End Property
Public Property Get Text() As String
Attribute Text.VB_Description = "Returns/sets the text to be displayed in a control."
Attribute Text.VB_HelpID = 88
Attribute Text.VB_UserMemId = 0
    Text = m_sText
End Property
'   *****

Private Sub Class_Terminate()
    Erase aSubItems ' eliminates the array
End Sub

'   ***** Returns the left of the list item. first column only
Public Property Get Left() As Long
Attribute Left.VB_Description = "Returns/sets the distance between the internal left edge of an object and the left edge of its container."
Attribute Left.VB_HelpID = 52
    Left = m_oListItems.Offset * m_oListItems.ScalePerPixel
End Property
'   *****

'   ***** Returns the top of the list item.
Public Property Get Top() As Long
Attribute Top.VB_Description = "Returns/sets the distance between the internal top edge of an object and the top edge of its container."
Attribute Top.VB_HelpID = 92
    Top = m_oListItems.GetListItemTop(m_lIndex) * m_oListItems.ScalePerPixel
End Property
'   *****

'   ***** Returns the width of the list item. first column only
Public Property Get Width() As Long
Attribute Width.VB_Description = "Returns/sets the width of an object."
Attribute Width.VB_HelpID = 98
    Width = m_oListItems.ColumnHeaderWidth * m_oListItems.ScalePerPixel
End Property
'   *****

'   ***** Returns the height of the list item. Based on the textheight
Public Property Get Height() As Long
Attribute Height.VB_Description = "Returns/sets the height of an object."
Attribute Height.VB_HelpID = 40
    Height = m_oListItems.TextHeight * m_oListItems.ScalePerPixel
End Property
'   *****

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set ListItems(CListItems As GTListItems)
    Set m_oListItems = CListItems
End Property
'   *****

'   ***** Returns/sets whether or not the item is selected.
Public Property Let Selected(bSelected As Boolean)
Attribute Selected.VB_Description = "Returns/sets a value which determines if a ListItem or Node object is selected."
Attribute Selected.VB_HelpID = 69
    m_bSelected = bSelected
    m_oListItems.ChildPropertyChanged Me, "Selected"
End Property
Public Property Get Selected() As Boolean
    Selected = m_bSelected
End Property
'   *****
