VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTListItems"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

'   ***** selection constants
Private Enum GTSelectionTypeConstants
    GTListbox
    gtListView
End Enum
'   *****

Private m_colListItems As Collection ' holds the GTListItem objects
Private m_lNumHeaders As Long ' holds the number of columns
Private m_lScrollVOffset As Long ' holds the vertical scroll value
Private m_lScrollHOffset As Long ' holds the horizontal scroll value
Private m_eSelType As GTSelectionTypeConstants ' holds the selection type
Private m_bSkipChildChanges As Boolean '  flag to not fire the changed event
Private m_iTextHeight As Integer ' holds the text height
Private m_lColumnHeaderHeight As Long ' holds the height of the headerw
Private m_lColumnHeaderWidth As Long ' holds the width of the first column
Private m_iAppearance As Integer ' holds the appearance
Private m_iBorderStyle As Integer ' holds the borderstyle
Private m_iOffset As Integer ' holds the offset (based on the border)
Private m_iScalePerPixel As Integer ' holds the number of scaleunits per pixel
Private m_lScaleHeight As Long ' holds the scaleheight of the control

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event ListItemAdded(ListItem As GTListItem)
Attribute ListItemAdded.VB_MemberFlags = "40"
Event ListItemRemoved(ListItem As GTListItem)
Attribute ListItemRemoved.VB_MemberFlags = "40"
Event ListItemChanged(ListItem As GTListItem, sProperty As String)
Attribute ListItemChanged.VB_MemberFlags = "40"
'   *****

'   ***** Sets the scaleheight property. This is the scaleheight of the usercontrol - the border
Friend Property Let ScaleHeight(lScaleHeight As Long)
    m_lScaleHeight = lScaleHeight
End Property
'   *****

'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
'   *****NewEnum must return the IUnknown interface of a collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colListItems.[_NewEnum]
End Function
'   *****

'   ***** Returns/sets the number of scale units per pixel. Has to be passed into the class.
Friend Property Let ScalePerPixel(iScalePerPixel As Integer)
    m_iScalePerPixel = iScalePerPixel
End Property
Friend Property Get ScalePerPixel() As Integer
    ScalePerPixel = m_iScalePerPixel
End Property
'   *****

'   ***** Returns/sets the appearance of the control. Has to be passed in to the class
Friend Property Let Appearance(iAppearance As Integer)
    m_iAppearance = iAppearance
End Property
Friend Property Get Appearance() As Integer
    Appearance = m_iAppearance
End Property
'   *****

'   ***** Returns/sets the borderstyle of the control. Has to be passed in to the class
Friend Property Let BorderStyle(iBorderStyle As Integer)
    m_iBorderStyle = iBorderStyle
End Property
Friend Property Get BorderStyle() As Integer
    BorderStyle = m_iBorderStyle
End Property
'   *****

'   ***** Returns/sets the offset (how far the left and top start from the top). Based on the border
Friend Property Let Offset(iOffset As Integer)
    m_iOffset = iOffset
End Property
Friend Property Get Offset() As Integer
    Offset = m_iOffset
End Property
'   *****

'   ***** Returns/sets the width of the first column only.
Friend Property Let ColumnHeaderWidth(lWidth As Long)
    m_lColumnHeaderWidth = lWidth
End Property
Friend Property Get ColumnHeaderWidth() As Long
    ColumnHeaderWidth = m_lColumnHeaderWidth
End Property
'   *****

'   ***** Returns/sets the columnheaderheight. Based on the height of the font. Has to be passed in.
Friend Property Let ColumnHeaderHeight(lColumnHeaderHeight As Long)
    m_lColumnHeaderHeight = lColumnHeaderHeight
End Property
'   *****

'   ***** Returns/sets the TextHeight. Based on the height of the font. Has to be passed in.
Friend Property Let TextHeight(iTextHeight As Integer)
    m_iTextHeight = iTextHeight
End Property
Friend Property Get TextHeight() As Integer
    TextHeight = m_iTextHeight
End Property
'   *****

'   ***** Sets the ScrollHOffset. This is the how much the control
'   ***** has been scrolled horizontally
Friend Property Let ScrollHOffset(lOffset As Long)
    m_lScrollHOffset = lOffset
End Property
'   *****

'   ***** Returns/sets the ScrollVOffset. This is the how much the control
'   ***** has been scrolled vertically
Friend Property Let ScrollVOffset(lOffset As Long)
    m_lScrollVOffset = lOffset
End Property
'   *****

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index As Variant) As GTListItem
Attribute Item.VB_Description = "Returns a specific item of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    If lIndex < 1 Or lIndex > m_colListItems.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colListItems(lIndex)
    
End Property
'   *****

Public Function Add(Optional Index, Optional Key, Optional Text) As GTListItem
Attribute Add.VB_Description = "Adds a ListItem object to a ListItems collection only at run time."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Key: Optional: the key of the new object,
'   *               Text: Optional: the text for the first column
'   * Return:   GTListItem: the object if a listitem was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTListItems object to the ListItems collection.
'   ***********************************************************************
    Dim oListItem As GTListItem
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    Set oListItem = New GTListItem
    
    m_bSkipChildChanges = True
    
    Set oListItem.ListItems = Me
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Text) Then
        oListItem.Text = Text
    End If
    
    If Not IsMissing(Key) Then
        oListItem.Key = Key
    End If
    
    oListItem.CreateSubItems m_lNumHeaders
    
'   ***** Inserts the listitem when the Index param is passed
    If IsMissing(Index) Then
        m_colListItems.Add oListItem
    Else
        m_colListItems.Add oListItem, , lIndex
    End If
'   *****

    ReIndexListItems
    
    m_bSkipChildChanges = False
    
    RaiseEvent ListItemAdded(oListItem)
    
    Set Add = oListItem
    
    Exit Function
    
Add_err:
    MsgBox Err.Description, vbInformation, App.Title
    Set oListItem = Nothing
    m_bSkipChildChanges = False
End Function

Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the object to remove
'   * Purpose:  Removes an object from the collection.
'   ***********************************************************************
    Dim lIndex As Variant
    Dim oTempListItem As GTListItem
    
    lIndex = ConvertIndex(Index)
    
    Set oTempListItem = Me(lIndex)
    
    m_colListItems.Remove lIndex
    
    ReIndexListItems
    
    RaiseEvent ListItemRemoved(oTempListItem)
    
End Sub
Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all objects
'   ***********************************************************************
    Set m_colListItems = New Collection
    RaiseEvent ListItemRemoved(Nothing)
End Sub

'   ***** Returns the number of objects in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = "Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colListItems.Count
End Property
'   *****

Friend Sub ColumnRemoved()
'   ***********************************************************************
'   * Purpose:  Called when a columnheader has been removed.
'   *               Loops thru the listitems collection and calls the
'   *               columnremoved method on the object.
'   ***********************************************************************
    Dim oListItem As GTListItem
    
    For Each oListItem In m_colListItems
        oListItem.ColumnRemoved
    Next
    
    m_lNumHeaders = m_lNumHeaders - 1
    
End Sub

Private Sub ReIndexListItems()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    g_bInternalUse = True
    
    For i = 1 To m_colListItems.Count
        Me(i).Index = i
    Next
    
    g_bInternalUse = False

End Sub

Private Sub Class_Initialize()
    Set m_colListItems = New Collection ' initializes the listitems
End Sub

Private Sub Class_Terminate()
    Set m_colListItems = Nothing ' releases the object
End Sub

Friend Sub ChildPropertyChanged(oListItem As GTListItem, sProperty As String)
'   ***********************************************************************
'   * Params:   oListItem: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTListItem object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If m_bSkipChildChanges Then
        Exit Sub
    End If
    
    RaiseEvent ListItemChanged(oListItem, sProperty)
End Sub
Public Function HitTest(x As Single, y As Single) As GTListItem
Attribute HitTest.VB_Description = "Returns a reference to the GTListItem object object located at the coordinates of x and y. Used with drag and drop operations."
Attribute HitTest.VB_MemberFlags = "40"
'   *********************************************************************************
'   * Params:   X: the left of the point to find, Y: the top of the point to find.
'   * Return:   GTListItem: the listitem that was found at the X,Y point; Nothing if
'   *               no listitem was found.
'   * Purpose:  Finds the listitem at the X,Y point and returns it.Loops thru the
'   *               collection checking and adding the listitem heights.
'   *********************************************************************************
    Dim oListItem As GTListItem
    Dim lTop As Long
    Dim lCurrentY As Long
    Dim bFirstRow As Boolean
    
    lTop = m_lColumnHeaderHeight + m_iOffset
    bFirstRow = True
    
    For Each oListItem In m_colListItems
        
        If lCurrentY > m_lScaleHeight - m_iOffset Then
            Exit For
        End If
        
'   ***** Loops thru checking for the listitem under the mouse. Have to offset for
'   ***** scrolling. Also, the top visible row always starts at the same point, so
'   ***** that is why there are ltop and lcurrenty variables.
        If lTop - m_lScrollVOffset >= m_lColumnHeaderHeight + m_iOffset Then
            
            If bFirstRow Then
                bFirstRow = False
                lCurrentY = m_lColumnHeaderHeight + m_iOffset
            Else
                lCurrentY = lCurrentY + m_iTextHeight
            End If
            
            If m_eSelType = GTListbox Then
                If y > lCurrentY And y <= (lCurrentY + m_iTextHeight) Then
                    Set HitTest = oListItem
                    Exit Function
                End If
            
            Else
    
                If x > (0 - m_lScrollHOffset) And x < (m_lColumnHeaderWidth - m_lScrollHOffset) Then
                    If y > lTop - m_lScrollVOffset And y <= (lTop + m_iTextHeight - m_lScrollVOffset) Then
                        Set HitTest = oListItem
                        Exit Function
                    End If
                End If
            End If
        End If
'   *****
        lTop = lTop + m_iTextHeight
        
    Next
    
End Function

'   ***** Returns/sets the SelectionType. Has to be passed in.
Friend Property Let SelectionType(eSelType As GTSelectionTypeConstants)
    m_eSelType = eSelType
End Property
Friend Property Get SelectionType() As GTSelectionTypeConstants
    SelectionType = m_eSelType
End Property
'   *****

Friend Sub ColumnAdded()
'   ***********************************************************************
'   * Purpose:  Called when a columnheader has been added.
'   *               Loops thru the listitems collection and calls the
'   *               columnadded method on the object.
'   ***********************************************************************
    Dim oListItem As GTListItem
    
    For Each oListItem In m_colListItems
        oListItem.ColumnAdded
    Next
    
    m_lNumHeaders = m_lNumHeaders + 1
End Sub
Friend Property Set ListItems(colListItems As Collection)
'   ***********************************************************************
'   * Param:    colListItems: the sorted listitems collection
'   * Purpose:  Sets the sorted listitems collection to the listitems
'   *               collection of GTListItems. This is used when sorting.
'   ***********************************************************************
    Set m_colListItems = colListItems
    
    m_bSkipChildChanges = True
    ReIndexListItems
    m_bSkipChildChanges = False
    
End Property
'   ***** Returns the listitems collection
Friend Property Get ListItems() As Collection
    Set ListItems = m_colListItems
End Property
'   *****

Friend Sub ResetSelected()
'   ***********************************************************************
'   * Purpose:  Unselects the listitems that are selected.
'   ***********************************************************************
    Dim oListItem As GTListItem
    
    m_bSkipChildChanges = True
    For Each oListItem In m_colListItems
        oListItem.Selected = False
    Next
    m_bSkipChildChanges = False
    
End Sub
Friend Sub SelectItems(oSelectedItem As GTListItem, oLastSelected As GTListItem)
'   ***********************************************************************
'   * Params:   oSelectedItem: the currently selected listitem, oLastSelected:
'   *               the last selected listitem.
'   * Purpose:  Loops thru the listitems collection from the first param
'   *               to the second param and selects all listitems along the way.
'   ***********************************************************************
    Dim iStep As Integer
    Dim i As Long
    
    If oSelectedItem.Index > oLastSelected.Index Then
        iStep = 1
    Else
        iStep = -1
    End If
    
    For i = oLastSelected.Index To oSelectedItem.Index Step iStep
        m_colListItems(i).Selected = True
    Next
    
End Sub

Friend Function GetListItemTop(lIndex As Long) As Long
'   ***********************************************************************
'   * Params:   lIndex: the index or key of the object get the top for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match the index property of a listitem to the
'   *               param, adding the height along the way.
'   ***********************************************************************
    Dim i As Integer
    Dim lTop As Long
    
    lTop = m_lColumnHeaderHeight
    
    For i = 1 To lIndex - 1
        lTop = lTop + m_iTextHeight
    Next
    
    GetListItemTop = lTop
    
End Function

Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oListItem As GTListItem
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oListItem In m_colListItems
        If StrComp(vTempKey, Trim(oListItem.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Friend Sub ResetNumHeaders()
'   ***********************************************************************
'   * Purpose:  Resets the number of columnheaders to 0.
'   ***********************************************************************
    m_lNumHeaders = 0
End Sub
Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oListItem As GTListItem
    
    For Each oListItem In m_colListItems
        If oListItem.Key = vIndex Then
            GetObjectIndexFromKey = oListItem.Index
            Exit Function
        End If
    Next
    
End Function
Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function

