VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTMessage"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit
'   ******************************************************************************
'   * Purpose:  This is the GTMessage object and is used in the Messages collection
'   * File:         GTMessage.cls
'   ******************************************************************************

'   ***** processing constants
Public Enum GTProcessingConstants
    gtBeforeDefaultProc
    gtAfterDefaultProc
End Enum
'   *****

'   ***** property variables
Private m_oMessages As GTMessages
Private m_sText As String
Private m_lValue As Long
Private m_vKey As Variant
Private m_vTag As Variant
Private m_lIndex As Long
Private m_bProcessDefaultProc As Boolean
Private m_eProcessType As GTProcessingConstants
Private m_lReturnValue As Long ' ver 1.0a
Private m_bUseReturnValue As Boolean ' ver 1.0a
'   *****

'   ***** Ver 1.0a. Returns/sets flag UseReturnValue. This tells MsgHook to use the ReturnValue property for the return value
Public Property Let UseReturnValue(bValue As Boolean)
    m_bUseReturnValue = bValue
End Property
Public Property Get UseReturnValue() As Boolean
    UseReturnValue = m_bUseReturnValue
End Property
'   *****

'   ***** Ver 1.0a.Returns/sets the ReturnValue for the message. This is for this message only.
Public Property Let ReturnValue(lValue As Long)
    m_lReturnValue = lValue
End Property
Public Property Get ReturnValue() As Long
    ReturnValue = m_lReturnValue
End Property
'   *****

'   ***** Returns/sets the ProcessType value constant. The default proc is fired before or after the Message event
'   ***** based on this property. For this message only.
Public Property Let ProcessType(eType As GTProcessingConstants)
Attribute ProcessType.VB_Description = "Returns/sets when intercepted message will be processed."
    m_eProcessType = eType
End Property
Public Property Get ProcessType() As GTProcessingConstants
    ProcessType = m_eProcessType
End Property
'   *****

'   ***** Returns/sets the ProcessDefaultProc flag. The default proc will/won't be fired based on this property.
'   ***** For this message only.
Public Property Let ProcessDefaultProc(bProcess As Boolean)
Attribute ProcessDefaultProc.VB_Description = "Returns/sets whether the default event will occur."
    m_bProcessDefaultProc = bProcess
End Property
Public Property Get ProcessDefaultProc() As Boolean
    ProcessDefaultProc = m_bProcessDefaultProc
End Property
'   *****

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set Messages(oMessages As GTMessages)
    Set m_oMessages = oMessages
End Property
'   *****

'   ***** Returns/Sets the Text property. This is any text to name the message, perferably the name of the message
Public Property Let Text(sText As String)
Attribute Text.VB_Description = "Returns/sets the text to be displayed in a control."
Attribute Text.VB_HelpID = 88
    m_sText = sText
    m_oMessages.ChildChanged Me, "Text"
End Property
Public Property Get Text() As String
    Text = m_sText
End Property
'   *****

'   **** used to set the value of index (for internal use only)
Friend Property Let InternalIndex(lIndex As Long)
Attribute InternalIndex.VB_MemberFlags = "40"
    m_lIndex = lIndex
End Property
'   *****

'   ***** returns the the value of Index
Public Property Get Index() As Long
Attribute Index.VB_Description = "Returns/sets the index of an object in a collection. Read-only at run time."
Attribute Index.VB_HelpID = 46
    Index = m_lIndex
End Property
'   *****

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
    If m_oMessages.IsDuplicateKey(vKey) Then
        Err.Raise vbObjectError + 1051, , "Key already exists."
        Exit Property
    End If
    
    If m_vKey = vKey Then
        Exit Property
    End If
    
    m_vKey = vKey
    m_oMessages.ChildChanged Me, "Key"
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

'   ***** Returns/Sets any extra info for the object
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    Set m_vTag = vTag
    m_oMessages.ChildChanged Me, "Tag"
End Property
'   *****
Public Property Let Tag(vTag As Variant)
Attribute Tag.VB_Description = "Stores any extra data needed for your program."
Attribute Tag.VB_HelpID = 87
'   ***** Ver 1.0a have to check if an object before checking the value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
    m_oMessages.ChildChanged Me, "Tag"
End Property
Public Property Get Tag() As Variant
'   ***** Ver 1.0a If an object, have to use Set
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
'   *****
End Property
'   *****

'   ***** the value of the message to intercept.
Public Property Let Value(lValue As Long)
    If m_oMessages.IsDuplicateMsg(lValue) Then
        Err.Raise vbObjectError + 1051, , "Message already exists."
        Exit Property
    End If
    m_lValue = lValue
    m_oMessages.ChildChanged Me, "Value"
End Property
Public Property Get Value() As Long
Attribute Value.VB_Description = "Returns/sets the value of an object."
Attribute Value.VB_HelpID = 94
Attribute Value.VB_UserMemId = 0
    Value = m_lValue
End Property
'   *****

Private Sub Class_Initialize()
'   **************************************************************
'   * Purpose: Initalize the property variables
'   **************************************************************
    m_bProcessDefaultProc = True
    m_eProcessType = gtBeforeDefaultProc
End Sub

Private Sub Class_Terminate()
    Set m_oMessages = Nothing ' release the object
End Sub

