VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTWindow"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  This is the GTWindow object and is used in the Windows collection
'   * File:         GTWindow.cls
'   ******************************************************************************
Option Explicit

'   ***** property variables
Private m_oWindows As GTWindows
Private m_sName As String
Private m_lSubClassHwnd As Long
Private m_vKey As Variant
Private m_vTag As Variant
Private m_lIndex As Long
Private WithEvents m_oMessages As GTMessages
Attribute m_oMessages.VB_VarHelpID = -1
Private m_lPrevWndProc As Long
'   *****

'   ***** Returns/sets the handle of the previous window proc. This is the wndproc that
'   ***** was subclassed need to fire the default proc
Friend Property Let PrevWndProc(hWnd As Long)
    m_lPrevWndProc = hWnd
End Property
Friend Property Get PrevWndProc() As Long
    PrevWndProc = m_lPrevWndProc
End Property
'   *****

'   ***** the GTMessages object
Public Property Get Messages() As GTMessages
Attribute Messages.VB_Description = "Returns a reference to a collection of GTMessage objects."
    Set Messages = m_oMessages
End Property
'   *****

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set Windows(oWindows As GTWindows)
    Set m_oWindows = oWindows
End Property
Friend Property Get Windows() As GTWindows
    Set Windows = m_oWindows
End Property
'   *****
'   ***** returns/sets the name of the window that was subclassed. Used at design time with
'   ***** proppages to hold the name of the control so can get the hwnd at run time.
Public Property Let Name(sName As String)
Attribute Name.VB_Description = "Returns/sets the text to be displayed in a control."
Attribute Name.VB_HelpID = 88
    m_sName = sName
    m_oWindows.ChildChanged Me, "Name"
End Property
Public Property Get Name() As String
    Name = m_sName
End Property
'   *****

'   **** used to set the value of index (for internal use only)
Friend Property Let InternalIndex(lIndex As Long)
Attribute InternalIndex.VB_MemberFlags = "40"
    m_lIndex = lIndex
    m_oMessages.WindowIndex = lIndex
End Property
'   *****

'   ***** returns the the value of Index
Public Property Get Index() As Long
Attribute Index.VB_Description = "Returns/sets the index of an object in a collection. Read-only at run time."
Attribute Index.VB_HelpID = 46
    Index = m_lIndex
End Property
'   *****

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
    If m_oWindows.IsDuplicateKey(vKey) Then
        Err.Raise vbObjectError + 1051, , "Key already exists."
        Exit Property
    End If
    
    If m_vKey = vKey Then
        Exit Property
    End If
    
    m_vKey = vKey
    m_oWindows.ChildChanged Me, "Key"
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

'   ***** Returns/Sets any extra info for the object
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    Set m_vTag = vTag
    m_oWindows.ChildChanged Me, "Tag"
End Property
'   *****
Public Property Let Tag(vTag As Variant)
'   ***** Ver 1.0a have to check if an object before checking the value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
    m_oWindows.ChildChanged Me, "Tag"
End Property
Public Property Get Tag() As Variant
'   ***** Ver 1.0a If an object, have to use Set
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
'   *****
End Property
'   *****
'   ***** the hwnd of the window that is being subclassed
Public Property Let SubClassHwnd(lSubClassHwnd As Long)
    m_lSubClassHwnd = lSubClassHwnd
    m_oWindows.ChildChanged Me, "SubClassHwnd"
End Property
Public Property Get SubClassHwnd() As Long
Attribute SubClassHwnd.VB_Description = "Returns/sets the value of an object."
Attribute SubClassHwnd.VB_HelpID = 94
Attribute SubClassHwnd.VB_UserMemId = 0
    SubClassHwnd = m_lSubClassHwnd
End Property
'   *****

Private Sub Class_Initialize()
    Set m_oMessages = New GTMessages ' intialize the collection object
End Sub

Private Sub Class_Terminate()
    Set m_oMessages = Nothing ' release the object
End Sub

Private Sub m_oMessages_Added(Message As GTMessage)
'   ************************************************************
'   * Param:    Message: the GTMessage object that was added
'   * Purpose:  Called when a GTMessage object is added. In turn calls the MessageAdded
'   *               proc of the GTWindows collection. Eventually an event is fired in the user control.
'   ************************************************************
    m_oWindows.MessageAdded Message
End Sub

Private Sub m_oMessages_Changed(Message As GTMessage, sProperty As String)
'   ************************************************************
'   * Param:    Message: the GTMessage object that was changed, sProperty: the property
'   *               was changed
'   * Purpose:  Called when a GTMessage object is changed. In turn calls the MessageChanged
'   *               proc of the GTWindows collection. Eventually an event is fired in the user control.
'   ************************************************************
    m_oWindows.MessageChanged Message, sProperty
End Sub

Private Sub m_oMessages_Removed(Message As GTMessage)
'   ************************************************************
'   * Param:    Message: the GTMessage object that was removed
'   * Purpose:  Called when a GTMessage object is removed. In turn calls the MessageRemoved
'   *               proc of the GTWindows collection. Eventually an event is fired in the user control.
'   ************************************************************
    m_oWindows.MessageRemoved Message
End Sub
