VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTWindows"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit
'   ******************************************************************************
'   * Purpose:  The GTWindow collection object.
'   * File:         GTWindows.cls
'   ******************************************************************************

Private m_colWindows As Collection ' holds the GTWindows objects
Private m_bSkipChildChanges As Boolean ' flag to not fire the changed event
Private m_bSubClass As Boolean ' whether or not the windows in this collection currently subclassed

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event Added(Window As GTWindow)
Attribute Added.VB_MemberFlags = "40"
Event Removed(Window As GTWindow)
Attribute Removed.VB_MemberFlags = "40"
Event Changed(Window As GTWindow, sProperty As String)
Attribute Changed.VB_MemberFlags = "40"
Event MessageAdded(Message As GTMessage)
Attribute MessageAdded.VB_MemberFlags = "40"
Event MessageRemoved(Message As GTMessage)
Attribute MessageRemoved.VB_MemberFlags = "40"
Event MessageChanged(Message As GTMessage, Property As String)
Attribute MessageChanged.VB_MemberFlags = "40"
Event CallBack(hWnd As Long, lMsg As Long, wParam As Long, lParam As Long)
Attribute CallBack.VB_MemberFlags = "40"
'   *****

'   ***** Returns/sets the subclass flag. This is used to tell if the windows in the collection are currently being
'   ***** subclassed. This is mainly used in situations where multiple windows classes are presents
Friend Property Let Subclass(bSubclass As Boolean)
    m_bSubClass = bSubclass
End Property
Friend Property Get Subclass() As Boolean
    Subclass = m_bSubClass
End Property
'   *****

'   ***** Returns the number of objects in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = " Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colWindows.Count
End Property
'   *****

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index) As GTWindow
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)

    If lIndex < 1 Or lIndex > m_colWindows.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colWindows(lIndex)
End Property
'   *****

'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
'   ***** NewEnum must return the IUnknown interface of a collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colWindows.[_NewEnum]
End Function
'   *****

Public Function Add(Optional Index, Optional Key, Optional Name, Optional SubClassHwnd) As GTWindow
Attribute Add.VB_Description = "Adds a GTWindow object to a GTWindows collection and returns a reference to the created object."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Key: Optional: the key of the new object,
'   *               Name: Optional: the name of the window to subclass, SubClassHwnd: the ahandle to the
'   *               window to subclass
'   * Return:   GTWindow: the object if a GTWindow was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTWindow object to the Windows collection.
'   ***********************************************************************
    Dim oWindow As GTWindow
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    Set oWindow = New GTWindow
    
    m_bSkipChildChanges = True
    
    Set oWindow.Windows = Me
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Key) Then
        oWindow.Key = Key
    End If
    
    If Not IsMissing(SubClassHwnd) Then
        oWindow.SubClassHwnd = SubClassHwnd
    End If
    
    If Not IsMissing(Name) Then
        oWindow.Name = Name
    End If
    
    If IsMissing(Index) Then
        oWindow.InternalIndex = m_colWindows.Count + 1
        m_colWindows.Add oWindow
    Else
        m_colWindows.Add oWindow, , , lIndex
        ReIndexObjects
    End If
    
    m_bSkipChildChanges = False
                            
    RaiseEvent Added(oWindow)
    
    Set Add = oWindow
    
    Exit Function
    
Add_err:
    m_bSkipChildChanges = False
    MsgBox Err.Description, vbInformation, App.Title
End Function
Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the object to remove
'   * Purpose:  Removes an object from the collection.
'   ***********************************************************************
    Dim oDeletedWindow As GTWindow
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    Set oDeletedWindow = Me(lIndex)
    
    m_colWindows.Remove lIndex
    
    ReIndexObjects
    
    RaiseEvent Removed(oDeletedWindow)
    
End Sub
Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oWindow As GTWindow
    
    For Each oWindow In m_colWindows
        If oWindow.Key = vIndex Then
            GetObjectIndexFromKey = oWindow.Index
            Exit Function
        End If
    Next
    
End Function

Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oWindow As GTWindow
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oWindow In m_colWindows
        If StrComp(vTempKey, Trim(oWindow.Key), vbTextCompare) = 0 Then
'        If LCase(Trim(vKey)) = LCase(Trim(oWindow.Key)) Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all objects
'   ***********************************************************************
    Set m_colWindows = New Collection
    
    RaiseEvent Removed(Nothing)
End Sub

Friend Sub ChildChanged(oWindow As GTWindow, sProperty As String)
'   ***********************************************************************
'   * Params:   oWindow: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTWindow object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If Not m_bSkipChildChanges Then
        RaiseEvent Changed(oWindow, sProperty)
    End If
End Sub

Friend Sub ReadObjectProps(oPropBag As PropertyBag)
'   ***********************************************************************
'   * Params:   oPropBag: the property bag to read property values from
'   * Purpose:  Reads the properties from the property bag. Reads only
'   *               the properties for objects that were changed at design time
'   *               thru the property pages.
'   ***********************************************************************
    Dim i As Long
    Dim j As Long
    Dim lCount As Long
    Dim oWindow As GTWindow
    
    lCount = oPropBag.ReadProperty("NumWindows", 0)
    
    For i = 1 To lCount
        
        Set oWindow = Me.Add()
        
        oWindow.Tag = oPropBag.ReadProperty("WindowTag" & i, "")
        oWindow.Key = oPropBag.ReadProperty("WindowKey" & i, "")
        oWindow.Name = oPropBag.ReadProperty("WindowText" & i, "")
        oWindow.SubClassHwnd = oPropBag.ReadProperty("WindowSubClassHwnd" & i, 0)
        
        oWindow.Messages.ReadObjectProps oPropBag
    Next
    
End Sub

Private Sub ReIndexObjects()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colWindows.Count
        Me(i).InternalIndex = i
    Next
    
End Sub

Friend Sub WriteObjectProps(oPropBag As PropertyBag)
'   ***********************************************************************
'   * Params:   oPropBag: the property bag to write property values to
'   * Purpose:  Writes the properties to the property bag. Writes only
'   *               the properties for objects that were changed at design time
'   *               thru the property pages.
'   ***********************************************************************
    Dim i As Long
    Dim j As Long
    Dim oWindow As GTWindow
    
    Call oPropBag.WriteProperty("NumWindows", m_colWindows.Count, 0)
    
    For Each oWindow In m_colWindows
        
        Call oPropBag.WriteProperty("WindowText" & oWindow.Index, oWindow.Name, "")
        Call oPropBag.WriteProperty("WindowTag" & oWindow.Index, oWindow.Tag, "")
        Call oPropBag.WriteProperty("WindowKey" & oWindow.Index, oWindow.Key, "")
        Call oPropBag.WriteProperty("WindowSubClassHwnd" & oWindow.Index, oWindow.SubClassHwnd, 0)
        
        oWindow.Messages.WriteObjectProps oPropBag
        
    Next
    
End Sub

Private Sub Class_Initialize()
    Set m_colWindows = New Collection ' initialize the collection
End Sub

Private Sub Class_Terminate()
    Set m_colWindows = Nothing ' release the object
End Sub

Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function

Friend Sub MessageAdded(oMessage As GTMessage)
'   ***********************************************************************
'   * Params:   oMessage: the object that was added
'   * Purpose:  Raises the MessageAdded event
'   ***********************************************************************
    RaiseEvent MessageAdded(oMessage)
End Sub
Friend Sub MessageRemoved(oMessage As GTMessage)
'   ***********************************************************************
'   * Params:   oMessage: the object that was removed
'   * Purpose:  Raises the MessageRemoved event
'   ***********************************************************************
    RaiseEvent MessageRemoved(oMessage)
End Sub
Friend Sub MessageChanged(oMessage As GTMessage, sProperty As String)
'   ***********************************************************************
'   * Params:   oMessage: the object that was changed, sProperty:
'   *               the property that was changed
'   * Purpose:  Raises the MessageChanged event
'   ***********************************************************************
    RaiseEvent MessageChanged(oMessage, sProperty)
End Sub

Friend Function IsHwndPresent() As Boolean
'   ***********************************************************************
'   * Return:   Boolean: whether or not there is a window to be subclassed
'   * Purpose:  Loops thru checking to see if there are any windows that are subclassed
'   ***********************************************************************
    Dim oWindow As GTWindow
    
    For Each oWindow In m_colWindows
        If oWindow.SubClassHwnd <> 0 Then
            IsHwndPresent = True
            Exit Function
        End If
    Next
    
End Function

Friend Sub ProcessMessage(ByVal hWnd As Long, ByVal lMsgID As Long, ByVal wParam As Long, ByVal lParam As Long)
'   ***********************************************************************
'   * Params:   hWnd: the hWnd of the window, lMsgID: the message that was intercepted,
'   *               wParam: the wParam, lParam: the lParam
'   * Purpose:  Raises the Callback event
'   ***********************************************************************
    RaiseEvent CallBack(hWnd, lMsgID, wParam, lParam)
End Sub
'Friend Function SendCurrentMessage(oWindow As GTWindow, hWnd As Long, lMsgID As Long, bProcessDefaultProc As Boolean, _
'                        eProcessType As GTProcessingConstants) As Boolean
Friend Function SendCurrentMessage(oWindow As GTWindow, hWnd As Long, lMsgID As Long, bProcessDefaultProc As Boolean, _
                        eProcessType As GTProcessingConstants, oRetMsg As GTMessage) As Boolean
'   ***********************************************************************
'   * Params:   oWindow: the window check for the message, hWnd: the hWnd of the window,
'   *               lMsgID: the message that was intercepted, bProcessDefaultProc: whether or not
'   *               to process the default proc (this value is returned), eProcessType: pre or post
'   *               processing of the message (this value is returned), oMessage: the Message that was found
'   * Return:   Boolean: whether or not to send the message and raise the Message event.
'   * Purpose:  Searches thru the messages for the window to see if any messages match the message that
'   *               was intercepted.
'   * Modified: Ver 1.0a    Added oMessage param to handle the return value
'   ***********************************************************************
    Dim oMessage As GTMessage
    
    For Each oMessage In oWindow.Messages
        If oMessage.Value = lMsgID Then
            Set oRetMsg = oMessage ' ver 1.0a
            bProcessDefaultProc = oMessage.ProcessDefaultProc
            eProcessType = oMessage.ProcessType
            SendCurrentMessage = True
            Exit Function
        End If
    Next
    
    SendCurrentMessage = False
End Function


