VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTPane"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  This is the GTPane object and is used in the Panes collection
'   * File:         GTPane.cls
'   ******************************************************************************
Option Explicit
'   ***** property variables
Private m_lLeft As Long
Private m_lTop As Long
Private m_lWidth As Long
Private m_lHeight As Long
Private m_lBackColor As OLE_COLOR
Private m_eAppearance As GTPaneAppearanceConstants
Private m_vKey As Variant
Private m_lIndex As Long
Private m_oPanes As GTPanes
Private m_sName As String
Private m_lSaveHeight As Long
Private m_lSaveWidth As Long
Private m_vTag As Variant
Private m_sControlName As String
Private m_lMin As Long
Private m_bResizeChildControl As Boolean
'   *****

'   ***** Returns/sets the resizechildcontrol property. If True, and the ControlName
'   ***** property isn't "" then the control is resize by the splitter control.
Public Property Let ResizeChildControl(bResize As Boolean)
Attribute ResizeChildControl.VB_Description = "Whether or not the control referenced in ControlName will be resized automatically."
Attribute ResizeChildControl.VB_HelpID = 67
    m_bResizeChildControl = bResize
    m_oPanes.ChildChanged Me, "ResizeChildControl"
End Property
Public Property Get ResizeChildControl() As Boolean
    ResizeChildControl = m_bResizeChildControl
End Property
'   *****

'   ***** Returns/sets the Minimum property. This is the smallest width/height a pane
'   ***** can be. For internal use only
Friend Property Let InternalMinimum(lMin As Long)
    Dim sWidthHeight As String
    
    If lMin > m_lWidth And m_lWidth <> 0 Then
        If m_oPanes.Orientation = gtHorizontal Then
            sWidthHeight = "Width"
        Else
            sWidthHeight = "Height"
        End If
        
        Err.Raise vbObjectError + 1051, App.Title, "Minimum must be less than or equal to " & sWidthHeight & ".", App.HelpFile
        Exit Property
    End If
    
    m_lMin = lMin
End Property
Friend Property Get InternalMinimum() As Long
Attribute InternalMinimum.VB_MemberFlags = "40"
    InternalMinimum = m_lMin
End Property
'   *****

'   ***** Returns/sets the Minimum property. This is the smallest width/height a pane
'   ***** can be. Returned in the scalemode of the container.
Public Property Let Minimum(lMin As Long)
Attribute Minimum.VB_Description = "The smallest width/height of the GTPane object."
Attribute Minimum.VB_HelpID = 57
    Dim sWidthHeight As String
    
    If lMin < 0 Then
        Err.Raise vbObjectError + 1052, App.Title, "Minimum must be greater than 0.", App.HelpFile
        Exit Property
    End If
    
    If lMin / m_oPanes.ScalePerPixel > m_lWidth And m_lWidth > 0 Then
        If m_oPanes.Orientation = gtHorizontal Then
            sWidthHeight = "Width"
        Else
            sWidthHeight = "Height"
        End If
        
        Err.Raise vbObjectError + 1051, App.Title, "Minimum must be less than or equal to " & sWidthHeight & ".", App.HelpFile
        Exit Property
    End If
    
    m_lMin = lMin / m_oPanes.ScalePerPixel
    
    m_oPanes.ChildChanged Me, "Minimum"

End Property
Public Property Get Minimum() As Long
    Minimum = m_lMin * m_oPanes.ScalePerPixel
End Property
'   *****

'   ***** Returns/sets the controlname to be automatically resized to fit the pane.
Public Property Let ControlName(sName As String)
Attribute ControlName.VB_Description = "The name of the control that will be resize automatically."
Attribute ControlName.VB_HelpID = 27
Attribute ControlName.VB_ProcData.VB_Invoke_PropertyPut = ";Misc"
    m_sControlName = sName
    m_oPanes.ChildChanged Me, "ControlName"
End Property
Public Property Get ControlName() As String
    ControlName = m_sControlName
End Property
'   *****

'   ***** Returns/Sets any extra info for the object
'   *****
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    Set m_vTag = vTag
    m_oPanes.ChildChanged Me, "Tag"
End Property
'   *****
Public Property Let Tag(vTag As Variant)
'   ***** Ver 1.0a have to check if an object before checking the value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
    m_oPanes.ChildChanged Me, "Tag"
End Property
Public Property Get Tag() As Variant
'   ***** Ver 1.0a If an object, have to use Set
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
'   *****
End Property
'   *****

'   ***** Returns the height before the pane was resized.
Friend Property Get SaveHeight() As Long
    SaveHeight = m_lSaveHeight
End Property
'   *****

'   ***** Returns the width before the pane was resized.
Friend Property Get SaveWidth() As Long
    SaveWidth = m_lSaveWidth
End Property
'   *****

'   ***** Returns/sets the name of the Pane.
Public Property Let Name(sName As String)
    m_sName = sName
    m_oPanes.ChildChanged Me, "Name"
End Property
Public Property Get Name() As String
Attribute Name.VB_Description = "Descriptive identifier for a GTPane object."
Attribute Name.VB_HelpID = 61
Attribute Name.VB_UserMemId = 0
    Name = m_sName
End Property
'   *****

'   ***** Returns/sets the left of the pane. Contained controls are always set in Twips.
Public Property Let Left(lLeft As Long)
Attribute Left.VB_Description = "Returns/sets the distance between the internal left edge of an object and the left edge of its container."
Attribute Left.VB_HelpID = 52
Attribute Left.VB_ProcData.VB_Invoke_PropertyPut = ";Position"
    Err.Raise vbObjectError + 1050, App.Title, "Left is Read-Only at runtime.", App.HelpFile
End Property
Public Property Get Left() As Long
    Dim lLeft As Long
    Dim iOffset As Integer
    
    If m_lLeft = 0 Then
        lLeft = m_oPanes.GetPaneLeft(Me)
    Else
        lLeft = m_lLeft
    End If

    If m_eAppearance = gtPane3D Then
        iOffset = 2
    End If
    
    Left = (lLeft + iOffset) * Screen.TwipsPerPixelX
End Property
'   *****

'   ***** Returns/sets the top of the pane. Contained controls are always set in Twips.
Public Property Let Top(lTop As Long)
Attribute Top.VB_Description = "Returns/sets the distance between the internal top edge of an object and the top edge of its container."
Attribute Top.VB_HelpID = 92
Attribute Top.VB_ProcData.VB_Invoke_PropertyPut = ";Position"
    Err.Raise vbObjectError + 1050, App.Title, "Top is Read-Only at runtime.", App.HelpFile
End Property
Public Property Get Top() As Long
    Dim lTop As Long
    Dim iOffset As Integer
    
    If m_lTop = 0 Then
        lTop = m_oPanes.GetPaneTop(Me)
    Else
        lTop = m_lTop
    End If
    
    If m_eAppearance = gtPane3D Then
        iOffset = 2
    End If
    
    Top = (lTop + iOffset) * Screen.TwipsPerPixelY
    
End Property
'   *****

'   ***** Returns/sets the width of the pane. Contained controls are always set in Twips.
Public Property Let Width(lWidth As Long)
Attribute Width.VB_Description = "Returns/sets the width of an object."
Attribute Width.VB_HelpID = 98
Attribute Width.VB_ProcData.VB_Invoke_PropertyPut = ";Scale"
    Dim lTempWidth As Long
    
    If m_oPanes.Orientation = gtVertical Then
        Err.Raise vbObjectError + 1050, App.Title, "Width is read-only at runtime.", _
            App.HelpFile
        Exit Property
    End If
    
    If m_oPanes.Count = 1 Then
        Err.Raise vbObjectError + 1052, App.Title, "Width is read-only when there is only 1 Pane.", _
            App.HelpFile
        Exit Property
    End If
    
    If lWidth / m_oPanes.ScalePerPixel < m_lMin Then
        lTempWidth = m_lMin
    Else
        lTempWidth = lWidth / m_oPanes.ScalePerPixel
    End If
    
    If m_eAppearance = gtPane3D Then
        lTempWidth = lTempWidth + 4
    End If
    
    m_lWidth = lTempWidth
    m_oPanes.ChildChanged Me, "Width"

'   ***** ver 1.0a Fixes problem with setting same width multiple times
    m_lSaveWidth = lTempWidth

End Property
Public Property Get Width() As Long
    Dim lWidth As Long
    Dim iAppearanceOffset As Integer
    
    If m_lWidth = -1 Then
        lWidth = m_oPanes.GetPaneWidth()
    Else
        lWidth = m_lWidth
    End If
    
    If m_eAppearance = gtPane3D Then
        lWidth = lWidth - 4
    End If
    
    If m_oPanes.Orientation = gtHorizontal Then
        Width = lWidth * Screen.TwipsPerPixelX
    Else
        If lWidth < 0 Then
            lWidth = 0
        End If
        Width = lWidth * Screen.TwipsPerPixelX
    End If
    
End Property
'   *****

'   ***** Returns/sets the Height of the pane. Contained controls are always set in Twips.
Public Property Let Height(lHeight As Long)
Attribute Height.VB_Description = "Returns/sets the height of an object."
Attribute Height.VB_HelpID = 40
Attribute Height.VB_ProcData.VB_Invoke_PropertyPut = ";Scale"
    Dim lTempHeight As Long
    
    If m_oPanes.Orientation = gtHorizontal Then
        Err.Raise vbObjectError + 1050, App.Title, "Height is read-only at runtime.", _
            App.HelpFile
        Exit Property
    End If
    
    If m_oPanes.Count = 1 Then
        Err.Raise vbObjectError + 1052, App.Title, "Height is read-only when there is only 1 Pane.", _
            App.HelpFile
        Exit Property
    End If
    
    If lHeight / m_oPanes.ScalePerPixel < m_lMin Then
        lTempHeight = m_lMin
    Else
        lTempHeight = lHeight / m_oPanes.ScalePerPixel
    End If
    
    If m_eAppearance = gtPane3D Then
        lTempHeight = lTempHeight + 4
    End If
    
    m_lHeight = lTempHeight
    m_oPanes.ChildChanged Me, "Height"
    
'   ***** ver 1.0a Fixes problem with setting same width multiple times
    m_lSaveHeight = lTempHeight
End Property
Public Property Get Height() As Long
    Dim lHeight As Long
    Dim iAppearanceOffset As Integer
    
    If m_lHeight = -1 Then
        lHeight = m_oPanes.GetPaneHeight()
    Else
        lHeight = m_lHeight
    End If
    
    If m_eAppearance = gtPane3D Then
        lHeight = lHeight - 4
    End If
    
    If m_oPanes.Orientation = gtVertical Then
        Height = lHeight * Screen.TwipsPerPixelY
    Else
        If lHeight < 0 Then
            lHeight = 0
        End If
        Height = lHeight * Screen.TwipsPerPixelY
    End If
End Property

'   ***** Returns/sets Appearance of the Pane: Flat or inset
Public Property Let Appearance(eAppearance As GTPaneAppearanceConstants)
    m_eAppearance = eAppearance
    m_oPanes.ChildChanged Me, "Appearance"
End Property
Public Property Get Appearance() As GTPaneAppearanceConstants
Attribute Appearance.VB_Description = " Returns/sets whether or not the Pane object is painted at run time with 3-D effects."
Attribute Appearance.VB_HelpID = 22
Attribute Appearance.VB_ProcData.VB_Invoke_Property = ";Appearance"
Attribute Appearance.VB_UserMemId = -520
    Appearance = m_eAppearance
End Property
'   *****

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
Attribute Key.VB_ProcData.VB_Invoke_PropertyPut = ";Misc"
    If m_oPanes.IsDuplicateKey(vKey) Then
        Err.Raise vbObjectError + 1051, , "Key already exists."
        Exit Property
    End If
    m_vKey = vKey
    m_oPanes.ChildChanged Me, "Key"
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

'   **** used to set the value of index (for internal use only)
Friend Property Let InternalIndex(lIndex As Long)
Attribute InternalIndex.VB_MemberFlags = "40"
    m_lIndex = lIndex
End Property
'   *****

'   ***** returns/sets the the value of Index
Public Property Let Index(lIndex As Long)
Attribute Index.VB_Description = "Returns/sets the index of an object in a collection. Read-only at run time."
Attribute Index.VB_HelpID = 46
Attribute Index.VB_ProcData.VB_Invoke_PropertyPut = ";Misc"
    If Not g_bInternalUse Then
        Err.Raise vbObjectError + 1050, App.Title, "Index is Read-Only at runtime.", _
            App.HelpFile
        Exit Property
    End If
    m_lIndex = lIndex
End Property
Public Property Get Index() As Long
    Index = m_lIndex
End Property
'   *****

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set Collection(oPanes As GTPanes)
Attribute Collection.VB_MemberFlags = "40"
    Set m_oPanes = oPanes
End Property
'   *****

Private Sub Class_Initialize()
'   **************************************************************
'   * Purpose: Initalize the property variables
'   **************************************************************
    m_lWidth = 1500
    m_lHeight = 1500
    m_eAppearance = gtPane3D
    m_lMin = 10
    m_bResizeChildControl = True
End Sub

Private Sub Class_Terminate()
    Set m_oPanes = Nothing ' release the object
End Sub

'   ***** Returns/sets the Left of the Pane in Pixels. Internal Use Only.
Friend Property Let InternalLeft(lLeft As Long)
    m_lLeft = lLeft
End Property
Friend Property Get InternalLeft() As Long
Attribute InternalLeft.VB_MemberFlags = "40"
    Dim lLeft As Long
    Dim iOffset As Integer
    
    If m_lLeft = 0 Then
        lLeft = m_oPanes.GetPaneLeft(Me)
    Else
        lLeft = m_lLeft
    End If

    InternalLeft = lLeft + iOffset
End Property
'   *****

'   ***** Returns/sets the top of the Pane in Pixels. Internal Use Only.
Friend Property Let InternalTop(lTop As Long)
    m_lTop = lTop
End Property
Friend Property Get InternalTop() As Long
Attribute InternalTop.VB_MemberFlags = "40"
    Dim lTop As Long
    Dim iOffset As Integer
    
    If m_lTop = 0 Then
        lTop = m_oPanes.GetPaneTop(Me)
    Else
        lTop = m_lTop
    End If
    
    InternalTop = lTop + iOffset
End Property
'   *****

'   ***** Returns/sets the width of the Pane in Pixels. Internal Use Only.
Friend Property Let InternalWidth(lWidth As Long)
    If m_oPanes.Orientation = gtHorizontal Then
        If lWidth < m_lMin Then
            Err.Raise vbObjectError + 1055, App.Title, "Width cannot be less than Min.", _
                App.HelpFile
            Exit Property
        End If
    End If
    
    m_lWidth = lWidth
    If Not g_bInternalUse Then
        m_lSaveWidth = lWidth
    End If
End Property
Friend Property Get InternalWidth() As Long
Attribute InternalWidth.VB_MemberFlags = "40"
    Dim iAppearanceOffset As Integer
    
    If m_lWidth = 0 Then
        InternalWidth = m_oPanes.GetPaneWidth()
    Else
        InternalWidth = m_lWidth
    End If
    
    InternalWidth = InternalWidth + iAppearanceOffset
End Property
'   *****

'   ***** Returns/sets the height of the Pane in Pixels. Internal User Only.
Friend Property Let InternalHeight(lHeight As Long)
    If m_oPanes.Orientation = gtVertical Then
        If lHeight < m_lMin Then
            Err.Raise vbObjectError + 1055, App.Title, "Height cannot be less than Min.", _
                App.HelpFile
            Exit Property
        End If
    End If
    
    m_lHeight = lHeight
    If Not g_bInternalUse Then
        m_lSaveHeight = lHeight
    End If
End Property
Friend Property Get InternalHeight() As Long
Attribute InternalHeight.VB_MemberFlags = "40"
    Dim iAppearanceOffset As Integer
    
    If m_lHeight = 0 Then
        InternalHeight = m_oPanes.GetPaneHeight()
    Else
        InternalHeight = m_lHeight
    End If
    
    InternalHeight = InternalHeight + iAppearanceOffset
End Property
'   *****

