VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTPanes"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  The GTPanes collection object.
'   * File:         GTPanes.cls
'   ******************************************************************************
Option Explicit

'   ***** Pane appearance constants
Public Enum GTPaneAppearanceConstants
    gtPaneFlat
    gtPane3D
End Enum
'   *****

'   ***** property variables
Private m_CSplitter As CSplitter
Private m_colPanes As Collection
Private m_bSkipChildChanges As Boolean
Private m_lLeft As Long
Private m_lTop As Long
Private m_lScaleHeight As Long
Private m_lScaleWidth As Long
Private m_iOffset As Integer
Private m_eOrientation As Integer
Private m_lSplitterSize As Long
Private m_eAppearance As Integer
Private m_eBorderStyle As Integer
Private m_iScalePerPixel As Integer
Private m_lAddHeight As Long
Private m_lAddWidth As Long
Private m_bDontResizeLastPane As Boolean
'   *****

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event PaneAdded(oPane As GTPane)
Attribute PaneAdded.VB_MemberFlags = "40"
Event PaneRemoved(ByVal oPane As GTPane)
Attribute PaneRemoved.VB_MemberFlags = "40"
Event PaneChanged(oPane As GTPane, sProperty As String)
Attribute PaneChanged.VB_MemberFlags = "40"
Event SplitterResized(oPane As GTPane, oPaneNext As GTPane)
Attribute SplitterResized.VB_MemberFlags = "40"
'   *****

'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
'   *****NewEnum must return the IUnknown interface of a collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colPanes.[_NewEnum]
End Function
'   *****

'   ***** Sets the ScaleWidth without the border
Friend Property Let ScaleWidth(lScaleWidth As Long)
    m_lScaleWidth = lScaleWidth
End Property
'   *****

'   ***** Sets the ScaleWidth without the border
Friend Property Let ScaleHeight(lScaleHeight As Long)
    m_lScaleHeight = lScaleHeight
End Property
'   *****

'   ***** Sets the flag not to resize the last pane. Was previously used during loading
'   ***** at design time.
Friend Property Let DontResizeLastPane(bDont As Boolean)
    m_bDontResizeLastPane = bDont
End Property
'   *****
'   ***** Sets the AddHeight property. Only used when loading the panes at design time.
Friend Property Let AddHeight(lHeight As Long)
    m_lAddHeight = lHeight
End Property
'   *****

'   ***** Sets the AddHeight property. Only used when loading the panes at design time.
Friend Property Let AddWidth(lWidth As Long)
    m_lAddWidth = lWidth
End Property
'   *****

'   ***** Returns/sets the Scaleunits per pixel. Can't get this value in a class, so it has to
'   ***** passed in.
Friend Property Let ScalePerPixel(iScalePerPixel As Integer)
    m_iScalePerPixel = iScalePerPixel
End Property
Friend Property Get ScalePerPixel() As Integer
    ScalePerPixel = m_iScalePerPixel
End Property
'   *****

'   ***** Sets the hwnd of the user control
Friend Property Let ParentHwnd(hWnd As Long)
    m_CSplitter.ParentHwnd = hWnd
End Property
'   *****

'   ***** Returns/sets the offset value so it is available in the class.
Friend Property Let Offset(iOffset As Integer)
    m_iOffset = iOffset
End Property
Friend Property Get Offset() As Integer
    Offset = m_iOffset
End Property
'   *****

'   ***** Returns/sets the Appearance value so it is available in the class.
Friend Property Let Appearance(eAppearance As Integer)
    m_eAppearance = eAppearance
End Property
Friend Property Get Appearance() As Integer
    Appearance = m_eAppearance
End Property
'   *****

'   ***** Returns/sets the BorderStyle value so it is available in the class.
Friend Property Let BorderStyle(eBorderStyle As Integer)
    m_eBorderStyle = eBorderStyle
End Property
Friend Property Get BorderStyle() As Integer
    BorderStyle = m_eBorderStyle
End Property
'   *****

'   ***** Returns/sets the size of the splitter in pixels so it is available to the class.
Friend Property Let SplitterSize(lSize As Long)
    If m_eOrientation = gtHorizontal Then
        m_CSplitter.Width = lSize
        m_CSplitter.Height = m_lScaleHeight
    Else
        m_CSplitter.Width = m_lScaleWidth
        m_CSplitter.Height = lSize
    End If
    m_lSplitterSize = lSize
End Property
Friend Property Get SplitterSize() As Long
    SplitterSize = m_lSplitterSize
End Property
'   *****

'   ***** Returns/sets the orientation so it is available to the class.
Friend Property Let Orientation(eOrientation As Integer)
    If m_eOrientation = eOrientation Then
        Exit Property
    End If
    
    m_eOrientation = eOrientation
    m_CSplitter.Orientation = eOrientation
    
    OrientationChange
    
End Property
Friend Property Get Orientation() As Integer
    Orientation = m_eOrientation
End Property
'   *****

Public Function Add(Optional Index, Optional Key, Optional Appearance, Optional ControlName) As GTPane
Attribute Add.VB_Description = "Adds a Pane object to a Panes collection and returns a reference to the created object."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Key: Optional: the key of the new object,
'   *               Appearance: Optional: the appearance constant, ControlName: the name of the control to size
'   *               to the new pane.
'   * Return:   GTPane: the object if a pane was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTPane object to the Panes collection.
'   ***********************************************************************
    Dim oPane As GTPane
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    Set oPane = New GTPane
    
    Set oPane.Collection = Me
    
    m_bSkipChildChanges = True
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Key) Then
        oPane.Key = Key
    End If
    
    If Not IsMissing(Appearance) Then
        oPane.Appearance = Appearance
    End If
    
    If Not IsMissing(ControlName) Then
        oPane.ControlName = ControlName
    End If
    
'   ***** Based on the orientation the new pane is added splitting the last pane
'   ***** or the index pane in half
    If m_eOrientation = gtHorizontal Then
        If m_lAddWidth = 0 Then
            If m_colPanes.Count > 0 Then
                    If IsMissing(Index) Then
                        Me(m_colPanes.Count).InternalWidth = (Me(m_colPanes.Count).InternalWidth - m_lSplitterSize) / 2
                        oPane.InternalWidth = Me(m_colPanes.Count).InternalWidth
                    Else
                        Me(lIndex).InternalWidth = (Me(lIndex).InternalWidth - m_lSplitterSize) / 2
                        oPane.InternalWidth = Me(lIndex).InternalWidth
                    End If
            Else
                oPane.InternalWidth = m_lScaleWidth
            End If
        Else
            oPane.InternalWidth = m_lAddWidth
        End If
    
        oPane.InternalHeight = -1
        
    Else
        If m_lAddHeight = 0 Then
            If m_colPanes.Count > 0 Then
                    If IsMissing(Index) Then
                        Me(m_colPanes.Count).InternalHeight = (Me(m_colPanes.Count).InternalHeight - m_lSplitterSize) / 2
                        oPane.InternalHeight = Me(m_colPanes.Count).InternalHeight
                    Else
                        Me(lIndex).InternalHeight = (Me(lIndex).InternalHeight - m_lSplitterSize) / 2
                        oPane.InternalHeight = Me(lIndex).InternalHeight
                    End If
            Else
                oPane.InternalHeight = m_lScaleHeight
            End If
        Else
            oPane.InternalHeight = m_lAddHeight
        End If
        
        oPane.InternalWidth = -1
        
    End If
'   *****

    oPane.Name = "Pane " & m_colPanes.Count + 1
    
'   ***** Adds the object to the collection
    If IsMissing(Index) Then
        m_colPanes.Add oPane
        oPane.InternalIndex = m_colPanes.Count
    Else
        m_colPanes.Add oPane, , , lIndex
        ReIndexPanes
    End If
'   *****

    m_bSkipChildChanges = False

    RaiseEvent PaneAdded(oPane)
    
    g_bInternalUse = False
    
    Set Add = oPane
    
    Exit Function
Add_err:
    MsgBox Err.Description, vbInformation, App.Title
    Set Add = Nothing
    m_bSkipChildChanges = False
End Function

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index) As GTPane
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)

    If lIndex < 1 Or lIndex > m_colPanes.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colPanes(lIndex)
End Property
'   *****

Private Sub ReIndexPanes()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colPanes.Count
        Me(i).InternalIndex = i
    Next
    
End Sub

Private Sub Class_Initialize()
'   ***********************************************************************
'   * Purpose:  Initializes the variables.
'   ***********************************************************************
    Set m_colPanes = New Collection
    Set m_CSplitter = New CSplitter
End Sub

Private Sub Class_Terminate()
'   ***********************************************************************
'   * Purpose:  Releases the objects
'   ***********************************************************************
    Set m_CSplitter = Nothing
    Set m_colPanes = Nothing
End Sub

'   ***** Returns the number of objects in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = "Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colPanes.Count
End Property
'   *****

Friend Sub ChildChanged(oPane As GTPane, sProperty As String)
'   ***********************************************************************
'   * Params:   oPane: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTPane object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If m_bSkipChildChanges Then
        Exit Sub
    End If
    
    RaiseEvent PaneChanged(oPane, sProperty)
End Sub
Friend Sub CreateFirstPane()
'   ***********************************************************************
'   * Purpose:  Called from the InitProperties event to initialize the
'   *               first pane.
'   ***********************************************************************
    Dim oPane As GTPane
    Dim oPaneNext As GTPane
    
    If m_eOrientation = gtHorizontal Then
        m_lAddWidth = m_lScaleWidth
        Set oPane = Add()
        m_lAddWidth = 0
    Else
        m_lAddHeight = m_lScaleHeight
        Set oPane = Add()
        m_lAddHeight = 0
    End If
    
    RaiseEvent SplitterResized(oPane, oPaneNext)

End Sub
Private Sub OrientationChange()
'   ***********************************************************************
'   * Purpose:  Called when the orientation changes. This will resize
'   *               all panes equally.
'   ***********************************************************************
    Dim oPane As GTPane
    
'   ***** Because the width and height are usually not equal, when changing
'   ***** the orientation the panes are all made equal widths/heights
    For Each oPane In m_colPanes
        If m_eOrientation = gtHorizontal Then
            oPane.InternalWidth = (m_lScaleWidth - (m_lSplitterSize * (m_colPanes.Count - 1))) / m_colPanes.Count
            oPane.InternalHeight = m_lScaleHeight
        Else
            oPane.InternalWidth = m_lScaleWidth
            oPane.InternalHeight = (m_lScaleHeight - (m_lSplitterSize * (m_colPanes.Count - 1))) / m_colPanes.Count
        End If
    Next
'   *****

End Sub

Friend Function HitTest(x As Single, y As Single) As GTPane
'   *********************************************************************************
'   * Params:   X: the left of the point to find, Y: the top of the point to find.
'   * Return:   GTPane: the pane that was found at the X,Y point; Nothing if
'   *               no pane was found.
'   * Purpose:  Finds the pane at the X,Y point and returns it.Loops thru the
'   *               collection checking and added the pane widths, heights, etc
'   *               depending on the orientation.
'   *********************************************************************************
    Dim oPane As GTPane
    Dim lLeft As Long
    Dim lTop As Long
    Dim i As Long
    
    If m_eOrientation = gtHorizontal Then
        lLeft = m_iOffset
        lTop = m_iOffset
    End If
    
    For i = 1 To m_colPanes.Count - 1 ' don't check for the last pane because that is the end of the control
        
        Set oPane = m_colPanes(i)
        
'   ***** Finds the splitter
        If m_eOrientation = gtHorizontal Then
            If lLeft + oPane.InternalWidth - 2 < x And lLeft + oPane.InternalWidth + m_lSplitterSize + 2 > x Then
                Set HitTest = oPane
                Exit Function
            Else
                lLeft = lLeft + oPane.InternalWidth + m_lSplitterSize
            End If
        Else
            If lTop + oPane.InternalHeight - 2 < y And lTop + oPane.InternalHeight + m_lSplitterSize + 2 > y Then
                Set HitTest = oPane
                Exit Function
            Else
                lTop = lTop + oPane.InternalHeight + m_lSplitterSize
            End If
        End If
'   *****

    Next
    
End Function

Friend Sub MouseDown(oPane As GTPane, x As Single, y As Single)
'   *********************************************************************************
'   * Params:   oPane: the pane that the splitter is to the right of, X: the x point
'   *               of the mouse, Y: the y point of the mouse
'   * Purpose:  Calls the MouseDown method of CSplitter. Also sets the bounding
'   *               rect so the splitter is bounded to the panes set (the panes to the left
'   *               and right of the splitter).
'   *********************************************************************************
    If Not oPane Is Nothing Then
        '   ***** This code block sets the boundaries for the current set of panes (opane and opane + 1)
        '   *****This allows the inverted rect to move only within the current pane set
        If m_eOrientation = gtHorizontal Then
            m_CSplitter.ParentWidth = Me(oPane.Index + 1).InternalLeft + Me(oPane.Index + 1).InternalWidth - oPane.InternalLeft
            m_CSplitter.ParentLeft = oPane.InternalLeft
        Else
            m_CSplitter.ParentHeight = Me(oPane.Index + 1).InternalTop + Me(oPane.Index + 1).InternalHeight - oPane.InternalTop
            m_CSplitter.ParentTop = oPane.InternalTop
        End If
        '   *****
       m_CSplitter.MouseDown 0, 0, x, y
    End If
End Sub
Friend Sub MouseMove(oPane As GTPane, x As Single, y As Single)
'   *********************************************************************************
'   * Params:   oPane: the pane that the splitter is to the right of, X: the x point
'   *               of the mouse, Y: the y point of the mouse
'   * Purpose:  Calls the MouseMove method of CSplitter.
'   *********************************************************************************
    If Not oPane Is Nothing Then
        m_CSplitter.MouseMove 0, 0, x, y
    End If
End Sub
Friend Sub MouseUp(oPane As GTPane)
'   *********************************************************************************
'   * Params:   oPane: the pane that is to be resized
'   * Purpose:  Calls the MouseMove method of CSplitter. Also calculates the
'   *               size of the param pane and the pane + 1. Also fires the SplitterResized
'   *               event.
'   *********************************************************************************
    If m_CSplitter.MouseUp Then
'   ***** Don't do anything if the splitter wasn't move. Will be -1 when not moved
        If m_CSplitter.Value = -1 Then
            Exit Sub
        End If
        
'   ***** The pane set is resized depending on the value of the splitter. Everything is checked, including
'   ***** the minimum for both panes of the set, to resize the panes. The Me key word is used to get the
'   ***** "internal" properties because they are friend props.
        If m_eOrientation = gtHorizontal Then
            If m_CSplitter.Value - oPane.InternalLeft < 0 Then
                If oPane.InternalMinimum = 0 Then
                    Me(oPane.Index + 1).InternalWidth = Me(oPane.Index + 1).InternalWidth + oPane.InternalWidth - 1 ' set to 1 when zero
                    oPane.InternalWidth = 1 ' set to 1 when zero
                Else
                    Me(oPane.Index + 1).InternalWidth = Me(oPane.Index + 1).InternalWidth + oPane.InternalWidth - oPane.InternalMinimum
                    oPane.InternalWidth = oPane.InternalMinimum
                End If
            ElseIf m_CSplitter.Value - oPane.InternalLeft <= oPane.InternalMinimum Then
                Me(oPane.Index + 1).InternalWidth = Me(oPane.Index + 1).InternalWidth + oPane.InternalWidth - oPane.InternalMinimum
                oPane.InternalWidth = oPane.InternalMinimum
            ElseIf oPane.InternalWidth > m_CSplitter.Value - oPane.InternalLeft Then
                Me(oPane.Index + 1).InternalWidth = Me(oPane.Index + 1).InternalWidth - ((m_CSplitter.Value - oPane.InternalLeft) - oPane.InternalWidth)
                oPane.InternalWidth = m_CSplitter.Value - oPane.InternalLeft
            ElseIf oPane.InternalWidth + Me(oPane.Index + 1).InternalWidth + m_lSplitterSize < m_CSplitter.Value - oPane.InternalLeft Then
                oPane.InternalWidth = oPane.InternalWidth + Me(oPane.Index + 1).InternalWidth - oPane.InternalMinimum
                Me(oPane.Index + 1).InternalWidth = oPane.InternalMinimum
            ElseIf m_CSplitter.Value > m_lScaleWidth And Me(oPane.Index + 1).InternalMinimum = 0 Then
                Me(oPane.Index + 1).InternalWidth = m_lSplitterSize + 1
                oPane.InternalWidth = m_CSplitter.Value - oPane.InternalLeft - (m_lSplitterSize + 1)
            ElseIf oPane.InternalWidth < m_CSplitter.Value - oPane.InternalLeft Then
                If Me(oPane.Index + 1).InternalWidth + (oPane.InternalWidth - (m_CSplitter.Value - oPane.InternalLeft)) < Me(oPane.Index + 1).InternalMinimum Then
                    If Not (Me(oPane.Index + 1).InternalWidth = Me(oPane.Index + 1).InternalMinimum) Then
                        Me(oPane.Index + 1).InternalWidth = Me(oPane.Index + 1).InternalMinimum
                        oPane.InternalWidth = m_CSplitter.Value - oPane.InternalLeft - Me(oPane.Index + 1).InternalMinimum
                    End If
                Else
                    Me(oPane.Index + 1).InternalWidth = Me(oPane.Index + 1).InternalWidth + (oPane.InternalWidth - (m_CSplitter.Value - oPane.InternalLeft))
                    oPane.InternalWidth = m_CSplitter.Value - oPane.InternalLeft
                End If
            End If
        Else
            If m_CSplitter.Value - oPane.InternalTop < 0 Then
                If oPane.InternalMinimum = 0 Then
                    Me(oPane.Index + 1).InternalHeight = Me(oPane.Index + 1).InternalHeight + oPane.InternalHeight - 1 ' set to 1 when zero
                    oPane.InternalHeight = 1 ' set to 1 when zero
                Else
                    Me(oPane.Index + 1).InternalHeight = Me(oPane.Index + 1).InternalHeight + oPane.InternalHeight - oPane.InternalMinimum
                    oPane.InternalHeight = oPane.InternalMinimum
                End If
            ElseIf m_CSplitter.Value - oPane.InternalTop <= oPane.InternalMinimum Then
                Me(oPane.Index + 1).InternalHeight = Me(oPane.Index + 1).InternalHeight + oPane.InternalHeight - oPane.InternalMinimum
                oPane.InternalHeight = oPane.InternalMinimum
            ElseIf oPane.InternalHeight > m_CSplitter.Value - oPane.InternalTop Then
                Me(oPane.Index + 1).InternalHeight = Me(oPane.Index + 1).InternalHeight - ((m_CSplitter.Value - oPane.InternalTop) - oPane.InternalHeight)
                oPane.InternalHeight = m_CSplitter.Value - oPane.InternalTop
            ElseIf oPane.InternalHeight + Me(oPane.Index + 1).InternalHeight + m_lSplitterSize < m_CSplitter.Value - oPane.InternalTop Then
                oPane.InternalHeight = oPane.InternalHeight + Me(oPane.Index + 1).InternalHeight - oPane.InternalMinimum
                Me(oPane.Index + 1).InternalHeight = oPane.InternalMinimum
            ElseIf m_CSplitter.Value > m_lScaleHeight And Me(oPane.Index + 1).InternalMinimum = 0 Then
                Me(oPane.Index + 1).InternalHeight = m_lSplitterSize + 1
                oPane.InternalHeight = m_CSplitter.Value - oPane.InternalTop - (m_lSplitterSize + 1)
            ElseIf oPane.InternalHeight < m_CSplitter.Value - oPane.InternalTop Then
                If Me(oPane.Index + 1).InternalHeight + (oPane.InternalHeight - (m_CSplitter.Value - oPane.InternalTop)) < Me(oPane.Index + 1).InternalMinimum Then
                    If Not (Me(oPane.Index + 1).InternalHeight = Me(oPane.Index + 1).InternalMinimum) Then
                        Me(oPane.Index + 1).InternalHeight = Me(oPane.Index + 1).InternalMinimum
                        oPane.InternalHeight = m_CSplitter.Value - oPane.InternalTop - Me(oPane.Index + 1).InternalMinimum
                    End If
                Else
                    Me(oPane.Index + 1).InternalHeight = Me(oPane.Index + 1).InternalHeight + (oPane.InternalHeight - (m_CSplitter.Value - oPane.InternalTop))
                    oPane.InternalHeight = m_CSplitter.Value - oPane.InternalTop
                End If
            End If
        End If
    End If
'   *****

    RaiseEvent SplitterResized(oPane, m_colPanes(oPane.Index + 1))
    
End Sub
Friend Sub Move(lLeft As Long, lTop As Long, lScaleWidth As Long, lScaleHeight As Long)
'   *********************************************************************************
'   * Params:   lLeft: the left of the user control, lTop: the top of the user control,
'   *               lScaleWidth: the scalewidth of the control- the offset, lScaleHeight:
'   *               the ScaleHeight of the control - the offset.
'   * Purpose: Called when the control resizes to reset the variables.Also resizes
'   *               the last pane.
'   *********************************************************************************
    m_lLeft = lLeft
    m_lTop = lTop
    m_lScaleWidth = lScaleWidth
    m_lScaleHeight = lScaleHeight
    
    m_CSplitter.ParentWidth = lScaleWidth + m_iOffset * 2
    m_CSplitter.ParentHeight = lScaleHeight + m_iOffset * 2
  
    If m_eOrientation = gtHorizontal Then
        m_CSplitter.Height = m_lScaleHeight + m_iOffset * 2
        m_CSplitter.Width = m_lSplitterSize
    Else
        m_CSplitter.Width = m_lScaleWidth + m_iOffset * 2
        m_CSplitter.Height = m_lSplitterSize
    End If
    
    ResizeLastPane False
    
End Sub

Friend Function GetPaneLeft(oTargetPane As GTPane) As Long
'   *********************************************************************************
'   * Params:   oTargetPane: The pane to get the left for.
'   * Return:   Long: the left of the pane.
'   * Purpose:  Loops thru the panes calculating the left for the target pane.
'   *********************************************************************************
    Dim oPane As GTPane
    Dim lLeft As Long
        
    If m_eOrientation = gtVertical Then
        GetPaneLeft = m_iOffset
        Exit Function
    End If
    
    lLeft = m_iOffset
    
    For Each oPane In m_colPanes
        If oPane Is oTargetPane Then
            GetPaneLeft = lLeft
            Exit Function
        End If
'   ***** Always add the width of the splitter
        lLeft = lLeft + oPane.InternalWidth + m_lSplitterSize
'   *****
    Next
    
    GetPaneLeft = 0
End Function

Friend Function GetPaneTop(oTargetPane As GTPane) As Long
'   *********************************************************************************
'   * Params:   oTargetPane: The pane to get the top for.
'   * Return:   Long: the top of the pane.
'   * Purpose:  Loops thru the panes calculating the top for the target pane.
'   *********************************************************************************
    Dim oPane As GTPane
    Dim lTop As Long
    
    If m_eOrientation = gtHorizontal Then
        GetPaneTop = m_iOffset
        Exit Function
    End If
    
    lTop = m_iOffset
    
    For Each oPane In m_colPanes
        If oPane Is oTargetPane Then
            GetPaneTop = lTop
            Exit Function
        End If
'   ***** Always add the height of the splitter
        lTop = lTop + oPane.InternalHeight + m_lSplitterSize
'   *****
    Next
    
    GetPaneTop = 0
End Function

Friend Function GetPaneWidth() As Long
'   *********************************************************************************
'   * Return:   Long: the width of the pane.
'   * Purpose:  Called when Orientation is Vertical.
'   *********************************************************************************
    GetPaneWidth = m_lScaleWidth
End Function
Friend Function GetPaneHeight() As Long
'   *********************************************************************************
'   * Return:   Long: the height of the pane.
'   * Purpose:  Called when Orientation is Horizontal.
'   *********************************************************************************
    GetPaneHeight = m_lScaleHeight
End Function

Friend Sub ResizeLastPane(bFireSplitterResize As Boolean)
'   *********************************************************************************
'   * Params:   bFireSplitterResize: whether or not to fire the SplitterResize
'   *               event.
'   * Purpose:  Resizes the last pane to fill the rest of the user control. This
'   *               needs to be done or else there would be gaps
'   *********************************************************************************
    If m_colPanes.Count = 0 Then
        Exit Sub
    End If
    
    If m_eOrientation = gtHorizontal Then
        If (m_lScaleWidth + m_iOffset) - Me(m_colPanes.Count).InternalLeft > Me(m_colPanes.Count).InternalMinimum Then
            Me(m_colPanes.Count).InternalWidth = (m_lScaleWidth + m_iOffset) - Me(m_colPanes.Count).InternalLeft
        Else ' set to the minimum
            Me(m_colPanes.Count).InternalWidth = Me(m_colPanes.Count).InternalMinimum
        End If
    Else
        If (m_lScaleHeight + m_iOffset) - Me(m_colPanes.Count).InternalTop > Me(m_colPanes.Count).InternalMinimum Then
            Me(m_colPanes.Count).InternalHeight = (m_lScaleHeight + m_iOffset) - Me(m_colPanes.Count).InternalTop
        Else ' set to the minimum
            Me(m_colPanes.Count).InternalHeight = Me(m_colPanes.Count).InternalMinimum
        End If
    End If
    
'   ***** Only resize when the flag is true and more than 1 pane. The flag is for resizing the last pane
'   ***** in code and not firing the SplitterResize event
    If bFireSplitterResize And m_colPanes.Count > 1 Then
        RaiseEvent SplitterResized(m_colPanes(m_colPanes.Count - 1), m_colPanes(m_colPanes.Count))
    End If
'   *****
End Sub

Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the object to remove
'   * Purpose:  Removes an object from the collection.
'   ***********************************************************************
    Dim lIndex As Long
    Dim oDeletedPane As GTPane
    
    lIndex = ConvertIndex(Index)
    
    Set oDeletedPane = Me(lIndex)
    
    m_colPanes.Remove lIndex
    
    ReIndexPanes
    
    ResizeLastPane True
    
    RaiseEvent PaneRemoved(oDeletedPane)

End Sub

Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oPane As GTPane
    
    For Each oPane In m_colPanes
        If oPane.Key = vIndex Then
            GetObjectIndexFromKey = oPane.Index
            Exit Function
        End If
    Next
    
End Function

Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all objects
'   ***********************************************************************
    Set m_colPanes = New Collection
    RaiseEvent PaneRemoved(Nothing)
End Sub

Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oPane As GTPane
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oPane In m_colPanes
        If StrComp(vTempKey, Trim(oPane.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function
