VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CDrawRect"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'   ******************************************************************************
'   * Purpose:  Handles drawing the any 3D rects that need to be draw
'   * File:         CDrawRect.cls
'   ******************************************************************************
Option Explicit

'  point type for use with the apis
Private Type POINTAPI
        x As Long
        y As Long
End Type

'   ***** pen style  constants
Private Const PS_SOLID = 0
'   *****
'   ***** system color constants
Private Const COLOR_3DLIGHT = 22
Private Const COLOR_3DDKSHADOW = 21
Private Const COLOR_3DFACE = 15
Private Const COLOR_3DHIGHLIGHT = 20
Private Const COLOR_3DSHADOW = 16
Private Const COLOR_BTNHIGHLIGHT = 20
Private Const COLOR_HIGHLIGHT = 13
Private Const COLOR_BTNFACE = 15
Private Const COLOR_BTNSHADOW = 16
Private Const COLOR_BTNTEXT = 18
Private Const COLOR_WINDOW = 5
'   *****
'   ***** the declarations used for this class
Private Declare Function RestoreDC Lib "gdi32" (ByVal hdc As Long, ByVal nSavedDC As Long) As Long
Private Declare Function SaveDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function CreatePen Lib "gdi32" (ByVal nPenStyle As Long, ByVal nWidth As Long, ByVal crColor As Long) As Long
Private Declare Function LineTo Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long) As Long
Private Declare Function MoveToEx Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long, lpPoint As POINTAPI) As Long
Private Declare Function GetSysColor Lib "user32" (ByVal nIndex As Long) As Long
Private Declare Function SetTextColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function GetBkColor Lib "gdi32" (ByVal hdc As Long) As Long
'   *****
'   ***** enum for the rect types
Public Enum Rect3DConstants
    Inset
    Raised
    BtnDown
    FixedSingle
    ActiveUp
    ActiveDown
    ActiveFlat
    ToolbarActive
    ToolbarActiveDown
    ToolbarSeparatorV
    ToolbarSeparatorH
End Enum
'   *****
'   ***** enum for the orientation (not used for all controls)
Public Enum RectOrientationConstants
    Horizontal
    Vertical
End Enum

Public Sub Draw3DRect(hDCDest As Long, lLeft As Long, lTop As Long, lWidth As Long, lHeight As Long, eRect3D As Rect3DConstants)
'   ******************************************************************************
'   * Params:   hDCDest: the device context to draw on, lLeft: the boundary of the rect,
'   *               lTop: the top boundary of the rect, lWidth: the width of the rect, lHeight:
'   *               the height of the rect, eRect3D: the type of rect to draw
'   * Purpose: Draws a 3D rectangle within the specified rect
'   ******************************************************************************
    Dim lRet As Long ' the return value
    Dim lSavePen As Long ' the pen that was previously selected into an object
    Dim pt As POINTAPI
    Dim lBROuter As Long ' the pen for the bottom and right of the outer rect
    Dim lLTOuter As Long ' the pen for the left and top of the outer rect
    Dim lLTInner As Long ' the pen for the left and top of the inner rect
    Dim lBRInner As Long ' the pen for the bottom and right of the inner rect
    Dim lLightColor As OLE_COLOR ' the light color
    Dim lShadow As OLE_COLOR ' the shadow color
    Dim lHighlight As OLE_COLOR ' the highlight color
    Dim lDarkShadow As OLE_COLOR ' the darkest color (usually black)
    
    lRet = SaveDC(hDCDest) ' save the dc info
    
    If eRect3D = ActiveFlat Then ' draws a border using the backcolor so it's not seen
        lShadow = GetBkColor(hDCDest)
    Else ' set the colors for each color here
        lShadow = GetSysColor(COLOR_3DSHADOW)
        lHighlight = GetSysColor(COLOR_3DHIGHLIGHT)
        lDarkShadow = GetSysColor(COLOR_3DDKSHADOW)
        lLightColor = GetSysColor(COLOR_3DLIGHT)
    End If
    
    Select Case eRect3D ' set the pens for each enum
        Case Raised ' four colors in a raised position, like a command button
            lBRInner = CreatePen(PS_SOLID, 1, lShadow)
            lLTOuter = CreatePen(PS_SOLID, 1, lHighlight)
            lBROuter = CreatePen(PS_SOLID, 1, lDarkShadow)
            lLTInner = CreatePen(PS_SOLID, 1, lLightColor)
        Case Inset ' four color inset like the Appearance property of a control
            lLTOuter = CreatePen(PS_SOLID, 1, lShadow)
            lBROuter = CreatePen(PS_SOLID, 1, lHighlight)
            lLTInner = CreatePen(PS_SOLID, 1, lDarkShadow)
            lBRInner = CreatePen(PS_SOLID, 1, lLightColor)
        Case BtnDown ' draws a rect using the shadow color
            lBRInner = CreatePen(PS_SOLID, 1, lLightColor)
            lLTOuter = CreatePen(PS_SOLID, 1, lShadow)
            lBROuter = CreatePen(PS_SOLID, 1, lShadow)
            lLTInner = CreatePen(PS_SOLID, 1, lLightColor)
        Case FixedSingle ' draws a rect using only black
            lLightColor = GetBkColor(hDCDest) ' gets the backcolor of the dc so light draws invisible
            
            lBRInner = CreatePen(PS_SOLID, 1, lLightColor)
            lLTOuter = CreatePen(PS_SOLID, 1, 0)
            lBROuter = CreatePen(PS_SOLID, 1, 0)
            lLTInner = CreatePen(PS_SOLID, 1, GetBkColor(hDCDest))
        Case ActiveUp ' draws 2 color rect in a raised position
            lBRInner = CreatePen(PS_SOLID, 1, lShadow)
            lLTOuter = CreatePen(PS_SOLID, 1, lHighlight)
            lBROuter = CreatePen(PS_SOLID, 1, lDarkShadow)
            lLTInner = CreatePen(PS_SOLID, 1, lLightColor)
        Case ActiveDown ' draws 2 color rect inset
            lBRInner = CreatePen(PS_SOLID, 1, lShadow)
            lLTOuter = CreatePen(PS_SOLID, 1, lDarkShadow)
            lBROuter = CreatePen(PS_SOLID, 1, lHighlight)
            lLTInner = CreatePen(PS_SOLID, 1, lLightColor)
        Case ActiveFlat ' draws 2 color rect invisible to background (uses the backcolor)
            lBRInner = CreatePen(PS_SOLID, 1, lShadow)
            lLTOuter = CreatePen(PS_SOLID, 1, lShadow)
            lBROuter = CreatePen(PS_SOLID, 1, lShadow)
            lLTInner = CreatePen(PS_SOLID, 1, lShadow)
        Case ToolbarActive, ToolbarSeparatorV, ToolbarSeparatorH ' draws 2 color rect or lines
            lBRInner = CreatePen(PS_SOLID, 1, lShadow)
            lLTOuter = CreatePen(PS_SOLID, 1, lHighlight)
            lBROuter = CreatePen(PS_SOLID, 1, lShadow)
            lLTInner = CreatePen(PS_SOLID, 1, lLightColor)
         Case ToolbarActiveDown ' draws 2 color rect inset
            lBRInner = CreatePen(PS_SOLID, 1, lHighlight)
            lLTOuter = CreatePen(PS_SOLID, 1, lShadow)
            lBROuter = CreatePen(PS_SOLID, 1, lHighlight)
            lLTInner = CreatePen(PS_SOLID, 1, lLightColor)
       
    End Select
    
'   ***** draws depending on the enum
    Select Case eRect3D
    
        Case ToolbarSeparatorV ' draws a toolbar separator (2 vertical lines) inset
            lSavePen = SelectObject(hDCDest, lBROuter) ' selects a pen and holds the replaced pen
        
            ' draws the shadow line
            MoveToEx hDCDest, lLeft, lTop + lHeight, pt
            LineTo hDCDest, lLeft, lTop
            
            lRet = SelectObject(hDCDest, lLTOuter) ' change pens
            
            ' draws the highlight line
            MoveToEx hDCDest, lLeft + 1, lTop + lHeight, pt
            LineTo hDCDest, lLeft + 1, lTop
            
        Case ToolbarSeparatorH ' draws a toolbar separator (2 horizontal lines) inset
            lSavePen = SelectObject(hDCDest, lBROuter) ' selects a pen and holds the replaced pen
        
            ' draws the shadow line
            MoveToEx hDCDest, lLeft + lWidth, lTop, pt
            LineTo hDCDest, lLeft, lTop
            
            lRet = SelectObject(hDCDest, lLTOuter) ' change pens
            
            ' draws the highlight line
            MoveToEx hDCDest, lLeft + lWidth, lTop + 1, pt
            LineTo hDCDest, lLeft, lTop + 1
        
        Case Else
            lSavePen = SelectObject(hDCDest, lLTOuter) ' selects a pen and holds the replaced pen
        
            ' draws the top outer line
            If lWidth < 2 Then ' when the width < 2 don't subtract the offset: becomes negative
                MoveToEx hDCDest, lLeft + lWidth, lTop, pt
            Else
                MoveToEx hDCDest, lLeft + lWidth - 2, lTop, pt
            End If
            LineTo hDCDest, lLeft, lTop
            
            ' draws the left outer line
            MoveToEx hDCDest, lLeft, lTop, pt
            LineTo hDCDest, lLeft, lTop + lHeight - 1
            
            lRet = SelectObject(hDCDest, lBROuter) ' change pens
            
            ' draws the bottom outer line
            MoveToEx hDCDest, lLeft, lTop + lHeight - 1, pt
            LineTo hDCDest, lLeft + lWidth - 1, lTop + lHeight - 1
            
            ' draws the right outer line
            MoveToEx hDCDest, lLeft + lWidth - 1, lTop + lHeight - 1, pt
            LineTo hDCDest, lLeft + lWidth - 1, lTop - 1
            
            
            Select Case eRect3D
                Case FixedSingle, ActiveUp, ActiveDown, ActiveFlat, ToolbarActive, ToolbarActiveDown
                    ' don't do anything, don't need the inner lines drawn
                Case Else
                    lRet = SelectObject(hDCDest, lLTInner) ' change pens
                    
                    ' draws the inner top line
                    If lWidth < 3 Then ' when the width < 3 don't subtract the offset: becomes negative
                        MoveToEx hDCDest, lLeft + lWidth, lTop + 1, pt
                    Else
                        MoveToEx hDCDest, lLeft + lWidth - 3, lTop + 1, pt
                    End If
                    LineTo hDCDest, lLeft + 1, lTop + 1
                    
                    ' draws the inner left line
                    MoveToEx hDCDest, lLeft + 1, lTop + 1, pt
                    LineTo hDCDest, lLeft + 1, lTop + lHeight - 2
                    
                    lRet = SelectObject(hDCDest, lBRInner) ' change pens
                
                    ' draws the inner bottom line
                    MoveToEx hDCDest, lLeft + 1, lTop + lHeight - 2, pt
                    If lWidth < 2 Then ' when the width < 2 don't subtract the offset: becomes negative
                        LineTo hDCDest, lLeft + lWidth, lTop + lHeight - 2
                    Else
                        LineTo hDCDest, lLeft + lWidth - 2, lTop + lHeight - 2
                    End If
                
                    ' draws the inner right line
                    If lWidth < 2 Then ' when the width < 2 don't subtract the offset: becomes negative
                        MoveToEx hDCDest, lLeft + lWidth, lTop + lHeight - 2, pt
                        LineTo hDCDest, lLeft + lWidth, lTop
                    Else
                        MoveToEx hDCDest, lLeft + lWidth - 2, lTop + lHeight - 2, pt
                        LineTo hDCDest, lLeft + lWidth - 2, lTop
                    End If
            
            End Select
    
    End Select
    
    lRet = SelectObject(hDCDest, lSavePen) ' reselect the original pen
    
'   ***** delete the pens that were created
    lRet = DeleteObject(lLTOuter)
    lRet = DeleteObject(lBROuter)
    lRet = DeleteObject(lLTInner)
    lRet = DeleteObject(lBRInner)
'   *****

    lRet = RestoreDC(hDCDest, -1) ' restore the dc to it original state
    
End Sub

Public Sub DrawRoundSplitter(hdc As Long, eOrientation As RectOrientationConstants, lSplitterSize As Long, iOffset As Integer, lLeft As Long, lTop As Long, lWidth As Long, lHeight As Long)
'   *****************************************************************
'   * Params:   hdc: the hdc to draw on, eOrientation: horizontal or vertical, lSplitterSize: the width/height of the area to draw on,
'   *               iOffset: the left or top depending on the orientation, lLeft: the left of the rect, lTop: the top of the rect,
'   *               lWidth: the width of the rect, lHeight: the height of the rect
'   * Purpose:  Used with the splitter control to draw a rounded looking splitter.
'   *****************************************************************
    Dim i As Integer
    Dim hFacePen As Long ' the pen with the face color
    Dim hShadowPen As Long ' the pen with the shadow color
    Dim hHighlightPen As Long ' the pen with the highlight color
    Dim hPen As Long ' the pen that is used to do the drawing
    Dim hOldPen As Long ' the original pen in the dc that will be reselected when finished
    Dim pt As POINTAPI
    
'   ***** create the pens required for drawing
    hShadowPen = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DSHADOW))
    hHighlightPen = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DHIGHLIGHT))
    hFacePen = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DFACE))
'   *****

    hOldPen = SelectObject(hdc, hShadowPen) ' select the first pen into the object and hold the old pen
    
    For i = 0 To lSplitterSize - 1 ' loop thru the width/height of the area to draw on
'        If lSplitterSize Mod 3 = 0 Then
'            Select Case i
'                Case 0:
'                    hPen = hHighlightPen
'                Case lSplitterSize - 1:
'                    hPen = hShadowPen
'                Case Else:
'                    hPen = hFacePen
'            End Select
'        Else
            Select Case i ' depending on the loop iteration, decides which pen to use
                Case 0:
                    hPen = hHighlightPen
                Case lSplitterSize - 1:
                    hPen = hShadowPen
                Case Else:
                    hPen = hFacePen
            End Select
'        End If
        
        SelectObject hdc, hPen ' select the object need for drawing the line
        
        If eOrientation = Horizontal Then ' draw lines vertical
            MoveToEx hdc, lLeft + lWidth + i, iOffset + lHeight, pt
            LineTo hdc, lLeft + lWidth + i, iOffset
        Else ' draw lines horizontal
            MoveToEx hdc, iOffset + lWidth, lTop + lHeight + i, pt
            LineTo hdc, iOffset, lTop + lHeight + i
        End If
        
    Next
    
    SelectObject hdc, hOldPen ' reselect the original pen
    
'   ***** free the objects that were created
    DeleteObject hShadowPen
    DeleteObject hHighlightPen
    DeleteObject hFacePen
'   *****
End Sub

