VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTButton"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  This is the GTButton object and is used in the Buttons collection
'   * File:         GTButton.cls
'   ******************************************************************************
Option Explicit

Private m_oButtons As GTButtons
Private m_sCaption As String
Private m_sDescription As String
Private m_sToolTipText As String
Private m_bEnabled As Boolean
Private m_bVisible As Boolean
Private m_vKey As Variant
Private m_vSmallPic As Variant
Private m_vLargePic As Variant
Private m_vTag As Variant
Private m_eStyle As GTButtonStyleConstants
Private m_eValue As GTButtonValueConstants
Private m_lIndex As Long
Private m_lLeft As Long
Private m_lTop As Long
Private m_lWidth As Long
Private m_lHeight As Long
Private m_bActive As Boolean
Private m_bDown As Boolean

'   ***** Down is set when the mouse is down on a button
Friend Property Let Down(bDown As Boolean) ' sets the value of down  (internal use only)
    m_bDown = bDown
End Property
Friend Property Get Down() As Boolean ' returns the value of down (internal use only)
    Down = m_bDown
End Property
'   *****

'   ***** Active is set when the mouse over the button and is over the button
Friend Property Let Active(bActive As Boolean) ' set the value of Active (internal use only)
    m_bActive = bActive
End Property
Friend Property Get Active() As Boolean ' returns the value of Active (Internal use only)
    Active = m_bActive
End Property
'   *****

'   ***** holds a reference to the collection so the ChildChanged proc can be called when a change occurs
Friend Property Set Buttons(oButtons As GTButtons)
    Set m_oButtons = oButtons
End Property
'   *****

'   ***** when ButtonStyle is Pictures and Text or Text Only, the Caption will be draw on the button
Public Property Let Caption(sCaption As String)
    m_sCaption = sCaption
    m_oButtons.ChildChanged Me, "Caption"
End Property
Public Property Get Caption() As String
Attribute Caption.VB_Description = "Returns/sets the text displayed in an object's title bar or below an object's icon."
Attribute Caption.VB_HelpID = 21
Attribute Caption.VB_UserMemId = 0
    Caption = m_sCaption
End Property
'   *****

'   ***** Can be used when using a status bar
Public Property Let Description(sDescription As String)
Attribute Description.VB_Description = "Returns/sets the description displayed when the user clicks a Button object during a customization operation."
Attribute Description.VB_HelpID = 28
    If m_sDescription = sDescription Then
        Exit Property
    End If
    
    m_sDescription = sDescription
    m_oButtons.ChildChanged Me, "Description"
End Property
Public Property Get Description() As String
    Description = m_sDescription
End Property
'   *****

'   ***** the text that shows in the tooltip
Public Property Let ToolTipText(sToolTipText As String)
Attribute ToolTipText.VB_Description = "Returns/sets the string displayed when a cursor hovers over an object."
Attribute ToolTipText.VB_HelpID = 91
    If m_sToolTipText = sToolTipText Then
        Exit Property
    End If
    m_sToolTipText = sToolTipText
    m_oButtons.ChildChanged Me, "ToolTipText"
End Property
Public Property Get ToolTipText() As String
    ToolTipText = m_sToolTipText
End Property
'   *****

'   ***** If Enabled is False, the image and any text will be draw disabled
Public Property Let Enabled(bEnabled As Boolean)
    If m_bEnabled = bEnabled Then
        Exit Property
    End If
    m_bEnabled = bEnabled
    m_oButtons.ChildChanged Me, "Enabled"
End Property
Public Property Get Enabled() As Boolean
Attribute Enabled.VB_Description = "Returns/sets a value that determines whether a form or control can respond to user-generated events."
Attribute Enabled.VB_HelpID = 18
Attribute Enabled.VB_UserMemId = -514
    Enabled = m_bEnabled
End Property
'   *****

'   ***** Returns/sets whether the button will be visible or not
Public Property Let Visible(bVisible As Boolean)
Attribute Visible.VB_Description = "Returns/sets a value that determines whether an object is visible or hidden."
Attribute Visible.VB_HelpID = 96
    If m_bVisible = bVisible Then
        Exit Property
    End If
    m_bVisible = bVisible
    m_oButtons.ChildChanged Me, "Visible"
End Property
Public Property Get Visible() As Boolean
    Visible = m_bVisible
End Property
'   *****
'   **** used to set the value of index (for internal use only)
Friend Property Let InternalIndex(lIndex As Long)
Attribute InternalIndex.VB_MemberFlags = "40"
    m_lIndex = lIndex
End Property
'   *****

'   ***** returns the the value of Index
Public Property Get Index() As Long
Attribute Index.VB_Description = "Returns/sets the index of an object in a collection. Read-only at run time."
Attribute Index.VB_HelpID = 46
    Index = m_lIndex
End Property
'   *****

'   ***** Returns/sets the value of Key. This value must be unique or an error is raised
Public Property Let Key(vKey As Variant)
Attribute Key.VB_Description = "Returns/sets the unique string of an object in a collection."
Attribute Key.VB_HelpID = 48
    If m_oButtons.IsDuplicateKey(vKey) Then
        Err.Raise vbObjectError + 1051, , "Key already exists."
        Exit Property
    End If
    
    If m_vKey = vKey Then
        Exit Property
    End If
    
    m_vKey = vKey
    m_oButtons.ChildChanged Me, "Key"
End Property
Public Property Get Key() As Variant
    Key = m_vKey
End Property
'   *****

'   ***** Returns/Sets any extra info for the button
'   ***** Ver 1.0a Added Set
Public Property Set Tag(vTag As Variant)
    
    Set m_vTag = vTag
    m_oButtons.ChildChanged Me, "Tag"
End Property
'   *****
Public Property Let Tag(vTag As Variant)
Attribute Tag.VB_Description = "Stores any extra data needed for your program."
Attribute Tag.VB_HelpID = 87
'   ***** ver 1.0a Have to check if and object before checking the value
    If Not IsObject(m_vTag) Then
        If m_vTag = vTag Then
            Exit Property
        End If
    End If
'   *****
    m_vTag = vTag
    m_oButtons.ChildChanged Me, "Tag"
End Property
Public Property Get Tag() As Variant
'   ***** Ver 1.0a Added to check for an object
    If IsObject(m_vTag) Then
        Set Tag = m_vTag
    Else
        Tag = m_vTag
    End If
'   *****
End Property
'   *****

'   ***** Returns/Sets the value of LargePicture. This is the index or key of an image in the LargeImages collection
Public Property Let LargePicture(vLargePic As Variant)
Attribute LargePicture.VB_Description = "Returns/sets the index or key of a ListImage object to be used when the View is Large."
Attribute LargePicture.VB_HelpID = 51
    If IsEmpty(vLargePic) Then
        Err.Raise vbObjectError + 1052, App.Title, "LargePicture must have a value."
        Exit Property
    ElseIf Trim(vLargePic) = "" Then
        Err.Raise vbObjectError + 1052, App.Title, "LargePicture must have a value."
        Exit Property
    End If
    
    If m_vLargePic = vLargePic Then
        Exit Property
    End If
    
    m_vLargePic = vLargePic
    m_oButtons.ChildChanged Me, "LargePicture"
End Property
Public Property Get LargePicture() As Variant
    LargePicture = m_vLargePic
End Property
'   *****
'   ***** Returns/Sets the value of SmallPicture. This is the index or key of an image in the SmallImages collection
Public Property Let SmallPicture(vSmallPic As Variant)
Attribute SmallPicture.VB_Description = "Returns/sets the index or key of a ListImage object to be used when the View is Small."
Attribute SmallPicture.VB_HelpID = 76
    If IsEmpty(vSmallPic) Then
        Err.Raise vbObjectError + 1052, App.Title, "SmallPicture must have a value."
        Exit Property
    ElseIf Trim(vSmallPic) = "" Then
        Err.Raise vbObjectError + 1052, App.Title, "SmallPicture must have a value."
        Exit Property
    End If
    
    If m_vSmallPic = vSmallPic Then
        Exit Property
    End If
    
    m_vSmallPic = vSmallPic
    m_oButtons.ChildChanged Me, "SmallPicture"
End Property
Public Property Get SmallPicture() As Variant
    SmallPicture = m_vSmallPic
End Property
'   *****

'   ***** returns the value of left in Pixels in twips. Read Only.
Public Property Get Left() As Long
Attribute Left.VB_Description = "Returns/sets the distance between the internal left edge of an object and the left edge of its container."
Attribute Left.VB_HelpID = 52
    Left = m_oButtons.GetButtonPt(Me).x * Screen.TwipsPerPixelX
End Property
'   *****
'   ***** returns the value of top in twips. Read Only.
Public Property Get Top() As Long
Attribute Top.VB_Description = "Returns/sets the distance between the internal top edge of an object and the top edge of its container."
Attribute Top.VB_HelpID = 92
    Top = m_oButtons.GetButtonPt(Me).y * Screen.TwipsPerPixelY
End Property
'   *****

'   ***** Returns/Sets the value of width in twips. This is only used for placeholders.
Public Property Let Width(lWidth As Long)
Attribute Width.VB_Description = "Returns/sets the width of an object."
Attribute Width.VB_HelpID = 98
    m_lWidth = lWidth \ Screen.TwipsPerPixelX
    m_oButtons.ChildChanged Me, "Width"
End Property
Public Property Get Width() As Long
    Width = m_lWidth * Screen.TwipsPerPixelX
End Property
'   *****

'   ***** Returns the value of height in twips. Read Only.
Public Property Get Height() As Long
Attribute Height.VB_Description = "Returns/sets the height of an object."
Attribute Height.VB_HelpID = 40
    Height = m_oButtons.ButtonHeight * Screen.TwipsPerPixelY
End Property
'   *****

'   ***** Returns/Sets the value constant of Style. This can be gtDefault, gtToggle, etc.
Public Property Let Style(eStyle As GTButtonStyleConstants)
Attribute Style.VB_Description = "Returns/sets the button style"
Attribute Style.VB_HelpID = 83
    If m_eStyle = eStyle Then
        Exit Property
    End If
    m_eStyle = eStyle
    m_oButtons.ChildChanged Me, "Style"
End Property
Public Property Get Style() As GTButtonStyleConstants
    Style = m_eStyle
End Property
'   *****

'   ***** Return/sets the value constant for Value. This is gtPressed (down) or gtUnpressed (up)
Public Property Let Value(eValue As GTButtonValueConstants)
Attribute Value.VB_Description = "Returns/sets the value of an object."
Attribute Value.VB_HelpID = 94
    If m_eValue = eValue Then
        Exit Property
    End If
    m_eValue = eValue
    m_oButtons.ChildChanged Me, "Value"
End Property
Public Property Get Value() As GTButtonValueConstants
    Value = m_eValue
End Property
'   *****

Private Sub Class_Initialize()
'   ****************************************************
'   * Purpose:  Initializes the properties associated with the object
'   ****************************************************
    m_bVisible = True
    m_bEnabled = True
    m_vLargePic = 0
    m_vSmallPic = 0
    m_eStyle = gtDefault
    m_eValue = gtUnpressed
End Sub

Private Sub Class_Terminate()
    Set m_oButtons = Nothing ' release the object
End Sub

'   ***** Returns/sets the left property. For internal use only
Friend Property Let InternalLeft(lLeft As Long)
    m_lLeft = lLeft
    m_oButtons.ChildChanged Me, "Left"
End Property
Friend Property Get InternalLeft() As Long
Attribute InternalLeft.VB_MemberFlags = "40"
    InternalLeft = m_lLeft
End Property
'   *****

'   ***** Returns/sets the top property. For internal use only
Friend Property Let InternalTop(lTop As Long)
    m_lTop = lTop
    m_oButtons.ChildChanged Me, "Top"
End Property
Friend Property Get InternalTop() As Long
Attribute InternalTop.VB_MemberFlags = "40"
    InternalTop = m_lTop
End Property
'   *****

'   ***** Returns/sets the Width property. For internal use only
Friend Property Let InternalWidth(lWidth As Long)
    m_lWidth = lWidth
    m_oButtons.ChildChanged Me, "Width"
End Property
Friend Property Get InternalWidth() As Long
Attribute InternalWidth.VB_MemberFlags = "40"
    InternalWidth = m_lWidth
End Property
'   *****

'   ***** Returns/sets the height property. For internal use only
Friend Property Let InternalHeight(lHeight As Long)
    m_lHeight = lHeight
    m_oButtons.ChildChanged Me, "Height"
End Property
Friend Property Get InternalHeight() As Long
Attribute InternalHeight.VB_MemberFlags = "40"
    InternalHeight = m_lHeight
End Property
'   *****
