VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTButtons"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  The GTButtons collection object.
'   * File:         GTButtons.cls
'   ******************************************************************************
Option Explicit
'   ***** button style constants
Public Enum GTButtonStyleConstants
    gtDefault
    gtToggle
    gtGroup
    gtGroupToggle
    gtSeparator
    gtPlaceHolder
End Enum
'   *****
'   ***** button value constants
Public Enum GTButtonValueConstants
    gtUnpressed
    gtPressed
End Enum
'   *****

Private m_colButtons As Collection ' holds a collection of GTButton objects
Private m_bSkipChildChanges As Boolean ' flag to not fire the changed event
Private m_iButtonWidth As Integer ' holds the button width
Private m_iButtonHeight As Integer ' holds the button height
Private m_eView As Integer ' holds the view (large or small images)
Private m_eButtonType As Integer ' holds the button type (images and text, images only, text only)
Private m_eAlign As AlignConstants ' holds the align status (left, top, bottom, right, none)
Private m_iLeftOffset As Integer ' holds how far from the left the first button starts
Private m_iTopOffset As Integer ' holds how far from the top the first button starts

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event ButtonAdded(Button As GTButton) ' fires when a button is added
Attribute ButtonAdded.VB_MemberFlags = "40"
Event ButtonRemoved(Button As GTButton) ' fires when a button is removed
Attribute ButtonRemoved.VB_MemberFlags = "40"
Event ButtonChanged(Button As GTButton, sProperty As String) ' fires when a button is changed
Attribute ButtonChanged.VB_MemberFlags = "40"
'   *****

'   ***** how far from the left the first button starts. Need this for determining the left of the button
Friend Property Let LeftOffset(iOffset As Integer)
    m_iLeftOffset = iOffset
End Property
'   *****

'   ***** how far from the top the first button starts. Need this for determining the top of the button
Friend Property Let TopOffset(iOffset As Integer)
    m_iTopOffset = iOffset
End Property
'   *****

'   ***** what position the toolbar is aligned (left, top, bottom, right, none). Need this for determining the position of the button
Friend Property Let Align(eAlign As AlignConstants)
    m_eAlign = eAlign
End Property
'   *****

'   ***** the width of the button. Need this for determining the position of the button
Friend Property Let ButtonWidth(ByVal iWidth As Integer)
    m_iButtonWidth = iWidth
End Property
'   *****

'   ***** returns/sets the buttonheight. Need this for determining the position of the button
Friend Property Get ButtonHeight() As Integer
    ButtonHeight = m_iButtonHeight
End Property
Friend Property Let ButtonHeight(ByVal iHeight As Integer)
    m_iButtonHeight = iHeight
End Property
'   *****

'   ***** what type the button is (images and text, images only, text only). Need this for determining the position of the button
Friend Property Let ButtonType(ByVal eType As Integer)
    m_eButtonType = eType
End Property
'   *****

'   ***** the view of the button (large or small pictures). Need this for determining the position of the button.
Friend Property Let View(ByVal eView As Integer)
    m_eView = eView
End Property
'   *****

'   ***** Returns the number of button object in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = " Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colButtons.Count
End Property
'   *****

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index) As GTButton
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)

    If lIndex < 1 Or lIndex > m_colButtons.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colButtons(lIndex)
End Property
'   *****

'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
' NewEnum must return the IUnknown interface of a
' collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colButtons.[_NewEnum]
End Function
'   *****

Public Function Add(Optional Index, Optional Key, Optional Caption, Optional Style, Optional LargePicture, _
                            Optional SmallPicture) As GTButton
Attribute Add.VB_Description = "Adds a Button object to a Buttons collection and returns a reference to the created object."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Key: Optional: the key of the new object,
'   *               Caption: Optional: the caption of the new object, Style: Optional: the Style (gtDefault, gtToggle, etc) of
'   *               the button, LargePicture: Optional: the index of key to GTListImage object in the LargeImages collection,
'   *               SmallPicture: Optional: the index of key to GTListImage object in the SmallImages collection
'   * Return:   GTButton: the object if a button was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTButton object to the Buttons collection.
'   ***********************************************************************
    Dim oButton As GTButton
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    Set oButton = New GTButton
    
    m_bSkipChildChanges = True
    
    Set oButton.Buttons = Me
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Key) Then
        oButton.Key = Key
    End If
    
    If Not IsMissing(Caption) Then
        oButton.Caption = Caption
    End If
    
    If Not IsMissing(Style) Then
        oButton.Style = Style
    End If
    
    If Not IsMissing(LargePicture) Then
        oButton.LargePicture = LargePicture
    End If
    
    If Not IsMissing(SmallPicture) Then
        oButton.SmallPicture = SmallPicture
    End If
    
    If IsMissing(Index) Then
        oButton.InternalIndex = m_colButtons.Count + 1
        m_colButtons.Add oButton
    Else
        m_colButtons.Add oButton, , , lIndex
        ReIndexObjects
    End If
    
    m_bSkipChildChanges = False
                            
    RaiseEvent ButtonAdded(oButton)
    
    Set Add = oButton
    
    Exit Function
    
Add_err:
    m_bSkipChildChanges = False
    MsgBox Err.Description, vbInformation, App.Title
End Function
Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the GTButton object to remove
'   * Purpose:  Removes a GTButton object from the Buttons collection.
'   ***********************************************************************
    Dim oDeletedButton As GTButton
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    Set oDeletedButton = Me(lIndex)
    
    m_colButtons.Remove lIndex
    
    ReIndexObjects
    
    RaiseEvent ButtonRemoved(oDeletedButton)
    
End Sub
Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the GTButton object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oButton As GTButton
    
    For Each oButton In m_colButtons
        If oButton.Key = vIndex Then
            GetObjectIndexFromKey = oButton.Index
            Exit Function
        End If
    Next
    
End Function

Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection of GTButton objects to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oButton As GTButton
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oButton In m_colButtons
        If StrComp(vTempKey, Trim(oButton.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all GTButton objects
'   ***********************************************************************
    Set m_colButtons = New Collection
    
    RaiseEvent ButtonRemoved(Nothing)
End Sub

Friend Sub ChildChanged(oButton As GTButton, sProperty As String)
'   ***********************************************************************
'   * Params:   oButton: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTButton object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If Not m_bSkipChildChanges Then
        RaiseEvent ButtonChanged(oButton, sProperty)
    End If
End Sub

Friend Sub ReadObjectProps(oPropBag As PropertyBag)
'   ***********************************************************************
'   * Params:   oPropBag: the property bag to read property values from
'   * Purpose:  Reads the properties from the property bag. Reads only
'   *               the properties for objects that were changed at design time
'   *               thru the property pages.
'   ***********************************************************************
    Dim i As Long
    Dim j As Long
    Dim lCount As Long
    Dim oButton As GTButton
    
    lCount = oPropBag.ReadProperty("NumButtons", 0)
    
    For i = 1 To lCount
        
        Set oButton = Me.Add()
        
        oButton.Tag = oPropBag.ReadProperty("ButtonTag" & i, "")
        oButton.Key = oPropBag.ReadProperty("ButtonKey" & i, "")
        oButton.Caption = oPropBag.ReadProperty("ButtonCaption" & i, "")
        oButton.Description = oPropBag.ReadProperty("ButtonDescription" & i, "")
        oButton.Enabled = oPropBag.ReadProperty("ButtonEnabled" & i, True)
        oButton.LargePicture = oPropBag.ReadProperty("ButtonLargePicture" & i, 0)
        oButton.SmallPicture = oPropBag.ReadProperty("ButtonSmallImage" & i, 0)
        oButton.Style = oPropBag.ReadProperty("ButtonStyle" & i, gtDefault)
        oButton.ToolTipText = oPropBag.ReadProperty("ButtonToolTipText" & i, "")
        oButton.Value = oPropBag.ReadProperty("ButtonValue" & i, gtUnpressed)
        oButton.Visible = oPropBag.ReadProperty("ButtonVisible" & i, True)
        oButton.Width = oPropBag.ReadProperty("ButtonWidth" & i, 0)
        
    Next
    
End Sub

Private Sub ReIndexObjects()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colButtons.Count
        Me(i).InternalIndex = i
    Next
    
End Sub

Friend Sub WriteObjectProps(oPropBag As PropertyBag)
'   ***********************************************************************
'   * Params:   oPropBag: the property bag to write property values to
'   * Purpose:  Writes the properties to the property bag. Writes only
'   *               the properties for objects that were changed at design time
'   *               thru the property pages.
'   ***********************************************************************
    Dim i As Long
    Dim j As Long
    Dim oButton As GTButton
    
    Call oPropBag.WriteProperty("NumButtons", m_colButtons.Count, 0)
    
    For i = 1 To m_colButtons.Count
        
        Set oButton = m_colButtons(i)
        
        Call oPropBag.WriteProperty("ButtonTag" & i, oButton.Tag, "")
        Call oPropBag.WriteProperty("ButtonCaption" & i, oButton.Caption, "")
        Call oPropBag.WriteProperty("ButtonDescription" & i, oButton.Description, "")
        Call oPropBag.WriteProperty("ButtonEnabled" & i, oButton.Enabled, True)
        Call oPropBag.WriteProperty("ButtonKey" & i, oButton.Key, "")
        Call oPropBag.WriteProperty("ButtonLargePicture" & i, oButton.LargePicture, 0)
        Call oPropBag.WriteProperty("ButtonSmallImage" & i, oButton.SmallPicture, 0)
        Call oPropBag.WriteProperty("ButtonStyle" & i, oButton.Style, gtDefault)
        Call oPropBag.WriteProperty("ButtonToolTipText" & i, oButton.ToolTipText, "")
        Call oPropBag.WriteProperty("ButtonValue" & i, oButton.Value, gtUnpressed)
        Call oPropBag.WriteProperty("ButtonVisible" & i, oButton.Visible, True)
        Call oPropBag.WriteProperty("ButtonWidth" & i, oButton.Width, 0)
        
    Next
    
End Sub

Private Sub Class_Initialize()
    Set m_colButtons = New Collection ' initializes the collection
End Sub

Private Sub Class_Terminate()
    Set m_colButtons = Nothing ' releases the collection and all it's objects
End Sub
Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function

Friend Function GetButtonPt(oTargetButton As GTButton) As POINTAPI
'   ***********************************************************************
'   * Params:   oTargetButton: the button to find the position of
'   * Purpose:  Loops thru the collection looking for the targetbutton. When
'   *               found the left and top are returned as a point.
'   ***********************************************************************
    Dim lLeft As Long
    Dim lTop As Long
    Dim lWidth As Long
    Dim lHeight As Long
    Dim oButton As GTButton
    Dim pt As POINTAPI
    
    lLeft = m_iLeftOffset
    lTop = m_iTopOffset
    
    For Each oButton In m_colButtons
        
        If oButton Is oTargetButton Then
            pt.x = lLeft
            pt.y = lTop
            GetButtonPt = pt
            Exit Function
        End If
        
        If oButton.Visible Then
        
'   ***** Ver 1.0a Not used
'            Select Case oButton.Style
'                Case gtDefault, gtToggle, gtGroup, gtGroupToggle
'                    lHeight = m_iButtonHeight
'                    lWidth = m_iButtonWidth
'
'                Case gtSeparator
'                    If m_eAlign = vbAlignLeft Or m_eAlign = vbAlignRight Then
'                        lHeight = BUTTON_SEPARATORWIDTH
'                        lWidth = m_iButtonWidth
'                    Else
'                        lWidth = BUTTON_SEPARATORWIDTH
'                        lHeight = m_iButtonHeight
'                    End If
'
'                Case gtPlaceHolder
'                    If m_eAlign = vbAlignLeft Or m_eAlign = vbAlignRight Then
'                        lHeight = m_iButtonHeight
'                        lWidth = m_iButtonWidth
'                    Else
'                        lWidth = oButton.Width
'                        lHeight = m_iButtonHeight
'                    End If
'
'            End Select
'   *****
            Select Case oButton.Style
                Case gtDefault, gtToggle, gtGroup, gtGroupToggle
                    If m_eAlign = vbAlignLeft Or m_eAlign = vbAlignRight Then
                        lTop = lTop + m_iButtonHeight
                    Else
                        lLeft = lLeft + m_iButtonWidth
                    End If
                    
                Case gtSeparator
                    If m_eAlign = vbAlignLeft Or m_eAlign = vbAlignRight Then
                        lTop = lTop + BUTTON_SEPARATORWIDTH
                    Else
                        lLeft = lLeft + BUTTON_SEPARATORWIDTH
                    End If
                    
                Case gtPlaceHolder
                    
                    If m_eAlign = vbAlignLeft Or m_eAlign = vbAlignRight Then
                        lTop = lTop + m_iButtonHeight
                    Else
'   ***** Ver 1.0a Needs to be Internal Width otherwise numbers are too large
                        lLeft = lLeft + oButton.InternalWidth
'                        lLeft = lLeft + oButton.Width
'   *****
                    End If
            
            End Select

        End If
        
    Next

End Function
'   ***** Ver 1.0a Added to remove all objects.
Friend Sub RemoveAll()
'   ***********************************************************************
'   * Version:  1.0a
'   * Purpose:  Releases the objects from the collection. Called from
'   *               the Terminate event. This is used instead of Clear so
'   *               ButtonRemoved event doesn't fire.
'   ***********************************************************************
    Set m_colButtons = Nothing
End Sub
'   *****

