VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GTListImages"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'   ******************************************************************************
'   * Purpose:  The GTListImages collection object.
'   * File:         ListImages.cls
'   ******************************************************************************
Option Explicit
'   ***** drawpicture constants
Public Enum GTDrawPictureConstants
    gtNormal
    gtTransparent
    gtDitheredBackground
    gtDisabled
End Enum
'   *****

Private m_CTransBlt As CTransBlt ' the drawing class
Private m_colImages As Collection ' holds the GTListImage objects
Private m_bSkipChildChanges As Boolean ' flag to not fire the changed event
Private m_lWidth As Long ' width of each image
Private m_lHeight As Long ' height of each image
Private m_iScalePerPixel As Integer ' the scale mode of the container based on 1 pixel
Private m_lMaskColor As OLE_COLOR ' the mask color for the collection
Private m_hDCDest As Long ' the dc of the user control
Private m_bUseMaskColor As Boolean ' flag to use the mask color
Private m_iHiMetricPerPixel As Integer ' himetric units per pixel
Private m_iTopOffset As Integer

'   ***** events are fired in the control so the control knows that something changed in the collection or object
Event ImageAdded(oListImage As GTListImage)
Attribute ImageAdded.VB_MemberFlags = "40"
Event ImageRemoved(oListImage As GTListImage)
Attribute ImageRemoved.VB_MemberFlags = "40"
Event ImageChanged(oListImage As GTListImage, sProperty As String)
Attribute ImageChanged.VB_MemberFlags = "40"
'   *****

Friend Property Let TopOffset(iOffset As Integer)
    m_iTopOffset = iOffset
End Property
Friend Property Get TopOffset() As Integer
    TopOffset = m_iTopOffset
End Property
'   ***** sets the number of himetric units in each pixel. Can't get this value in the class so it has to be passed
'   ***** Internal use only
Friend Property Let HiMetricPerPixel(iScale As Integer)
    m_iHiMetricPerPixel = iScale
End Property
'   *****

'   ***** Returns/sets the value of UseMaskColor. The property is read and written from here
'   ***** Internal use only
Friend Property Let UseMaskColor(bUseMaskColor As Boolean)
    m_bUseMaskColor = bUseMaskColor
End Property
Friend Property Get UseMaskColor() As Boolean
    UseMaskColor = m_bUseMaskColor
End Property
'   *****

'   ***** Returns the number of objects in the collection
Public Property Get Count() As Long
Attribute Count.VB_Description = "Returns the number of objects in a collection."
Attribute Count.VB_HelpID = 24
    Count = m_colImages.Count
End Property
'   *****

'   ***** the hdc of the usercontrol. Only used when creating the mask pictures, not for drawing
'   ***** Internal Use Only
Friend Property Let DestinationDC(hdc As Long)
    m_hDCDest = hdc
End Property
Friend Property Get DestinationDC() As Long
    DestinationDC = m_hDCDest
End Property
'   *****

'   ***** Returns/sets the MaskColor. The property is read and written from here
'   ***** Internal Use Only
Friend Property Let MaskColor(lColor As OLE_COLOR)
    m_lMaskColor = lColor
    CreateAllMaskPictures ' recreates all the mask pictures when the maskcolor is reset
End Property
Friend Property Get MaskColor() As OLE_COLOR
    MaskColor = m_lMaskColor
End Property
'   *****

'   ***** the number of scale units per pixel. Can't get this value in the class, so it needs to passed in
Friend Property Let ScalePerPixel(iScale As Integer)
    m_iScalePerPixel = iScale
End Property
Friend Property Get ScalePerPixel() As Integer
    ScalePerPixel = m_iScalePerPixel
End Property
'   *****

'   ***** Returns/sets the width that the pictures will be drawn by default. This property is set either by the user before
'   ***** any images are added to the collection or by the width of the first image. ReadOnly when Images
'   ***** present
Public Property Let Width(lWidth As Long)
    m_lWidth = lWidth
End Property
Public Property Get Width() As Long
    Width = m_lWidth
End Property
'   *****

'   ***** Returns/sets the Height that the pictures will be drawn by default. This property is set either by the user before
'   ***** any images are added to the collection or by the Height of the first image. ReadOnly when Images
'   ***** present
Public Property Let Height(lHeight As Long)
    m_lHeight = lHeight
End Property
Public Property Get Height() As Long
    Height = m_lHeight
End Property
'   *****

'   ***** Returns the IUnnown interface. Used to allow For Each loops when using a collection object in the user control
' NewEnum must return the IUnknown interface of a
' collection's enumerator.
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = m_colImages.[_NewEnum]
End Function
'   *****

'   ***** Returns the Item in the collection. This is the default property.
Public Property Get Item(Index) As GTListImage
Attribute Item.VB_Description = "Returns a specific member of a Collection object either by position or by key."
Attribute Item.VB_HelpID = 20
Attribute Item.VB_UserMemId = 0
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)

    If lIndex < 1 Or lIndex > m_colImages.Count Then
        Err.Raise vbObjectError + 1053, , "Index out of bounds."
        Exit Property
    End If
    
    Set Item = m_colImages(lIndex)
End Property
'   *****

Public Function Add(Optional Index As Variant, Optional Key As Variant, Optional Tag As Variant, _
                                Optional Picture As Variant) As GTListImage
Attribute Add.VB_Description = "Adds a ListImage object to a ListImages collection and returns a reference to the created object."
'   ***********************************************************************
'   * Params:   Index: Optional: the index to insert the new object after, Key: Optional: the key of the new object,
'   *               Tag: Optional: the tag of the new object, Picture: Optional: a stdPicture object,
'   * Return:   GTListImage: the object if a listimage was created, Nothing if an error occurred.
'   * Purpose:  Adds a GTListImage object to the ListImages collection.
'   ***********************************************************************
    Dim oListImage As GTListImage
    Dim lIndex As Long
    
'    On Error GoTo Add_err
    
    Set oListImage = New GTListImage
    
    m_bSkipChildChanges = True
        
    Set oListImage.ListImages = Me
    
    If Not IsMissing(Index) Then
        lIndex = ConvertIndex(Index)
    End If
    
    If Not IsMissing(Tag) Then
        oListImage.Tag = Tag
    End If
    
    If Not IsMissing(Key) Then
        oListImage.Key = Key
    End If
    
    If Not IsMissing(Picture) Then
        Set oListImage.Picture = Picture
        If m_colImages.Count = 0 Then
            If m_lWidth = 0 And m_lHeight = 0 Then
                m_lWidth = Picture.Width / m_iHiMetricPerPixel
                m_lHeight = Picture.Height / m_iHiMetricPerPixel
            End If
        End If
    End If
    
    If IsMissing(Index) Then
        oListImage.InternalIndex = m_colImages.Count + 1
        m_colImages.Add oListImage
    Else
        m_colImages.Add oListImage, , , lIndex
        ReIndexObjects
    End If
    
    m_bSkipChildChanges = False
    
    RaiseEvent ImageAdded(oListImage)
    
    Set Add = oListImage
    
    Exit Function
    
Add_err:
    m_bSkipChildChanges = False
    MsgBox Err.Description, vbInformation, App.Title
End Function
Public Sub Remove(Index As Variant)
Attribute Remove.VB_Description = "Removes a specific member from a collection."
'   ***********************************************************************
'   * Params:   Index: the index or key of the object to remove
'   * Purpose:  Removes an object from the collection.
'   ***********************************************************************
    Dim oDeletedListImage As GTListImage
    Dim lIndex As Long
    
    lIndex = ConvertIndex(Index)
    
    Set oDeletedListImage = m_colImages(lIndex)
    
    m_colImages.Remove lIndex
    
    If m_colImages.Count = 0 Then
        ResetDimensions
    Else
        ReIndexObjects
    End If
    
    RaiseEvent ImageRemoved(oDeletedListImage)
    
End Sub
Public Sub Clear()
Attribute Clear.VB_Description = "Removes all objects in a collection."
'   ***********************************************************************
'   * Purpose:  Reinitializes the collection, removing all objects
'   ***********************************************************************
    Set m_colImages = New Collection
    
    ResetDimensions
    
    RaiseEvent ImageRemoved(Nothing)
End Sub

Private Sub Class_Initialize()
'   ***********************************************************************
'   * Purpose:  Initializes the classes and properties
'   ***********************************************************************
    Set m_CTransBlt = New CTransBlt
    Set m_colImages = New Collection
    m_bUseMaskColor = True
End Sub

Private Sub Class_Terminate()
    Set m_CTransBlt = Nothing ' release the object
    Set m_colImages = Nothing ' release the object
End Sub

Friend Sub ChildChanged(oListImage As GTListImage, sProperty As String)
'   ***********************************************************************
'   * Params:   oListImage: the object that was changed, sProperty: the name of the
'   *               property that was changed.
'   * Purpose:  Called when a property in a GTListImage object is changed. This is
'   *               necessary because there is no notification that a property in the
'   *               object was changed.
'   ***********************************************************************
    If Not m_bSkipChildChanges Then
        RaiseEvent ImageChanged(oListImage, sProperty)
    End If
End Sub

Private Sub ReIndexObjects()
'   ***********************************************************************
'   * Purpose:  Resets the value of the Index property for each object. This
'   *               is done after a remove or if an object was inserted in the collection
'   *               rather than added to the end.
'   ***********************************************************************
    Dim i As Long
    
    For i = 1 To m_colImages.Count
        Me(i).InternalIndex = i
    Next
    
End Sub

Private Function GetObjectIndexFromKey(vIndex As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the index or key of the object get the index for
'   * Return:   Long: the index of the object, 0 if not found
'   * Purpose:  Tries to match a key to an object and return the index property.
'   ***********************************************************************
    Dim oListImage As GTListImage
    
    For Each oListImage In m_colImages
        If oListImage.Key = vIndex Then
            GetObjectIndexFromKey = oListImage.Index
            Exit Function
        End If
    Next
    
End Function

Friend Function IsDuplicateKey(vKey As Variant) As Boolean
'   ***********************************************************************
'   * Params:   vKey: the key to look for
'   * Return:   Boolean: True if the key was found, False if not
'   * Purpose:  Loops thru the collection to see if the key
'   *               already exists. Returns the findings.
'   ***********************************************************************
    Dim oListImage As GTListImage
    Dim vTempKey As Variant
    
    vTempKey = Trim(vKey)
    
    If vTempKey = "" Then
        IsDuplicateKey = False
        Exit Function
    End If
    
    For Each oListImage In m_colImages
        If StrComp(vTempKey, Trim(oListImage.Key), vbTextCompare) = 0 Then
            IsDuplicateKey = True
            Exit Function
        End If
    Next
    
    IsDuplicateKey = False
End Function

Friend Function CreateMaskPicture(hBmpSource As Long) As Long
'   ***********************************************************************
'   * Params:   hBmpSource: the handle to the source bitmap
'   * Return:   Long: the handle to the mask bitmap
'   * Purpose:  Calls the CreateMask proc in CTransBlt to create the mask of the
'   *               source and returns the mask picture's handle
'   ***********************************************************************
    Dim hBmpMask As Long
    
    CreateMaskPicture = m_CTransBlt.CreateMask(m_hDCDest, hBmpSource, m_lMaskColor)
    
End Function
Friend Sub CreateAllMaskPictures()
'   ***********************************************************************
'   * Purpose:  Loops thru the collection and calls createmask for object.
'   *               Called when the MaskColor has changed.
'   ***********************************************************************
    Dim oListImage As GTListImage
    Dim hBmpMask As Long
    
    For Each oListImage In m_colImages
        hBmpMask = CreateMaskPicture(oListImage.Picture.Handle)
        oListImage.MaskPicture = hBmpMask
    Next
    
End Sub

Friend Sub DrawPicture(hDCDest As Long, X As Long, Y As Long, hBmpSource As Long, hBmpMask As Long, lWidth As Long, lHeight As Long, _
                                    Optional eStyle As GTDrawPictureConstants = gtNormal, Optional vBackColor As Variant)
'   **********************************************************************
'   * Params:   hDCDest: the dc to draw on, X: the left point, Y: the top point, Width: Optional: the width to draw
'   *               the picture, Height: Optional: the height to draw the picture, Style: Optional: a GTDrawPictureConstant,
'   *               BackColor: Optional: the backcolor for the dc
'   * Purpose:  Draws the image on the dc based on the Style param
'   **********************************************************************
    Dim lPicWidth As Long
    Dim lPicHeight As Long
    
    lPicWidth = lWidth
    lPicHeight = lHeight
    
    If lPicWidth = 0 Then
        lPicWidth = m_lWidth
    End If
    
    If lPicHeight = 0 Then
        lPicHeight = m_lHeight
    End If
    
    If eStyle = gtTransparent And m_bUseMaskColor Then
        m_CTransBlt.DrawTransparentPicture hDCDest, hBmpSource, hBmpMask, X, Y, lPicWidth, lPicHeight, vBackColor
    Else
        
        Select Case eStyle
            Case gtDitheredBackground
                m_CTransBlt.DrawDitheredBackground hDCDest, hBmpSource, hBmpMask, X, Y, lPicWidth, lPicHeight, vBackColor
            
            Case gtDisabled
                m_CTransBlt.DrawDisabledPicture hDCDest, hBmpSource, hBmpMask, X, Y, lPicWidth, lPicHeight, vBackColor

            Case Else
                m_CTransBlt.DrawNormalPicture hDCDest, hBmpSource, hBmpMask, X, Y, lPicWidth, lPicHeight
            
        End Select
        
    End If
    
End Sub

Friend Sub ReadObjectProps(oPropBag As PropertyBag, sName As String)
'   ***********************************************************************
'   * Params:   oPropBag: the property bag to read property values from
'   * Purpose:  Reads the properties from the property bag. Reads only
'   *               the properties for objects that were changed at design time
'   *               thru the property pages.
'   ***********************************************************************
    Dim i As Long
    Dim j As Long
    Dim lCount As Long
    Dim oListImage As GTListImage
    
    lCount = oPropBag.ReadProperty("Num" & sName, 0)
    
    For i = 1 To lCount
        
        Set oListImage = Me.Add()
        
        Set oListImage.Picture = oPropBag.ReadProperty(sName & "Picture" & i, Nothing)
        oListImage.Tag = oPropBag.ReadProperty(sName & "Tag" & i, "")
        oListImage.Key = oPropBag.ReadProperty(sName & "Key" & i, "")
    
    Next
    
End Sub

Friend Sub WriteObjectProps(oPropBag As PropertyBag, sName As String)
'   ***********************************************************************
'   * Params:   oPropBag: the property bag to write property values to
'   * Purpose:  Writes the properties to the property bag. Writes only
'   *               the properties for objects that were changed at design time
'   *               thru the property pages.
'   ***********************************************************************
    Dim i As Long
    Dim j As Long
    Dim oListImage As GTListImage
    
    Call oPropBag.WriteProperty("Num" & sName, m_colImages.Count, 0)
    
    For i = 1 To m_colImages.Count
        
        Set oListImage = m_colImages(i)
        
        Call oPropBag.WriteProperty(sName & "Picture" & i, oListImage.Picture, Nothing)
        Call oPropBag.WriteProperty(sName & "Tag" & i, oListImage.Tag, "")
        Call oPropBag.WriteProperty(sName & "Key" & i, oListImage.Key, "")
        
    Next
    
End Sub

Private Sub ResetDimensions()
'   ***********************************************************************
'   * Purpose:  Resets the dimensions of the image when the Clear method
'   *               is called.
'   ***********************************************************************
    m_lWidth = 0
    m_lHeight = 0
End Sub

Private Function ConvertIndex(Index As Variant) As Long
'   ***********************************************************************
'   * Params:   Index: the value of an Index or key
'   * Return:   Long: the index of the object
'   * Purpose:  Converts the param to a long. Also, if the param is a key,
'   *               the GetOjbectFromKey proc will be called to get the index.
'   ***********************************************************************
    Dim lIndex As Long
    
    If IsNumeric(Index) Then
        ConvertIndex = CLng(Index)
    Else
        ConvertIndex = GetObjectIndexFromKey(Index)
    End If
    
End Function

